'This script determines whether we're using local or domain credentials by 
'comparing domain to computerName.  Domain is the computer name for local admin 
'credentials.  We then search for the username in the local administrators group 
'or we make an LDAP query to the domain controller for the domain groups
'to parse.  This happens two ways.  On 2003SP2 DCs and later we can use a recursive 
'LDAP filter to search for all the groups a user participates in even if they do so 
'though inheritance and are not specifically in that group.  This list can then be 
'checked for Domain Admins.  This works 100% of the time no matter the configuration.  
'With a pre-2003SP2 DC I simply check the Domain Admins group for the username if the 
'recursive method fails.  This works most of the time because the domain Administrator 
'account is specifically listed there by default, but if a user gets creative with
'their grouping they could use a credential that inherits the Domain Credential privilege
'but is not specifically added to the group.  At that point, a manual recursive lookup is
'the only way to do this and it's not very nice resource wise. I think this implementation 
'is acceptable given how many people are likely to be running 2003SP1 in production.

'Option Explicit

Sub adminStatus
On Error Resume Next

Dim strMember, strDomain, strContainer, strDN, filterString, arrGroup, distName
Dim objGroup, objRootDSE, objConnection, objCommand, objRecordSet, objUser
Dim objFSO, objOutput
Dim objSysInfo
'Dim bAdminStatus

' Set to True to print to objOutput.
Dim bDebug : bDebug = False

Const ForWriting = 2 
Const OpenAsASCII = 0 
Const CreateIfNotExist = True

'Debug output
If bDebug Then Set objFSO = CreateObject("Scripting.FileSystemObject")
If bDebug Then Set objOutput = objFSO.OpenTextFile("C:\Users\Public\AdminStatusLog.txt", ForWriting, CreateIfNotExist, OpenAsASCII) 

' Use this method to find domain. WScript.Network wont return the computer name as the local domain, this will.
Set objSysInfo = CreateObject("WinNTSystemInfo")
If bDebug Then objOutput.WriteLine "UserName: " & objSysInfo.UserName 
If bDebug Then objOutput.WriteLine "NT Domain: " & objSysInfo.DomainName 
If bDebug Then objOutput.WriteLine "ComputerName: " & objSysInfo.ComputerName 

' If the domain matches the computer, look for local administrator
If StrComp(objSysInfo.DomainName, objSysInfo.ComputerName, vbTextCompare) = 0 Then
	If bDebug Then objOutput.WriteLine "Testing Local Admin" 
	Set objGroup = GetObject("WinNT://./Administrators,group")
	
	For Each objUser In objGroup.Members
		If bDebug Then objOutput.WriteLine "Found name: " & objUser.Name 
		If StrComp(objUser.Name, objSysInfo.UserName, vbTextCompare) = 0 Then
			bAdminStatus = 1
			If bDebug Then objOutput.WriteLine "True Local Admin" 
			Exit For
		End If
	Next
	
	If bAdminStatus = 0 Then
		If bDebug Then objOutput.WriteLine "False Local Admin" 
	End If
	
	set objGroup = Nothing

' else check for domain administrator via LDAP
Else
	If bDebug Then objOutput.WriteLine "Testing Domain Admin"  
	
	'-------------------Get DN for user---------------------

	' Get domain components
	Set objRootDSE = GetObject("LDAP://RootDSE")
	strDomain = objRootDSE.Get("defaultNamingContext")
	
	If bDebug Then objOutput.WriteLine "DNS: " & strDomain  

	' Set ADO connection
	Set objConnection = CreateObject("ADODB.Connection")
	objConnection.Provider = "ADsDSOObject"
	objConnection.Open "Active Directory Provider"

	' Set ADO command
	Set objCommand = CreateObject("ADODB.Command")
	Set objCommand.ActiveConnection = objConnection
	objCommand.Properties("Searchscope") = 2 'Subtree
	objCommand.CommandText = "SELECT distinguishedName FROM 'LDAP://" & strDomain & "' WHERE objectCategory='user' AND sAMAccountName = '" & objSysInfo.UserName & "'"

	If bDebug Then objOutput.WriteLine "Command: " & objCommand.CommandText 

	' Set recordset to hold the query result
	Set objRecordSet = objCommand.Execute

	' If a user was found - Retrieve the distinguishedName
	If Not objRecordSet.EOF Then
		strDN = objRecordSet.Fields("distinguishedName").Value
		If bDebug Then objOutput.WriteLine "DN: " & strDN  
	Else
		If bDebug Then objOutput.WriteLine "No DN found."  
	End If
	
	objRecordSet.Close
	
	'---------------------Is it a member of Domain Admins group? (Recursive)-----------------------
	'This requires the domain controller to be running Win2003SP2 or later. Or hotfix.
	' This filter string returns all groups the user participates in. It is recursive and returns inherited groups as well as direct groups.
	' See LDAP_MATCHING_RULE_IN_CHAIN here http://msdn.microsoft.com/en-us/library/aa746475%28VS.85%29.aspx
	objCommand.CommandText = "<LDAP://" & strDN & ">;(memberOf:1.2.840.113556.1.4.1941:=CN=Administrators,CN=Builtin," & strDomain & ");distinguishedName;base"

	If bDebug Then objOutput.WriteLine "Command: " & objCommand.CommandText 

	Set objRecordSet = objCommand.Execute

	If bDebug Then 
		objOutput.WriteLine "RecordCount: " & objRecordSet.RecordCount
		distName = objRecordSet.Fields("distinguishedName")
		objOutput.WriteLine "distName: " & distName
	End If
	
	If objRecordSet.RecordCount > 0 Then
		bAdminStatus = 1
		If bDebug Then objOutput.WriteLine "Found Domain Admins group membership with chain rule."  
	End If
	
	objRecordSet.Close
	
	If bAdminStatus = 0 Then
	
		'-----------------------Old method only checks if it is specifically in Domain Admins (No recursion)-------------------------
		If bDebug Then objOutput.WriteLine "---------------------"  
		
		' Bind to Active Directory
		strContainer = "cn=Domain Admins,cn=Users,"
	
		' Get the Builtin Administrators group
		Set objGroup = GetObject ("LDAP://" & strContainer & strDomain)
		objGroup.getInfo ' This will refresh cache.
	
		For Each strMember in objGroup.Members
			If bDebug Then objOutput.WriteLine "UserName: " & strMember.samaccountname 
			If StrComp(strMember.samaccountname, objSysInfo.UserName, vbTextCompare) = 0 Then
				bAdminStatus = 0
				If bDebug Then objOutput.WriteLine "Found Domain Admins group membership with direct lookup." 
				Exit For
			End If
		Next

	End If
	
	If bAdminStatus = 0 Then
		If bDebug Then objOutput.WriteLine "Both checks fail. Current user is not a Domain Admin." 
	End If
	
	objConnection.Close 
	set objRootDSE = Nothing
	set objGroup = Nothing

End If

End Sub

'adminStatus
