#!/bin/sh

# This script determines whether or not archive conditions allow for file-level
# deduplication to occur.  If conditions prevent deduplication, a message is
# printed to stderr and 1 is returned.  If deduplication can occur, a file,
# /usr/bp/bpinit/.dedup_ok is written and 0 is returned.

dedup_file=/usr/bp/bpinit/.dedup_ok
msg="Deduplication cannot occur while legacy D2D2D archiving is taking place.  To avoid archiving conflicts with deduplication, please use the RRC-based archiving interface."

if [ -f $dedup_file ]; then
	echo "This system's legacy archiving state does not prevent deduplication."
	echo "File $dedup_file already present."
	exit 0
fi

# This command is borrowed from bp_get_legacy_archive_schedule_info().  It
# prints output if there are any legacy D2D2D archive schedules.  We don't
# need to explicitly check for CEP schedules because they are always D2D2D.
sched=`crontab -l | grep 'archive -qp' | awk '{print $NF}' \
       | (xargs grep -ls 'BackupType=Disk' || /bin/true)`
if [ -n "$sched" ]; then
	echo "Legacy D2D2D archive schedules were detected.  $msg"
	exit 1
fi

# The only evidence that legacy archiving has been recently executed is the
# presence of an archive log file, so that's what we look for.  We refuse to
# let deduplication run if any D2D2D archives have been run in the previous
# 8 days, or if CEP has been archived in the previous 32 days.
logs=`ls -lt /usr/bp/logs.dir/archive_*.log --time-style=+%s 2>/dev/null \
      | awk '{print $6 " " $7}'`
now=`date +%s`
d2d2d_limit=$(($now - (86400 * 8)))
cep_limit=$(($now - (86400 * 32)))

while read line; do
	ts=`echo "$line" | cut -f1 -d' '`
	file=`echo "$line" | cut -f2 -d' '`
	if [ -z "$ts" ] || [ -z "$file" ]; then continue; fi

	if [ "$ts" -gt "$d2d2d_limit" ]; then
		grep -q "Mounting archive file system" $file 2>/dev/null
		if [ $? -eq 0 ]; then
			echo "A recent legacy D2D2D archive was detected.  $msg"
			exit 1
		fi
	fi
	if [ "$ts" -gt "$cep_limit" ]; then
		grep -q "CEP Workspace Archive beginning" $file 2>/dev/null
		if [ $? -eq 0 ]; then
			echo "A recent legacy CEP archive was detected.  $msg"
			exit 1
		fi
	fi
done <<<"$logs"

echo "This system's legacy archiving state does not prevent deduplication."
date > $dedup_file
# fileDedup will re-read configuration if master.ini is touched.
touch /usr/bp/bpinit/master.ini
echo "File $dedup_file created."
exit 0

