#!/bin/bash
# This script creates the PostgreSQL database (bpdb) from scratch or updates it.
#
# It is called from the postinstall when installing the rpm.  When the rpm is
# being installed for the first time (a fresh install), this script is invoked
# with the "create" argument; when the rpm is being upgraded, it is called with
# the "update" argument.  All output goes to the install log.
#
# This script may also be called "standalone" from the command line to allow
# the create or update operations to be run at any time.  If run from the command
# line, the output goes to $BPDIR/logs.dir/setup_postgresql.log.
#
#
# Change log entries
# Bug# Date    Who Description
# ---- ------- --- -----------------------------------------------------------------
# 2691 27Sep06 LCG Initial implementation
# 2428 06Nov06 LCG Autovacuum config requires stats_start_collector, stats_row_level on 
# 3025 08Mar07 LCG Incorporate 3.0 schema
# 3135 21Mar07 LCG Create the DB in /backups/UnitrendsDataBase
# 3265 22May07 LCG Exit if adding default customer/location/system/user fails
# 3006 11Jun07 GDP Create default customer/location/system and user if not present
# 3557 09Jul07 LCG Update postgresql.conf with our tuned version
# 3366 11Jul07 LCG Force restart of server using fast mode if smart restart fails
# 3687 13Sep07 LCG Give DB reload/restart more time to complete before reporting error
# 3944 07Dec07 GDP SSL and connection service file setup for remote db connections
# 3586 17Jan08 LCG Run initdb with C locale and SQL_ASCII encoding
# 6794 07Jan09 LCG Support for new "role" column in systems table
# 18150 16Jan14 VSK Connection pooling: Implementation of pgbouncer
# 18866 03Mar14 VSK Installation should preserve intended postgres configuration
# 19121 03Mar14 VSK For CentOS5, do not log any pgbouncer messages in install-rr

Usage()
{
echo "
Usage: $0 [create|update] <logfile>
	where logfile is optional.
"
exit 1
}


# Restart the database if needed as part of the setup_postgresql.sh script
restart_db()
{
	echo "WARNING: Restarting the Postgresql database..." >>$LOG
	touch $BPDIR/lists.dir/.disable_rrc 
	$BPDIR/bin/stop_db.sh >>$LOG 2>&1
	$BPDIR/bin/start_db.sh >>$LOG 2>&1

	# Find the latest Postgresql log
	postgres_log=`ls -t $BPDIR/data/pg_log|awk '{print $1}'|head -1`
	# Find the current size of the file to diff messages later on

	# Check the status of Postgres
	sudo -iu postgres pg_ctl status -D $BPDIR/data >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ];
	then
		echo"ERROR: Unable to start the Postgresql database..." >>$LOG
		finish_and_exit $ret
	fi
	
	wait_time=0
	ready_found=0
	recovery_found=0
	recovery_msg=0
	total_wait_time=60
	postgres_log_ln=0
	current_postgres_log_ln=0

	# Wait no more than 5 minutes or 300 seconds for the datbase to be ready
	psql -U postgres bpdb -c"select 1 from bp.backups" > /dev/null 2>&1
        ready_found=$?
        if [[ $ready_found != 0 ]]; then
		current_postgres_log_ln=`wc -l $BPDIR/data/pg_log/$postgres_log|awk '{print $1}'`
		while (( $wait_time < $total_wait_time ))
		do
			postgres_log_ln=`wc -l $BPDIR/data/pg_log/$postgres_log|awk '{print $1}'`
			(( postgres_log_ln_diff=current_postgres_log_ln-postgres_log_ln ))
			# Look for certain keywords if found exit
			tail_lines=`echo "-"$postgres_log_ln_diff`
			psql -U postgres bpdb -c"select 1 from bp.backups" > /dev/null 2>&1
        		ready_found=$?
			# if we haven't found the "is ready" message keep waiting
        		if [[ $ready_found == 0 ]]; then
				break;
        		fi
			# Look for recovery mode and just display a message once
        		tail $tail_lines $BPDIR/data/pg_log/$postgres_log | grep -q "recovery in progress"
        		recovery_found=$?
			if [[ $recovery_found == 0 ]] && (( $recovery_msg == 0 )); then
				echo "NOTICE: Database is starting up.  Please stand-by."
				echo "NOTICE: Database is starting up.  Please stand-by." >> $LOG
				recovery_msg=1
				total_wait_time=300
			fi
			sleep 20
			(( wait_time=wait_time+20 ))
		done
	fi
	# if no message is found, postgres could be in recovery pending for a long time and we need to stop
	psql -U postgres bpdb -c"select 1 from bp.backups" > /dev/null 2>&1
        ready_found=$?
	if [[ $ready_found != 0 ]]; then
        	ret=1
               	echo "ERROR: Please verify the $BPDIR/data/pg_log/$postgres_log log. Database seems to be down, aborting..."
               	echo "ERROR: Please verify the $BPDIR/data/pg_log/$postgres_log log. Database seems to be down, aborting..." >> $LOG
               	finish_and_exit $ret
	fi

	$BPDIR/bin/dispatch start >>$LOG 2>&1
	rm -f $BPDIR/lists.dir/.disable_rrc 
}

check_timezone()
{
   clock=/etc/sysconfig/clock
     # ZONE="America/New_York"
   pgconf=/tmp/postgresql.conf-copy
   pgschema=/usr/bp/schema/postgresql.conf
     # timezone = 'America/New_York'   #!1422655563!
     # log_timezone = 'America/New_York'
   phpini=/etc/php.ini
     # date.timezone = "America/New_York"

   change=0
   grep ZONE $clock |grep -q ' '
   if [ $? -eq 0 ]; then
      echo "Fixing space in ZONE" >>$LOG
      sed -i -e 's/ /_/'g $clock
      change=1
   fi
   . $clock
   echo "ZONE=$ZONE" >>$LOG
   grep -q "^date.timezone" $phpini
   if [ $? -ne 0 ]; then
      echo "Fixing php.ini timezone" >>$LOG
      sed -i -e "/date.timezone/adate.timezone = \"${ZONE}\"/" $phpini
      change=1
   fi

   # /usr/bp/data/postgresql.conf cannot be changed yet
   if [ -f $pgschema ] && [ $change -eq 1 ]; then
      cp -f $pgschema $pgconf
      sed -i -e "/^timezone/ctimezone = \'$ZONE\'" $pgconf >>$LOG 2>&1
      sed -i -e "/^log_timezone/clog_timezone = \'$ZONE\'" $pgconf >>$LOG 2>&1
      diff $pgconf $pgschema  >>$LOG 2>&1
      if [ $? -ne 0 ]; then
         cp -f $pgconf $pgschema
      fi
      rm -f $pgconf
   fi
}

if [ "$#" -eq 0 ]
then
	Usage
fi

# Global variables
if [ -z "$BPDIR" ]
then
	if [ -f /etc/default/bp.ini ]
	then
		eval `grep BPDIR /etc/default/bp.ini`
	else
		echo "BPDIR variable must be set and exported."
		exit 1
	fi
fi
BPBIN=$BPDIR/bin
BPSCHEMA=$BPDIR/schema
BPLOGDIR=$BPDIR/logs.dir

scenario=`echo $1 | tr [:lower:] [:upper:]`
if [ "$scenario" != "CREATE" -a "$scenario" != "UPDATE" ]
then
	Usage
fi

if [ "$2" ]
then
	LOG="$2"
else
	LOG=$BPLOGDIR/setup_postgresql.log
fi

# The log must be writeable by all, since some commands are executed by postgres.
if [ ! -f $LOG ]
then
	touch $LOG 2>/dev/null 1>&2
fi
chmod a+w $LOG 2>/dev/null 1>&2

function finish_and_exit() {
if [ $1 -ne 0 ]
then
	echo "$0:"
	echo "Errors occurred during $scenario of PostgreSQL database!" | tee -a $LOG
	echo "See $LOG for details."
else
	echo "$scenario operations for PostgreSQL database bpdb were successful." >>$LOG
fi
echo "Completed $0 $scenario on `date`." >>$LOG

# Put perms back to 644 on the log when done.
rm -f $PG_TUNE
chmod 0644 $LOG 2>/dev/null 1>&2

exit $1
}

trap " echo $0 interrupted! | tee -a $LOG ; finish_and_exit 1 " 1 2 3 15

## 18866       venkat                03/05/2014
function get_pgContext() {

  local funcname=${FUNCNAME-"get_pgContext"}

  ## tunables currently active in pg_settings
  TMP_PGTUNE=/tmp/.pg_tune

  ## tunables saved during sys_tune
  PG_TUNE=/usr/bp/etc/.pg_tune

  ## current conf file
  PG_CONF=/usr/bp/data/postgresql.conf

  ## http://www.postgresql.org/docs/current/static/view-pg-settings.html
  ## +-------------+---------------------+
  ## |  context    |  requires_restart   |
  ## +-------------+---------------------+
  ## |  internal   |  unchangeable (n/a) |
  ## |  postmaster |  yes                |
  ## |  sighup     |  no                 |
  ## |  backend    |  no                 |
  ## |  superuser  |  yes (session level)|
  ## |  user       |  no (session level) |
  ## +-------------+---------------------+
  context="reload"

  ## separate restart contexts with |
  ## eg: "postmaster|internal|"
  restart="postmaster|superuser"

  ## user/db/command
  dbname="postgres"
  dbuser="postgres"
  PGCMD="psql $dbname $dbuser -F= -XqltAc "

  [ ! -f $PG_TUNE ] && echo $context && return

  ## make sure we don't have a previously saved pg_settings config
  [ -f $TMP_PGTUNE ] && rm -f $TMP_PGTUNE && touch $TMP_PGTUNE

  ## get the parameters we want checked/tuned/
  pg_params=`awk -F"=" '{printf "\""$1"\",\n"}' ${PG_TUNE}`
  pg_params=`echo "${pg_params%%,}"`
  pg_params=`echo "$pg_params"|sed "s/\"/'/g"`
  $PGCMD "select name,setting,context from pg_settings where name in ($pg_params);" >> $TMP_PGTUNE

  ## if sys_tune differs with the existing tunables, find config parameter 
  ## that changed and if the context requires a restart or reload
  pg_params=`cat $TMP_PGTUNE|awk -F"=" '{printf "-e %s ",$1}'`
  pg_tmp_params_values=(`cat $PG_CONF|grep -w $pg_params|cut -f1 -d'#'|awk -F"=" '{print $2}'`)
  pg_params=`cat $TMP_PGTUNE|awk -F"=" '{print $1}'`
  pg_params_values=(`awk 'BEGIN {FS="="}; {print $2}' $PG_TUNE`)
  diff -EBbwq <(echo ${pg_params_values}) <(echo ${pg_tmp_params_values}) > /dev/null 2>&1
  if [ $? -eq 1 ]; then
    for item in ${pg_params[@]}; do
      # check if value has changed
      rvalue=`egrep -w $item $PG_TUNE | cut -f2 -d'=' | cut -f1 -d'#'` ## rvalue => value we want to set
      rvalue=${rvalue// /} 
      lvalue=`cut -f1 -d'#' $PG_CONF|egrep -w $item | cut -f2 -d'=' | cut -f1 -d'#'` ## lvalue => value existing in current settings
      lvalue=${lvalue// /} 
      [ "${rvalue}" == "${lvalue}" ] && continue ## nothing to do if values are same

      egrep -w "$item" $TMP_PGTUNE | egrep "$restart" > /dev/null
      if [ $? -eq 0 ]; then
        ## we have a parameter that changed and requires a restart.
        echo `date +%F.%H:%M:%S`" [$funcname] : $item=$rvalue change detected from sys_tune. Context requires db restart" >> $LOG
        context="restart"
        break
      else
        ## we have a parameter that changed and does NOT require a restart.
        echo `date +%F.%H:%M:%S`" [$funcname] : $item=$rvalue change detected from sys_tune. Context requires db reload" >> $LOG
        context="reload"
      fi
    done
  fi

  echo $context

}

function update_conf_file() {

    local funcname=${FUNCNAME-"update_conf_file"}

    # Account for the fact that if the /usr/bp/data/postgresql.conf file doesn't exists
    # we will use the default values and that is not good.  Run sys_tune
    if [ ! -e $BPDIR/data/postgresql.conf ]; then
	cp -f $BPSCHEMA/postgresql.conf $BPDIR/data/postgresql.conf
	chown postgres:postgres $BPDIR/data/postgresql.conf
	chmod 0600 $BPDIR/data/postgresql.conf
    fi 
    /usr/bp/bin/sys_tune >> $LOG 2>&1

    if [ ! -z "$1" ]; then
      ## our config and the one in data dir differ 
      ## this happens only  
      ##  a) if user/support modified config in data dir
      ##  b) if we changed sys_tune parameters
      pg_context=$(get_pgContext)

      echo -n `date +%F.%H:%M:%S`" [$funcname] : Checking if sys_tune changed configuration... " >> $LOG

      if [ "$pg_context" == "none" ]; then
        echo "NO (No updates)" >> $LOG
        return 1 ## nothing changed
      elif [ "$pg_context" == "reload" ]; then
        echo "YES (Needs DB $pg_context)" >> $LOG
	mode=$pg_context
      else
        echo "YES (Needs DB $pg_context)" >> $LOG
        echo "Configuration changes will take effect after restart" >> $LOG 
	mode=$pg_context
      fi
    fi

	# Replace the configuration file with our tuned version
	echo "Updating postgresql.conf configuration file..." >>$LOG
	mv -f $BPDIR/data/postgresql.conf $BPDIR/data/postgresql.conf.orig.$$
	cp -f $BPSCHEMA/postgresql.conf $BPDIR/data/postgresql.conf
	chown postgres:postgres $BPDIR/data/postgresql.conf
	chmod 0600 $BPDIR/data/postgresql.conf
    return 0
}

# Set up the SSL keyfile and certificate
function create_ssl_config() {
	origmask=`umask`
	umask 0377

	echo "Generating SSL keyfile..." >>$LOG
	openssl genrsa 1024 > $BPDIR/data/server.key 2>>$LOG
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "SSL creation failed: \"openssl genrsa\" returned $ret." >>$LOG
		finish_and_exit $ret
	fi

	echo "Generating SSL certificate..." >>$LOG
	openssl req -new -x509 -nodes -sha1 -days 3650 -key $BPDIR/data/server.key \
	   -subj "/C=US/ST=./L=./O=Unitrends/CN=`hostname`" > $BPDIR/data/server.crt 2>>$LOG
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "SSL creation failed: \"openssl req\" returned $ret." >>$LOG
		finish_and_exit $ret
	fi

	chown postgres:postgres $BPDIR/data/server.key $BPDIR/data/server.crt
	echo "SSL setup successful." >>$LOG
	umask $origmask 2>/dev/null
}

# Tells Postgres to reload the configuration ($1 = "reload") or restart ($1 = "restart")
function reset_postgres() {
mode=$1
# create a pid file if it is missing, reload fails otherwise
pgpidf=$BPDIR/data/postmaster.pid 
if [ ! -f $pgpidf ]; then
   pgpid=`ps -ef |grep -v grep |grep postg |awk '{ print $3 }' |tail -n1`
   echo "$pgpid"  >$pgpidf
   echo "$BPDIR/data" >>$pgpidf
   chown postgres:postgres $pgpidf
   chmod 600 $pgpidf
fi
case "$mode" in
	reload)
		echo "Reloading PostgreSQL configuration after modifications..." >> $LOG
		echo "Using command \"sudo -iu postgres pg_ctl $mode -D $BPDIR/data -w -m smart\"" >>$LOG
		# The -w option causes pg_ctl to wait until reload is done
		sudo -iu postgres pg_ctl $mode -D $BPDIR/data -w -m smart>>$LOG 2>&1
		sudo -iu postgres pg_ctl status -D $BPDIR/data >>$LOG 2>&1
		ret=$?
		if [ $ret -ne 0 ]
		then
			echo "Configuration \"$mode\" operation returned $ret." >>$LOG
			finish_and_exit $ret
		else
			echo "Configuration \"$mode\" operation was successful." >>$LOG
		fi
		;;

	restart)
		echo "Restarting PostgreSQL database after modifications to the configuration..." >> $LOG
		restart_db
		echo "Configuration \"$mode\" operation was successful." >>$LOG
		;;

	*)
		;;
esac
}

##__________________________________________________________________
## pgbouncer service setting up of pg_service.conf control functions
function pgbouncer_control() {

  ## default is to append without checking for duplicates
  ## svcMode {1: append into new config, 2: update to existing config}
  local svcMode=${1:-"1"}
  local funcname=${FUNCNAME-"pgbouncer_control"}
  local progname="pgbouncer"
  local svcConnParams="$BPSCHEMA/pg_service.control.txt"
  local pgServicePath="$BPDIR/data"
  local pgServiceFile="pg_service.conf"

  [ ! -f $svcConnParams ] && return 1;

  ## check if pgbouncer is setup and if so, add 
  $progname -V &> /dev/null;
  pgb_status=$?
  echo `date +%F.%H:%M:%S`" [$funcname] : checking if $progname installed: status=$pgb_status" >>$LOG

  if [ $pgb_status -eq 0 ]; then

    ## pg_service.conf create mode
    if [ $svcMode -eq 1 ]; then
      echo `date +%F.%H:%M:%S`" [$funcname] : using append mode " >>$LOG
      cat $svcConnParams >> $pgServicePath/$pgServiceFile
      pgb_status=$?
      echo `date +%F.%H:%M:%S`" [$funcname] : append $svcConnParams to $BPDIR/data/pg_service.conf returns status=$pgb_status" >>$LOG

    ## pg_service.conf append mode
    elif [ $svcMode -eq 2 ]; then

      echo `date +%F.%H:%M:%S`" [$funcname] : using update mode " >>$LOG
      pgb_status=0 ## ok      
      ## this gets complicated. if file exists, it may have many
      ## control func's defined each with their own lvalue=rvalue pairs.
      ## if its a replication target, or cross-vault some of the func's are there to establish
      ## remote (ssl-based) db-access. we preserve those and replace only those control 
      ## func's that we have marked for replacement (i.e, those from the template)

      ## placeholder for copying existing pg_service.conf
      tmpSvcFile=".$pgServiceFile.tmp.txt"

      ## get control func headers from our template (they are enclosed in brackets, eg [dbconn1]
      declare -a keywrd
      for item in `grep "^\[" $svcConnParams | sed -e "s/\[\|\]//g"`; do
        keywrd[${#keywrd[@]}]="$item"
      done

      ## if we have nothing to replace, we are done
      [ ${#keywrd[@]} -lt 1 ] && return $pgb_status

      ## construct a string by chaining the headers to make a 
      ## awk-aware regexp (eg, /\[dbconn1\]/||/\[dbconn2\]/ )
      keyargs="/"
      cnt=`expr ${#keywrd[@]} - 1`
      if [ $cnt -gt 0 ]; then
        for i in `seq 0 $cnt`; do
          keyargs=$keyargs"\[${keywrd[$i]}\]/"
          if [ $i -lt $cnt ]; then
            keyargs=$keyargs"||/"
          fi
        done
      fi

      tmpDir="/tmp"
      rm -f $tmpDir/$tmpSvcFile &> /dev/null;

      ## now we are ready to strip the old headers that match exactly the ones from our template
      ## cleanup white spaces and new lines get comments, control func's and headers as follows:

      ## get keywords | remove blank lines | remove blank/tab spaces
      ## remove space or tabs before  | after | and between keyword and brackets
      ## process line, if it has keyword, mark line for deletion and continue down the file until the end of block,
      ## else mark for keeping and continue down the file until the block ends, including the one before EOF and save it in temp
      egrep "#|\[|=" $pgServicePath/$pgServiceFile | awk '{$1=$1}{print}' | awk '{gsub(/^[ \t]+|[ \t]+$/,"");if('/./')print}' \
      | sed -e 's/^[ \t]*//' -e 's/[ \t]*$//' -e 's/\[[ \t]/\[/' -e 's/[ \t]\]/\]/' \
      | awk '{if('$keyargs') printf "_FINI_\n_REMOVE_\n"; else {if(/\[/){print"_FINI_\n_KEEP_\n"$0} else print;}}END{print "_FINI_"}' \
      > $tmpDir/$tmpSvcFile
      ## its safer to extract the func headers to keep rather than deleting
      ## the headers that are to be replaced by the new ones from template. 
      ## get only those headers that we must keep and do an in-place edit
      [ -f $tmpDir/$tmpSvcFile ] && [ `wc -l $tmpDir/$tmpSvcFile | awk '{print $1}'` -gt 0 ]
      pgb_status=$?
      awk '/_KEEP_/{printf "-e"NR","}/_FINI_/{print NR"p"}' $tmpDir/$tmpSvcFile \
      | awk '{if(/[0-9]*\,[0-9]*p/)print}' | xargs sed -i -n $tmpDir/$tmpSvcFile

      ## sanity check to make sure we have at least the basic control func [localhost]
      system="localhost"
      sed -e "s/_FINI_//g" -e "s/_KEEP_//g" $tmpDir/$tmpSvcFile | grep "^\[$system\]" &> /dev/null
      pgb_status=$?
      if [ $pgb_status -ne 0 ]; then
        echo `date +%F.%H:%M:%S`" [$funcname] : could not retrieve control functions from $svcConnParams: status=$pgb_status" >>$LOG
      else
        sed -e "s/_FINI_//g" -e "s/_KEEP_//g" $tmpDir/$tmpSvcFile > $tmpDir/$tmpSvcFile.1
        echo >> $tmpDir/$tmpSvcFile.1
        cat $svcConnParams >> $tmpDir/$tmpSvcFile.1
        ## remove extra blank lines (if any) and duplicate consecutive lines (if any)
        cat -s $tmpDir/$tmpSvcFile.1 | sed '$!N; /^\(.*\)\n\1$/!P; D' > $pgServicePath/$pgServiceFile
        echo `date +%F.%H:%M:%S`" [$funcname] : update $svcConnParams to $BPDIR/data/pg_service.conf returns status=$pgb_status" >>$LOG
	rm -f $tmpDir/$tmpSvcFile.1 $tmpDir/$tmpSvcFile &> /dev/null;
      fi
    else
      pgb_status=1
      echo `date +%F.%H:%M:%S`" [$funcname] : unrecognized option svcMode=$svcMode: status=$pgb_status" >>$LOG
    fi
  fi
  return $pgb_status
}

##__________________________________________________________________
## pgbouncer service setting up of pg_service.conf control functions
function create_connsvc() {

  local funcname=${FUNCNAME-"create_connsvc"}

  local svcTemplFile="pg_service.conf.sample"
  local svcSearchDirs="pgsql*"
  local svcDestDir="/etc/sysconfig/pgsql"
  local pgServiceFile="pg_service.conf"
  local pgServicePath="$BPDIR/data"

  pgdirs="`find /usr/ -name $svcSearchDirs -type d | xargs`"
  [ "$pgdirs" == "" ] && echo `date +%F.%H:%M:%S`" [$funcname] : failed to find dirs /usr/$svcSearchDirs. status=$pgb_status" >>$LOG && return 1

  svcTemplPath="`find $pgdirs -name $svcTemplFile | head -n 1`"
  [ "$svcTemplPath" == "" ] && echo `date +%F.%H:%M:%S`" [$funcname] : failed to find $svcTemplFile in $pgdirs. status=$pgb_status" >>$LOG && return 1

  echo `date +%F.%H:%M:%S`" [$funcname] : creating connection service file" >>$LOG
  if [ -r $svcTemplPath ]; then
    cp -f $svcTemplPath $svcDestDir/$pgServiceFile 2>/dev/null
    cpret=$?
  else
    touch $svcDestDir/$pgServiceFile
    cpret=$?
  fi
  ln -sf  $svcDestDir/$pgServiceFile $pgServicePath/$pgServiceFile 2>/dev/null
  lnret=$?
  if [ $cpret -ne 0 -o $lnret -ne 0 ]; then
    echo `date +%F.%H:%M:%S`" [$funcname] : connection service file creation failed." >>$LOG
    finish_and_exit 1
  fi

  system="localhost"
  echo -en "
[$system]
user=postgres
connect_timeout=5
" >> $pgServicePath/$pgServiceFile 
  conn_status=$?

  echo `date +%F.%H:%M:%S`" [$funcname] : connection service file creation returns status=$conn_status" >>$LOG
  return $conn_status
}



##########################################################################
# main program begins here

echo "=== Begin $0 $scenario on `date`." >>$LOG

netstat -anp | grep PGSQL.5432 >/dev/null
if [ $? -eq 0 ]
then
	DBisRunning=TRUE
else
	DBisRunning=FALSE
fi
default_db_dir=/backups/UnitrendsDataBase
default_platform_dir=/var/opt/unitrends/database/UnitrendsDataBase
PG_TUNE=/usr/bp/etc/.pg_tune

if [ $scenario = CREATE ]
then
	# For one case of DPU restore "create" is passed with an extra param
	# This happens only when a 5.0.1 archive/vaulted state is restored
	# to a current version of the software. An old copy of schema is 
	# first installed, then DR restores some tables, and then the 
	# schema is upgraded to latest. All this is driven by DR. 
	# The required pre DB clean up is not done by the DR script, 
	# and therefore is handled here.
	if [ "$2" = "dpurestore" ] 
	then
		BPSCHEMA=$BPDIR/schema_oldScripts
		LOG=$BPLOGDIR/dr_dbsetup.log
		# Do not stop uarchive daemon when stopping DB
		if [ $DBisRunning = TRUE ]
		then
			$BPDIR/bin/stop_db.sh no_uarchive >>$LOG 2>&1
			ret=$?
			if [ $ret -ne 0 ]
			then
				finish_and_exit $ret
			fi
		fi
		netstat -anp | grep PGSQL.5432 >/dev/null
		if [ $? -eq 0 ]
		then
			DBisRunning=TRUE
		else
			DBisRunning=FALSE
			DB_PATH=`readlink -f $BPDIR/data 2>/dev/null`
			if [ $? -ne 0 ]
				then
				echo "Failed to find database location" >> $LOG
				finish_and_exit $?
			fi
			if [ -d $DB_PATH.dr ]
				then 
				`rm -rf $DB_PATH.dr`
			fi

			echo "Saving copy of current database" >> $LOG
			mv -if $DB_PATH{,.dr} >> $LOG
			if [ $? -ne 0 ]
			then
				echo "Failed to remove $DB_PATH" >> $LOG
				finish_and_exit $?
			fi
		fi
	fi
	
	if [ $DBisRunning = TRUE ]
	then
		echo "The PostgreSQL database is running!" >>$LOG
		echo " - Back up any data that should not be lost (using pg_dump or pg_dumpall)." >>$LOG
		echo " - Shutdown the database ($BPBIN/stop_db.sh)." >>$LOG
		echo " - Remove or move the database directory (/backups/UnitrendsDataBase)." >>$LOG
		finish_and_exit 1
	fi
	if [ ! -d $default_db_dir ] && [ ! -L $default_db_dir ]
	then
		# Default location dir doesn't exist, and is not a symlink. 
		# Stateless creates a symlink which needs to be retained when DR creates the DB again
		`grep -q database_separate_filesystem /var/opt/unitrends/platform_capabilities 2>/dev/null`
		if [ $? -eq 0 ]; then
			# Create a link to the database file system.
			# platform_capabilities stores the path to use for the database
			# If it does not, use /var/opt/unitrends/database/UnitrendsDatabase
			# If something here ever changes, CleanAndCreateDB() in cmc_dpu_restore_utility should
			# be changed too.
			final_db_dir=`grep database_separate_filesystem /var/opt/unitrends/platform_capabilities | cut -d ':' -f 2`
			if [ -z "$final_db_dir" ]; then
				final_db_dir=$default_platform_dir
			fi
			if [ -d "$final_db_dir" ]; then
				if [ "`ls -A $final_db_dir`" ]; then
					echo "The database directory ($final_db_dir) is not empty!" >> $LOG
					echo " - Remove the contents of the directory." >> $LOG
					finish_and_exit 1
				fi
			fi

			echo "Creating $final_db_dir" >> $LOG
			mkdir -p $final_db_dir
			# This symlink shouldn't exist at this point
			echo "Linking $final_db_dir to $default_db_dir" >> $LOG
			ln -s $final_db_dir $default_db_dir
		else
			# We didn't find an existing link for /backups/UnitrendsDatabase, nor
			# found anything in platform_capabilities. Create the default directory.
			# our DB will reside here
			final_db_dir=$default_db_dir;
			echo "Creating $final_db_dir" >> $LOG
			mkdir -p $final_db_dir
		fi
	elif [ -L $default_db_dir ]
	then
		# Our desired default_db_dir is symlinked to some location already. 
		# Let us create this new DB there instead.
		final_db_dir=`readlink -f $default_db_dir 2>/dev/null`
		if [ $? -ne 0 ]
		then
			echo "$default_db_dir is a link, but failed to find its location." >> $LOG
			finish_and_exit 1
		fi
		# DR DB creation kick off script is responsible to keep this directory empty
		# We don't have the default_db_dir symlinked to a non-platform-capability location 
		# at the time of a fresh installation.
		echo "Creating $final_db_dir" >> $LOG
		mkdir -p $final_db_dir
	elif [ -d $default_db_dir ]
	then
		# Error case. This should not happen in an ideal scenario.
		# The check below will fail, but will atleast not check 
		# an empty (or unset $final_db_dir)
		final_db_dir=$default_db_dir;
	fi

	# At this point recheck if our final DB location is empty.
	if [ "`ls -A $final_db_dir`" ]
	then
		echo "The database directory ($final_db_dir) is not empty!" >>$LOG
		echo " - Remove the contents of the directory." >>$LOG
		finish_and_exit 1
	fi
	# Make sure the permissions are set correctly
	chown postgres:postgres /backups/UnitrendsDataBase
	chmod 0700 /backups/UnitrendsDataBase
	if [ ! /backups/UnitrendsDataBase -ef $BPDIR/data ]
	then
		ln -s /backups/UnitrendsDataBase $BPDIR/data
	fi
	
	sudo -u postgres locale >> $LOG 2>&1
	
	# UNIBP-653: For some mysterious reason, the locale settings 
	# seem to change that causes database creation to fail during DR. 
	# For non-UTF8 locales, change it to UTF8. Don't change anything if 
	# LANG is set to either C or POSIX.
	langbase=`sudo -u postgres echo $LANG | cut -f1 -d'.'`
	if [ $langbase != "C" ] && [ $langbase != "POSIX" ] && [ ! -e /usr/share/i18n/locales/$langbase ]
	then
		# We have an invalid lang, set it to en_US.UTF-8
		echo "Lang=$langbase is not valid. Setting to en_US.UTF-8" >> $LOG

		export LANG="en_US.UTF-8"
		export LC_ALL="en_US.UTF-8"
	fi
	`sudo -u postgres echo $LANG | grep -q UTF-8`
	if [ $? -ne 0 ]
	then
		# No UTF-8 in lang
		if [ $langbase != "C" ] && [ $langbase != "POSIX" ]
		then
			export LANG="${langbase}.UTF-8"
			export LC_ALL="${langbase}.UTF-8"
		fi
	fi
	echo "Lang is set to $LANG." >>$LOG

	echo "Running initdb to create a new PostgreSQL database cluster..." >>$LOG
	sudo -u postgres initdb -D $BPDIR/data -E UTF-8 >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: initdb returned $ret." >>$LOG
		finish_and_exit $ret
	fi

	create_ssl_config
	update_conf_file
	
	echo "Starting postmaster..." >>$LOG
	sudo -iu postgres pg_ctl start -D $BPDIR/data -l $BPDIR/data/logfile -w  >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: pg_ctl start returned $ret." >>$LOG
		finish_and_exit $ret
	fi

	sudo -iu postgres createdb bpdb >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: createdb returned $ret." >>$LOG
		finish_and_exit $ret
	fi
fi # end of CREATE block


if [ $scenario = UPDATE ]
then
	# Try to start up postmaster if it is not running.
	if [ $DBisRunning = FALSE ]
	then
		echo "PostgreSQL database is not running; attempting startup..." >>$LOG
		sudo -iu postgres pg_ctl start -D $BPDIR/data -l $BPDIR/data/logfile -w  >>$LOG 2>&1
		ret=$?
		if [ $ret -ne 0 ]
		then
			echo "ERROR: PostgreSQL database is not running and could not be started!" >>$LOG
			finish_and_exit $ret
		else
			echo "Startup was successful, continuing..." >>$LOG
		fi
	fi
fi

createlang -U postgres -l bpdb | grep -i plpgsql >/dev/null
if [ $? -ne 0 ]
then
	echo "Installing PL/pgSQL language in bpdb database..." >>$LOG
	createlang -U postgres plpgsql bpdb >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: Language installation returned $ret." >>$LOG
		finish_and_exit $ret
	else
		echo "Language installation was successful." >>$LOG
	fi
fi

# Edit or update the database configurations file(s) if needed.
# Note: check the documentation at www.postgresql.org to see if the parameter being
# changed requires postmaster to only re-read its configuration.  Changes that require
# restart of the server should set "mode" to "restart", otherwise, "mode" should be
# "reload".

mode="reload"

# Add an entry for Exchange Server roles
grep "bpexch,wguest" $BPDIR/data/pg_hba.conf >/dev/null
if [ $? -ne 0 ]
then
	echo "host    bpdb    +bpexch,wguest  0.0.0.0/0              md5" >>$BPDIR/data/pg_hba.conf
	mode="reload"
fi

# Create an SSL keyfile and self-signed certificate for remote connections
if [ ! -f $BPDIR/data/server.key -o ! -f $BPDIR/data/server.crt ]
then
	create_ssl_config
	mode="restart"
fi

# Create connection service file
# 19121: We don't want to log any messages for pgbouncer in CentOS5
## get the version of os. regexp extracts the version number 
## from string: eg:  "2.6.32-431.el6.x86_64"
pgb_cosver=`uname -r | sed -e "s/.*el//" -e "s/\..*//" -e "s/.//2g"`

if [ ! -f $BPDIR/data/pg_service.conf ]; then
    create_connsvc; 
    if [ $pgb_cosver -gt 5 ]; then
        ## if pgbouncer is installed, then add service control functions
        svcMode=1 ## append mode
        pgbouncer_control $svcMode; 
        pgb_status=$?
        if [ $pgb_status -ne 0 ]; then
            echo `date +%F.%H:%M:%S`" [pgbouncer_control] : failed to add pgbouncer control functions. status=$pgb_status" >>$LOG
        fi
    fi
else
    if [ $pgb_cosver -gt 5 ]; then
        svcMode=2  ## update mode
        pgbouncer_control $svcMode;
        pgb_status=$?
        if [ $pgb_status -ne 0 ]; then
            echo `date +%F.%H:%M:%S`" [pgbouncer_control] : failed to add pgbouncer control functions. status=$pgb_status" >>$LOG
        fi
    fi
fi
chown postgres:postgres $BPDIR/data/pg_service.conf

# Update configuration file
# Restart is not required for all configuration changes.

# Make sure database timezone is set to system timezone
check_timezone
diff -EBbwq $BPSCHEMA/postgresql.conf $BPDIR/data/postgresql.conf >/dev/null 2>&1
if [ $? -ne 0 ]
then
    ## (todo 18040): if pg_settings context requires restart for the parameter
    ## modification to take effect, then we do a health check and 
    ## if it passes - we restart, else we throw an error and exit
    mode="reload"
    update_conf_file diff_found
    update_status=$?
    if [ $update_status -eq 0 ]; then
        # Tell postmaster to re-read or restart the configuration
        reset_postgres $mode
    fi
fi

# Now apply all the SQL scripts.
# These SQL files are run on both fresh install and upgrade.
# bpdb.sql  - the base schema
# install_world.sql - schema to support Exchange vaulting
# create_functions.sql - create or replace any functions
for sql_file in bpdb.sql install_world.sql create_functions.sql
do
	echo "Loading $sql_file..." >>$LOG
	sudo -iu postgres psql -v ON_ERROR_STOP=TRUE -f $BPSCHEMA/$sql_file bpdb >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: Load of $sql_file returned $ret." >>$LOG
		finish_and_exit $ret
	else
		echo "Load of $sql_file was successful." >>$LOG
	fi
done

# This set is only applied when the database is first created on a fresh install.
# load_tables.sql - populates tables of static data
# create_views.sql - creates views
# If we are updating from a non-rpm version of the product (pre-3.0), this should 
# be treated like a fresh install for these files.
if [ $scenario = CREATE -o -f $BPDIR/UpgradeFromNonRPMversion ]
then
	for sql_file in load_tables.sql create_views.sql
	do
		echo "Loading $sql_file..." >>$LOG
		sudo -iu postgres psql -v ON_ERROR_STOP=TRUE -f $BPSCHEMA/$sql_file bpdb >>$LOG 2>&1
		ret=$?
		if [ $ret -ne 0 ]
		then
			echo "ERROR: Load of $sql_file returned $ret." >>$LOG
			finish_and_exit $ret
		else
			echo "Load of $sql_file was successful." >>$LOG
		fi
	done
fi

# Now run any modifications -- this is run both on fresh install & upgrade
for sql_file in update_bpdb.sql notifications.sql
do
	echo "Loading $sql_file..." >>$LOG
	sudo -iu postgres psql -v ON_ERROR_STOP=TRUE -f $BPSCHEMA/$sql_file bpdb >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: Load of $sql_file returned $ret." >>$LOG
		finish_and_exit $ret
	else
		echo "Load of $sql_file was successful." >>$LOG
	fi
done

# Now add the local system and a root/unitrends1 superuser to the database if not present already
# Also, update any existing systems without assigned locations to be at the "Default Location"
hostname=`hostname`
# Beware: this command doesn't strip leading whitespace from strings.
# Add the -A option if you want unaligned output.
psql="psql bpdb -U postgres -v ON_ERROR_STOP=TRUE -qt 2>>$LOG"

select_local_system_sql="SELECT COUNT(*) FROM bp.systems WHERE is_local_system = TRUE;"
select_orphaned_system_sql="SELECT COUNT(*) FROM bp.systems WHERE location_id IS NULL;"
has_local_system=`echo "$select_local_system_sql" | $psql`
ls_status=$?
has_orphaned_system=`echo "$select_orphaned_system_sql" | $psql`  # may be > 1, so compare as 0 or non-zero
os_status=$?
if [ $ls_status -ne 0 -o $os_status -ne 0 ]
then
	echo "Error querying for local and orphaned systems in database" >>$LOG
	finish_and_exit 1
fi

# if the local system doesn't exist or there are orphaned systems, add the default customer/location
if [ $has_local_system -eq 0 -o $has_orphaned_system -ne 0 ]
then
	customer_sql="INSERT INTO bp.customers (name) SELECT 'Default Customer' AS name WHERE (SELECT COUNT(*) FROM bp.customers WHERE name = 'Default Customer') = 0;"
	location_sql="INSERT INTO bp.locations (name, customer_id) SELECT 'Default Location' AS name, (SELECT customer_id FROM bp.customers WHERE name = 'Default Customer') AS customer_id WHERE (SELECT COUNT(*) FROM bp.locations WHERE name = 'Default Location') = 0;"
	echo "$customer_sql $location_sql" | $psql
	if [ $? -ne 0 ]
	then
		echo "Error adding default customer and location to database" >>$LOG
		finish_and_exit 1
	fi

	# create the local system if one doesn't exist
	# 6794: the role is 'DPU' on fresh install
	if [ $has_local_system -eq 0 ]
	then
		system_sql="INSERT INTO bp.systems (name, host, is_local_system, location_id, role) VALUES ('${hostname}', 'localhost', TRUE, (SELECT location_id FROM bp.locations WHERE name = 'Default Location'), 'DPU');"
		echo "$system_sql" | $psql
		if [ $? -ne 0 ]
		then
			echo "Error adding local system to database" >>$LOG
			finish_and_exit 1
		fi
	fi

	# set the location_id of any systems without assigned locations
	if [ $has_orphaned_system -ne 0 ]
	then
		system_u_sql="UPDATE bp.systems SET location_id = (SELECT location_id FROM bp.locations WHERE name = 'Default Location') WHERE location_id IS NULL;";
		echo "$system_u_sql" | $psql
		if [ $? -ne 0 ]
		then
			echo "Error adding local system to database" >>$LOG
			finish_and_exit 1
		fi
	fi
fi

bpname=`psql bpdb postgres -t -c "select name from bp.systems where is_local_system = TRUE" |awk '{ print $1 }'`
if [ "x${bpname}" != "x${hostname}" ]; then
	echo "local bpname not equal $bpname != $hostname" >>$LOG
	system_sql="update bp.systems set name = '${hostname}' where is_local_system = TRUE"
	echo "$system_sql" | $psql
fi

select_user_sql="SELECT COUNT(*) FROM bp.users;"
# create the default root/unitrends1 user
users=`echo "$select_user_sql" | $psql`
if [ $? -ne 0 ]
then
	echo "Error querying for existing web interface users" >>$LOG
	finish_and_exit 1
fi
if [ "$users" -eq 0 ]; then
	user_sql="BEGIN; INSERT INTO bp.users (username, password) VALUES ('root', '38776419428ba4444f1d9b9f7aeda5623b7e5bbf'); INSERT INTO bp.superusers (user_id) SELECT user_id FROM bp.users WHERE username = 'root'; COMMIT;";
	echo "$user_sql" | $psql
	if [ $? -ne 0 ]; then
		echo "Error creating default user in database" >>$LOG
		finish_and_exit 1
	fi
fi
echo "Database entries are present for user and local system." >>$LOG

# Add connection service entries for all systems in the database
systems=`psql -U postgres bpdb -At -c 'SELECT host FROM bp.systems'`
if [ $? -ne 0 ]; then
	echo "Error querying for systems to add to the connection services file" >>$LOG
	finish_and_exit 1
fi
for system in $systems; do
	# localhost was added when the file was created
	if [ "$system" != "localhost" ]
	then
		# add connection service entries that are not already present
		if [ -z "`cat $BPDIR/data/pg_service.conf | grep \"^\[$system\]\"`" ]
		then
			echo -en "
[$system]
user=postgres
host=$system
connect_timeout=3
"			>> $BPDIR/data/pg_service.conf
		fi
	fi
done

# 6794: set values for the new "role" column for existing records in the
# systems table.  This migration occurs only once, on the first upgrade to
# release 4.1.
select_role_sql="SELECT role FROM bp.systems WHERE is_local_system = true"
local_role=`echo "$select_role_sql" | $psql -A 2>>$LOG`
if [ $? -ne 0 ]
then
	echo "Error querying for role of local system" >>$LOG
	finish_and_exit 1
else
	if [ "$local_role" = "Unknown" ]
	then
		echo "Migrating role values in systems table..." >>$LOG
		$BPDIR/bin/migrate_roles.php >>$LOG 2>&1
		if [ $? -ne 0 ] 
		then
			echo "Error updating the role column in the systems table." >>$LOG
			echo "Run $BPDIR/migrate_roles.php to complete the migration.">> $LOG
			finish_and_exit 1
		else
			echo Update of role column in the systems table was successful. >> $LOG
		fi
	fi
fi

# Populate the managers table with external systems having DB access.
# This migration occurs only once, on the first upgrade to release 4.1.
select_mgrs_sql="SELECT count(*) FROM bp.managers"
mgrs=`echo "$select_mgrs_sql" | $psql -A 2>>$LOG`
if [ $? -ne 0 ]
then
	echo "Error querying for number of managers" >>$LOG
	finish_and_exit 1
else
	if [ $mgrs -eq 0 ]
	then
		if grep "^hostssl" $BPDIR/data/pg_hba.conf >/dev/null
		then
			echo "Identifying remote managing systems..." >>$LOG
			$BPDIR/bin/migrate_mgrs.php >>$LOG 2>&1
			if [ $? -ne 0 ]
			then
				echo "Error populating the managers table." >>$LOG
				echo "Run $BPDIR/migrate_mgrs.php to complete the migration.">> $LOG
				finish_and_exit 1
			fi
		fi
	fi
fi


# Ensure there is a database install id present.  It uniquely identifies
# this particular dataset, and is restored upon DPU Restore of state.
install_id=`uuidgen -t`
install_id_sql="INSERT INTO bp.state (install_id) SELECT '$install_id' WHERE NOT EXISTS (SELECT * FROM bp.state)"
echo "$install_id_sql" | $psql
if [ $? -ne 0 ]; then
	echo "Error creating database install id" >>$LOG
	finish_and_exit 1
fi

# If no migration is occurring, then set the migration status as 'done'
if [ $scenario = CREATE ] && [ ! -f $BPDIR/lists.dir/.migrate_db ]; then
	echo "No database migration is necessary." >>$LOG
	migration_sql="UPDATE bp.operation_status SET status='done' WHERE operation_id=2"
	echo "$migration_sql" | $psql
	if [ $? -ne 0 ]; then
		echo "Error updating migration status" >>$LOG
		finish_and_exit 1
	fi
else
	echo "Not updating database migration status." >>$LOG
fi

#Bug 11787
#Before finishing, set the version in systems table. If, for some reason,
#appliance state is vaulted/archived with this information..can have problems.
#The version is again set from postinstall on a fresh/upgrade from rpm.
set `rpm -q --queryformat '%{version}-%{release}\n' unitrends-rr`
if [ $# -eq 1 ]
then
	version=$1
else
	version=$2
fi
echo "Updating version in the systems table to $version..." >>$LOG
psql -Upostgres bpdb -c \
	"UPDATE bp.systems SET version='$version' WHERE host='localhost'" >>$LOG 2>&1
if [ $? -eq 0 ]
then
	echo "Successfully updated version in systems table to $version" >>$LOG
else
	echo "ERROR: failed to update version in systems table to $version" >>$LOG
fi

finish_and_exit 0
