#!/bin/sh
## This script stops all BP programs and finally the database

# Change log entries
# Bug# Date    Who Description
# ---- ------- --- -----------------------------------------------------------------
# 18234 11Mar14 VSK stop_bp.sh will do killbins and stop_db.sh modified to call it

. /etc/default/bp.ini 2> /dev/null

SLEEPTIME=20
DBDIR="$BPDIR/data"
PIDFILE="$DBDIR/postmaster.pid"
BPONLY=0
STOPPG="sudo -iu postgres pg_ctl stop -D $DBDIR "
STOPFILE="$DBDIR/db_stopped"
kill9ok=0
# this list needs to match what is in preinstall
KILLBINS="alertman cmc_delete_device xbpmenu xbpview xbackup xrestore bp_datasweep vaultServer exch_securesync \
          xpromon bpserverd bkup updatedb reorgdb x_backup x_bpsummary x_promon \
          x_restore xbm_boot xbm_cd xbm_floppy xbpconfig xbpschedule x_bpview \
          restore_client statussyncd archive lcdman recipe \
          bpcleanup bpsync uarchive fileDedup bputil space_reclaimer \
          restore_daemon inlineHashdb applianceAgent footprintReportUtil \
          metadataUtil SISreclaim retentionMgr"


### functions
#----------------------------------------------------------------------
function list_psD() {
   # detect if any process is in uninterruptible sleep (STAT has D)
   ps aux | while read line
   do
      stat=`echo $line |awk '{ print $8 }'`
      echo $stat |grep "D" >/dev/null 2>&1
      if [ $? -eq 0 ]; then
         echo "$line"
      fi
   done
}

## VSK: 04/04/2014: Function to terminate backend pids
#----------------------------------------------------------------------
function termPGBackend() {

  local funcname=${FUNCNAME-"termPGBackend"}
  local curdb="bpdb"
  local dbuser="postgres"
  local osversion=`uname -r | sed -e "s/.*el//" -e "s/\..*//" -e "s/.//2g"`

  ## centos 5 has column "procpid" and function: pg_cancel_backend()
  local procpid="procpid"
  local terminate="cancel"
  ## centos 6 or higher has column "pid" and function: pg_terminate_backend()
  if [ $osversion -gt 5 ]; then
    procpid="pid"
    terminate="terminate"
  fi  

  ## query to terminate all backend pids except myself
  query="SELECT pg_${terminate}_backend(${procpid}) FROM pg_stat_activity"
  clause=" WHERE ${procpid} <> pg_backend_pid() AND datname=current_database();"
  psql $curdb $dbuser -qtc "\q" > /dev/null 2>&1
  dbstate=$?

  ## if db is accessible then terminate pids using query
  if [ $dbstate -eq 0 ]; then
    echo `date +%F.%H:%M:%S`"| Disconnecting BP processes from Database"
    cat <<-EoQ | psql $curdb $dbuser -Xqltc "$query $clause" > /dev/null 2>&1 < /dev/null
EoQ
    ## if db is in Q-state try getting pids other than exludePattern (below)
    ## and terminate them if any of then exist
    excludePattern="logger|checkpointer|writer|autovacuum|-D|stats"
    declare -a dbpids
    dbpids=(`ps --no-headers -o pid,cmd -u $dbuser --sort=pid | egrep -v "$excludePattern" | awk '{print $1}'`)
    for pid in ${dbpids[@]}; do
      kill -TERM $pid > /dev/null 2>&1
    done
  fi  
}
#----------------------------------------------------------------------
## ## VSK: 04/04/2014: Function to check if db is active or not
function pgUp() {
  ## return codes: 0 means UP and 2 means down
  psql bpdb postgres -Xqltc "\q" > /dev/null 2>&1; 
  echo $?
}

#----------------------------------------------------------------------
function promptDone() {
  echo `date +%F.%H:%M:%S`"| Done"
  tail -n20 $STOPFILE >/tmp/dbstop.tmp 2>/dev/null
  cp -f /tmp/dbstop.tmp $STOPFILE
  rm -f /tmp/dbstop.tmp
  echo "`date +%F.%H:%M:%S`| database stopped " >>$STOPFILE
}

#----------------------------------------------------------------------
# need this because 'pgrep -f postgres' wont catch postmaster
pgpids() {
   ps -ef |grep -v grep |grep postgres |awk '{ print $2 }'
}

#----------------------------------------------------------------------
# main logic

if [ "x$1" = "xbp" ]; then
   BPONLY=1
   echo "Stop BP processes, but not Database"
fi

#create lock file to stop new vaultServer instances from touching the db
touch $BPDIR/lists.dir/vaultServer.lock

# Stop tasker in background so that if it hangs we won't hang this script.
tasker_pid=`pgrep tasker`
if [ -n "$tasker_pid" ]
then
	echo -n `date +%F.%H:%M:%S`"| Stopping tasker..."
	$BPDIR/bin/dispatch stop &
	# get the PID of dispatch
	dispatch_pid=$!
	sleep 1
	for i in `seq $SLEEPTIME`
	do
		# see if that PID is still running, pgrep could return multiple others
		pids=`pgrep dispatch`
		echo $pids | grep $dispatch_pid >/dev/null
		if [ $? -ne 0 ]
		then
			break
		else
			echo -n '.'
			sleep 1
		fi
	done

	pids=`pgrep dispatch`
	echo $pids | grep $dispatch_pid >/dev/null
	if [ $? -eq 0 ]
	then
		# kill the hung dispatch, plus tasker itself
		kill $dispatch_pid
		kill $tasker_pid
	fi
	echo
fi

echo
echo `date +%F.%H:%M:%S`"| Stopping BP programs..."

# Stop any running inventory sync
if pgrep inventory_sync >/dev/null
then
	# the cron job is detectable by this name
	pkill inventory_sync
else
	invsyncPid=`psql -Upostgres bpdb -Atc "select pid from bp.inventory_sync" 2>/dev/null `
	if [ -n "$invsyncPid" -a "$invsyncPid" != "0" ]
	then
		kill $invsyncPid
	fi
fi

# Handle vcd differently.  We only want to kill the parent process,
# then wait for it to exit (after the children have been cleaned up).
# gives more time for processes to clean up and exit (especially fileDedup).
if pgrep -x vcd >/dev/null 2>&1
then
        pkill -P 1 vcd
        timeout=30
        sleeps=0
        while pgrep -x vcd >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
        do
                ((sleeps=$sleeps + 1))
        done
    if [ $kill9ok -eq 1 ]; then
        if pgrep -x vcd >/dev/null 2>&1
        then
                # We don't want installation to fail
                pkill -9 vcd
        fi
    fi
fi

# Don't kill any of our ancestors
savepids=
chkpid=$PPID
while [ -n "$chkpid" -a "$chkpid" -ne 0 ]
do
	savepids="$chkpid $savepids"
	chkpid=`ps --pid $chkpid -o ppid --no-headers`
done

for Pn in $KILLBINS
do
	# When stoping database as part of Disaster recovery, 
	# donot kill uarchive, restore_client and applianceAgent. Bug:10920
	if [ "$1" = no_uarchive ] 
	then
		if [ "$Pn" = uarchive ] || [ "$Pn" = restore_client ] || [ "$Pn" = applianceAgent ]
		then
			continue
		fi
	fi
	pid=`ps -C $Pn -o pid --no-headers`
	for Tn in $pid
	do
		save=0
		for savepid in $savepids
		do
			if [ "$Tn" -eq "$savepid" ]; then
				save=1
				break
			fi
		done
		[ "$save" -eq 1 ] && continue
		kill $Tn > /dev/null  2>&1
#		if [ $? -eq 0 ]; then
#			echo "  ${Pn} (pid $Tn) is being stopped" >>$LOG
#		fi
	done
done

# Handle autosynth separately.  We only want to kill the parent process,
# # then wait for it to exit (after the children have been cleaned up).
if pgrep -x autosynth >/dev/null 2>&1
then
	pkill -P 1 autosynth
	timeout=30
	sleeps=0
	while pgrep -x autosynth >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
	do
		((sleeps=$sleeps + 1))
	done
    if [ $kill9ok -eq 1 ]; then
	if pgrep -x autosynth >/dev/null 2>&1
	then
		pkill -9 autosynth
	fi
    fi
fi

# Handle securesync differently.  We only want to kill the parent process,
# then wait for it to exit (after the children have been cleaned up).
# 8972: Move the handling of securesync after the loop for stopping KILLBINS;
# gives more time for processes to clean up and exit (especially fileDedup).
if pgrep -x securesync >/dev/null 2>&1
then
	pkill -P 1 securesync
	timeout=30
	sleeps=0
	while pgrep -x securesync >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
	do
		# Pre-4.2 securesync would retry after relaying sigterm to bpsync,
		# so kill any continuing bpsync processes until the daemon exits.
		pkill bpsync
		((sleeps=$sleeps + 1))
	done
    if [ $kill9ok -eq 1 ]; then
	if pgrep -x securesync >/dev/null 2>&1 || pgrep -x bpsync >/dev/null 2>&1
	then
		# We don't want installation to fail
		pkill -9 securesync
		pkill -9 bpsync
	fi
    fi
fi

# 8972: Wait for fileDedup processes to exit.  Unlike securesync, we've
# signaled all of them since some may not be children of the daemon.
if pgrep -x fileDedup >/dev/null 2>&1
then
	timeout=30
	sleeps=0
	while pgrep -x fileDedup >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
	do
		((sleeps=$sleeps + 1))
	done
    if [ $kill9ok -eq 1 ]; then
	if pgrep -x fileDedup >/dev/null 2>&1
	then
		pkill -9 fileDedup
	fi
    fi
fi

#inlineHashdb's should already be killed by now. If not, kill now. 
if pgrep -x inlineHashdb >/dev/null 2>&1
then
	timeout=30
	sleeps=0
	while pgrep -x inlineHashdb >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
	do
		((sleeps=$sleeps + 1))
	done
    if [ $kill9ok -eq 1 ]; then
	pid=`ps -C inlineHashdb -o pid --no-headers`
	for Tn in $pid
	do
		kill -9 $Tn > /dev/null  2>&1
	done
    fi
fi

if pgrep -x devmonitor >/dev/null 2>&1
then
	if pgrep -x devmonitor >/dev/null 2>&1
	then
		pkill devmonitor
	fi
	timeout=30
	sleeps=0
	while pgrep -x devmonitor >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
	do
		((sleeps=$sleeps + 1))
	done
    if [ $kill9ok -eq 1 ]; then
	if pgrep -x devmonitor >/dev/null 2>&1
	then
		pkill -9 devmonitor
	fi
    fi
fi
$BPDIR/bin/cryptoDaemon -k >/dev/null 2>&1  # Suppress status message

if [ $BPONLY -eq 1 ]; then
  termPGBackend
  exit 0
fi

echo `date +%F.%H:%M:%S`"| Stopping database..."

dpids=`list_psD |awk '{ print $2 }'`
pgstat=$(pgUp)
if [ "x$dpids" != "x" ]; then
  # if hung, termPG and send another kill
  termPGBackend
  sleep 2
  for p in $dpids
  do
    kill $p
  done
elif [ $pgstat -ne 2 ]; then 
  termPGBackend
  sleep 1
else
  psql -Upostgres bpdb -c"checkpoint" > /dev/null 2>&1
  $STOPPG -w -m immediate >/dev/null 2>&1
  sleep 1
fi

# try immediate stop
psql -Upostgres bpdb -c"checkpoint" > /dev/null 2>&1
$STOPPG -w -m immediate >/dev/null
pids=`pgpids`

if [ -z "$pids" ]; then
	promptDone
	exit 0
fi

sleep 1
pids=`pgpids`
pgstat=$(pgUp)

if [ -z "$pids" ] && [ $pgstat -ne 2 ]; then
    for i in `seq $SLEEPTIME`
    do
	pids=`pgpids`
	if [ -z "$pids" ]; then
		promptDone
		exit 0
	fi
	echo -n '.'
	sleep 1
    done
fi

echo ''
pids=`pgpids`
if [ -z "$pids" ]; then
	promptDone
	exit 0
fi

# try a fast stop
if [ -f "$PIDFILE" ]
then
	psql -Upostgres bpdb -c"checkpoint" > /dev/null 2>&1
	$STOPPG -w -m fast >/dev/null
	if [ ! -f "$PIDFILE" ]
	then
		promptDone
		exit 0
	fi
fi

pgstat=$(pgUp)
if [ $pgstat -ne 2 ]; then
  # stop immediate whether or not PPID was found
  echo `date +%F.%H:%M:%S`"| Forced to send kill signal to postmaster..."
  psql -Upostgres bpdb -c"checkpoint" > /dev/null 2>&1
  $STOPPG -w -m immediate
fi
sleep 1

# kill the postmaster
# first, get the pid of postmaster
if [ -f "$PIDFILE" ]; then
	PARENTPID=`cat "$PIDFILE" | head -1`
else
	PARENTPID=`pgrep -P1 postmaster`
fi

# postgresql 9: postgres
if [ -z "$PARENTPID" ]; then
	PARENTPID=`pgrep -P1 postgres`
fi

# Sometimes there is no postmaster pidfile, so check if postgres is 
# still running, and if so kill children, then parent.
# Use safer kill -15 here, before failsafe kill -9 below.
pids=`pgpids`
sleep 1
if [ ! -z "$pids" ] && [ $pgstat -ne 2 ]; then
   echo `date +%F.%H:%M:%S`"| Sending kill to postgres processes ..."
   # kill children first, then parent
   for p in $pids; do
      if [ $p -ne $PARENTPID ]; then
         kill $p 2>/dev/null
      fi
   done
   # check in case parent stopped on its own during the loop (#17481)
   ps -ef | grep $PARENTPID |grep -v grep >/dev/null 2>&1
   if [ $? -eq 0 ]; then
      kill $PARENTPID
   fi
fi

sleep 1
if [ -f "$PIDFILE" ]; then
	PARENTPID=`cat "$PIDFILE" | head -1`
else
	PARENTPID=`pgrep -P1 postmaster`
fi
if [ -z "$PARENTPID" ]; then
	PARENTPID=`pgrep -P1 postgres`
fi

# wait for postgres to terminate
  timeout=30
  sleeps=0
  while pgrep "postmaster|postgres" >/dev/null 2>&1 && [[ $sleeps -lt $timeout ]] && sleep 1
  do
	((sleeps=$sleeps + 1))
  done

# force kill of children, then parent, if still running
if [ -n "$PARENTPID" ]
then
    if [ $kill9ok -eq 1 ]; then
	echo -n "killing postmaster (pid $PARENTPID) and child processes "
	for PID in `ps h --ppid "$PARENTPID" | awk '{print $1}'`; do
		echo -n "$PID "
		kill -9 $PID 2>/dev/null
	done;
	kill -9 $PARENTPID
	echo ''
    fi
	if [ -f "$PIDFILE" ]
	then
		rm -f "$PIDFILE"
	fi
fi

promptDone
exit 0
