SET search_path TO public;

CREATE OR REPLACE FUNCTION table_exists(sch_name VARCHAR,tbl_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
-- table names are case insensitive, but where VARCHAR compare function is
	RETURN ((SELECT COUNT(*) FROM pg_catalog.pg_tables WHERE
	lower(schemaname)=lower(sch_name) AND lower(tablename)=lower(tbl_name))>0);
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION sequence_exists(sch_name VARCHAR,seq_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
    RETURN ((
	SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n
    WHERE c.relkind = 'S'
    	AND c.relnamespace = n.oid
		AND lower(n.nspname) = lower(sch_name)
		AND lower(c.relname) = lower(seq_name)
	) > 0);
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION create_schemas()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
	sch VARCHAR[];
BEGIN
	res := 0;
-- assign more array elements here to add schemas
	sch[1] := 'bp';
	sch[2] := 'statussync';

	FOR i IN 1..array_upper(sch,1) LOOP
		IF (SELECT COUNT(*) FROM pg_catalog.pg_namespace WHERE nspname=sch[i])=0
		THEN
			RAISE NOTICE 'Creating % schema', sch[i];
			EXECUTE 'CREATE SCHEMA ' || quote_ident(sch[i]);
		END IF;
	END LOOP;

	RETURN res;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT create_schemas();
DROP FUNCTION create_schemas();


SET search_path TO bp,public;

CREATE OR REPLACE FUNCTION create_bp()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
BEGIN
	res:=0;

-- begin core tables
IF NOT table_exists('bp', 'nodes') THEN
	CREATE TABLE nodes (
		node_no INTEGER PRIMARY KEY,
		node_name VARCHAR(32) UNIQUE,
		type INTEGER,
		os INTEGER,
		vendor BIGINT,
		status BIGINT,
		last_master BIGINT,
		last_master_start_time BIGINT,
		last_incremental BIGINT,
		last_incremental_start_time BIGINT,
		last_selective BIGINT,
		last_selective_start_time BIGINT,
		last_baremetal BIGINT,
		last_baremetal_start_time BIGINT,
		priority INTEGER,
		options BIGINT,
		features BIGINT,
		fstatus BIGINT,
		properties BIGINT,
		bm_media_test_date BIGINT);
END IF;
		
IF NOT table_exists('bp', 'application_lookup') THEN
	-- This table identifies applications that
	--   o use the same client-DPU communications protocol
	--   o are presented to the customer as the same 'application type'
	--
	-- This table maps app_id to human readable
	--   o type of application (e.g. SQL Server)
	--   o specific application (e.g. SQL Server 2006, postgres
	--
	-- To add new applications:
	--   See application_instances for guidelines to add
	-- 
	-- app_id - applications that share the same app_id,
	--          share the same client-DPU protocol choices
	--          and so nominally execute the same DPU code
	--          and code branches.
	--          Any run-time DPU algorithm selection
	--          dependent on app_id must be independent
	--          of run-time data in the current message,
	--          and as feasible, should be independent
	--          of run-time data in prior messages.
	CREATE TABLE application_lookup (
		app_id INTEGER PRIMARY KEY,
		name VARCHAR NOT NULL,
		type VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'node_applications') THEN
	-- This table identifies applications that
	--   o were used for or are available for backup
	--   o ave available via agent on specified client (node_no)
	--
	-- app_id see application_lookup
	--
	-- current - true available for current backups
	--           false application is not availabe for current bakups,
	--                 though may become available in future,
	--                 so may be needed for restore
	--   o type of application (e.g. SQL Server)
	--   o specific application (e.g. SQL Server 2006, postgres
	--
	-- what applications are available on given node
	-- what nodes make given application available
	--
	-- node_no - identifies agent though which to communication
	--
	CREATE TABLE node_applications (
		node_no INTEGER NOT NULL REFERENCES nodes(node_no) ON DELETE CASCADE,
		app_id INTEGER NOT NULL REFERENCES application_lookup(app_id),
		current BOOLEAN NOT NULL DEFAULT TRUE,
		visible BOOLEAN NOT NULL DEFAULT TRUE,
		PRIMARY KEY (node_no, app_id));

	CREATE FUNCTION node_applications_invalidate_children() RETURNS TRIGGER AS $node_app_tag$
	BEGIN
		-- If current is set to false, all referring applications are no longer current
		IF NEW.current = FALSE THEN
			UPDATE bp.application_instances SET current = FALSE
			WHERE current != FALSE AND node_no = NEW.node_no AND app_id = NEW.app_id;
		END IF;
		RETURN NULL;
	END
	$node_app_tag$ LANGUAGE 'plpgsql';

	CREATE TRIGGER node_applications_update_current AFTER UPDATE ON node_applications
		FOR EACH ROW EXECUTE PROCEDURE node_applications_invalidate_children();
END IF;

IF NOT table_exists('bp', 'application_instances') THEN
	-- This table identifies all objects that
	--   o can be backed up
	--   o are bundled and unbundled by peculiar applications
	--     (i.e. not handled as files with ctar)
	--
	-- This table maps instance_id to
	--   o node_no+app_id[+key1[+key2[+key3[+key4]]]]
	--     unique object that can be backed up
        --     or has been backed up       
	--   o app_id. (see application_lookup)
	--   o node_no
	--     client with SQL agent for communication
	--     (application and data may actually reside elsewhere,
	--      but all communication from DPU is nominally through agent)
	--
	-- Note that once set for an instance_id, keys cannot be changed.
	--
	-- The meaning of each key is tabulated here,
        --   selected by application_lookup.type
        --   (or as necessary or equivalent) by app_id.
	--
	-- type        app_id keyN meaning
	-- file-level   1-  1     unused
	-- Exchange     2- 19   1 database/storage group
	-- SQL Server  21- 39   1 server
	--                      2 instance
	--                      3 database
	--
	-- To add new applications:
	--   the decision whether or not to add an app_id, key
	--   or property should maintain orthongonality
	--   (in order to minimize usage complexity)
	--   In general, add
	--     app_id to identify changes to client-DPU protocol
	--       e.g. new fields for new versions of an application
	--     key to distinguish items that can be backed up separately
	--       e.g. different products/targets of the same application
	--     prop to identify changes that may affect the same backup
	--       e.g. SQL Server recovery model
        --
	-- All application instances must refer to valid node_applications
	--   (enforced via ON DELETE CASCADE)
	--
	CREATE TABLE application_instances (
		instance_id SERIAL PRIMARY KEY NOT NULL,
		node_no INTEGER NOT NULL,
		app_id INTEGER NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		key1 VARCHAR,
		key2 VARCHAR,
		key3 VARCHAR,
		key4 VARCHAR,
		FOREIGN KEY (node_no, app_id) REFERENCES node_applications (node_no, app_id) ON DELETE CASCADE,
		UNIQUE (node_no, app_id, key1, key2, key3, key4));
END IF;

IF NOT table_exists('bp', 'application_properties') THEN
	-- This table maps instance_id to
	--   o application specific data that may change
        --
	-- key selects data to
        --   o return to application with restore, query, ...
 	--   o select options related to restore, query, ...
	--
	-- The meaning and precise string of each key is tabulated here,
        --   selected by application_lookup.type
        --   (or as necessary or equivalent) by app_id.
        --
	--  type      app_id prop_key prop_value notes
	-- file-level  1-  1 unused
	-- Exchange    2- 19 unused
	-- SQL Server 21- 39 recovery_model
	--                            full       full
	--                            simple     simple
	--                   online   
	--                     TRUE (or absent) available for backup
	--                     FALSE backup may fail
	--
	-- All application properties must refer to valid application_instances
	--   (enforced via ON DELETE CASCADE)
	--
	CREATE TABLE application_properties (
		instance_id INTEGER,
		prop_key VARCHAR,
		prop_value VARCHAR,
		FOREIGN KEY (instance_id)
			REFERENCES application_instances (instance_id)
			ON DELETE CASCADE
	);
END IF;

IF NOT table_exists('bp', 'jobs') THEN
	CREATE TABLE jobs (
		job_no BIGINT PRIMARY KEY,
		batch_no BIGINT,
		log_name VARCHAR(32),
		node_no INTEGER,
		node_group INTEGER,
		start_dir_no BIGINT,
		x_command VARCHAR(512),
		queued_date BIGINT,
		time_scheduled_for BIGINT,
		start_time BIGINT,
		elapsed_time INTEGER,
		status BIGINT,
		cmd_before VARCHAR(80),
		cmd_after VARCHAR(80),
		remote_cmd_before VARCHAR(80),
		remote_cmd_after VARCHAR(80),
		tape_no BIGINT,
		fileset_no BIGINT,
		device_no INTEGER,
		attempt_cnt INTEGER,
		time_stamp BIGINT,
		pid BIGINT,
		backup_no BIGINT,
		password BIGINT,
		blocksize INTEGER,
		verify_after INTEGER,
		jdt INTEGER,
		dependant_jobno BIGINT,
		comment VARCHAR(80),
		lcs_no INTEGER,
		percent_done INTEGER,
		operator_name VARCHAR(100),
		dependant_backupno BIGINT,
		jlast_message VARCHAR(248),
		instance_id INTEGER REFERENCES bp.application_instances (instance_id) ON DELETE SET NULL,
		schedule_id INTEGER);
END IF;

IF NOT table_exists('bp', 'storage') THEN
        CREATE TABLE storage (
                storage_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
                storage_name VARCHAR(32) UNIQUE,
                storage_type VARCHAR,
                is_online BOOLEAN);
        ALTER SEQUENCE storage_storage_id_seq RESTART WITH 2;
        INSERT INTO storage (storage_id, storage_name, storage_type, is_online)
          VALUES (1, 'Internal', 'internal', true);
END IF;

IF NOT table_exists('bp', 'iscsi') THEN
        CREATE TABLE iscsi (
                storage_id INTEGER NOT NULL,
                address VARCHAR NOT NULL,
                port VARCHAR NOT NULL,
                target VARCHAR NOT NULL,
                lun VARCHAR NOT NULL,
                uname VARCHAR,
                password VARCHAR,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                PRIMARY KEY (address, port, target, lun));
END IF;

IF NOT table_exists('bp', 'nas') THEN
        CREATE TABLE nas (
                storage_id INTEGER NOT NULL,
                address VARCHAR NOT NULL,
                port VARCHAR NOT NULL,
                protocol VARCHAR NOT NULL,
                uname VARCHAR,
                password VARCHAR,
                share_name VARCHAR,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                PRIMARY KEY (address, port, protocol));
END IF;

IF NOT table_exists('bp', 'internal') THEN
        CREATE TABLE internal (
                storage_id INTEGER NOT NULL,
                disk_id VARCHAR NOT NULL,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                PRIMARY KEY (disk_id));
END IF;

IF NOT table_exists('bp', 'devices') THEN
	CREATE TABLE devices (
		device_no INTEGER PRIMARY KEY,
		dev_name VARCHAR(32) UNIQUE,
		device_type INTEGER,
		node_no INTEGER,
		capacity BIGINT,
		blocksize INTEGER,
		cur_fileset INTEGER,
		comment VARCHAR(80),
		status BIGINT,
		cur_k_position BIGINT,
		time_stamp BIGINT,
		scsi_blksize INTEGER,
		is_scsi BOOLEAN,
		dev_rw_name VARCHAR(32),
		dev_nr_name VARCHAR(32),
		dev_cmd_name VARCHAR(32),
		pid BIGINT,
		same_as_devices VARCHAR(32),
		auto_switch VARCHAR(32),
		tape_class CHAR(8),
		reservation_pid BIGINT,
		scsi_caps BIGINT,
		scsi_id INTEGER,
		scsi_lun INTEGER,
		scsi_ha INTEGER,
		tested BOOLEAN,
		drive_level INTEGER,
		fd INTEGER,
		tape_no BIGINT,
		properties BIGINT,
		juke_drive_no INTEGER,
		no_label BOOLEAN,
		mux_maxcnt INTEGER,
		mux_sched INTEGER,
		mux_running INTEGER,
		is_online BOOLEAN,
		slot_no INTEGER,
		dev_changer_name VARCHAR(30),
		use_juke BOOLEAN,
		juke_libno INTEGER,
		comment_short VARCHAR(44),
		total_write_usages BIGINT,
		total_write_time BIGINT,
		backup_no BIGINT,
		juke_slot_info INTEGER,
		juke_node_no INTEGER,
		is_deleted BOOLEAN NOT NULL DEFAULT FALSE,
		dedup BOOLEAN NOT NULL DEFAULT TRUE,
	 	storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id));
END IF;

IF NOT table_exists('bp', 'tapes') THEN
	CREATE TABLE tapes (
		tape_no BIGINT PRIMARY KEY,
		backup_no BIGINT,
		label VARCHAR(80),
		status BIGINT,
		media_pool INTEGER,
		magazine_no INTEGER,
		expiration_date BIGINT,
		soft_errors INTEGER,
		hard_errors INTEGER,
		total_uncompressed_data INTEGER,
		total_compressed_data INTEGER,
		total_megs BIGINT,
		location VARCHAR(80),
		capacity BIGINT,
		when_labelled BIGINT,
		usages INTEGER,
		scsi_blksize INTEGER,
		physical_fsno BIGINT,
		juke_slot INTEGER,
		juke_slot_info INTEGER,
		juke_drive_no INTEGER,
		juke_libno INTEGER,
		tape_class CHAR(8),
		sets_of_data BIGINT,
		blocksize INTEGER,
		scratch_cnt INTEGER,
		import_no BIGINT,
		no_label BOOLEAN);
END IF;
		
IF NOT table_exists('bp', 'backups') THEN
	CREATE TABLE backups (
		backup_no BIGINT PRIMARY KEY,
		attempt_cnt INTEGER,
		node_no INTEGER,
		start_time BIGINT,
		tape_no BIGINT,
		job_no BIGINT UNIQUE,
		fileset_no BIGINT,
		elapsed_time BIGINT,
		server_name VARCHAR(32),
		operator_name VARCHAR(100),
		properties BIGINT,
		total_netpaths BIGINT,
		total_megs BIGINT,
		multivolume INTEGER,
		sync_status INTEGER,
		sync_resv INTEGER,
		status BIGINT,
		start_dir_no BIGINT,
		incomplete INTEGER,
		notbacked INTEGER,
		type INTEGER,
		verify_after INTEGER,
		is_verified BOOLEAN,
		exit_code BIGINT,
		total_kilo BIGINT,
		blocksize INTEGER,
		lcs_no INTEGER,
		x_command_short VARCHAR(256),
		all_filesets_present BOOLEAN,
		pid BIGINT,
		data_bkno BIGINT);
		-- Also see backups.instance_id, added in update_bpdb.sql
END IF;
		
IF NOT table_exists('bp', 'filesets') THEN
	CREATE TABLE filesets (
		fileset_no BIGINT PRIMARY KEY,
		ssn BIGINT,
		vol_no BIGINT,
		tape_no BIGINT, 
		start_time BIGINT,
		elapsed_time BIGINT,
		total_megs BIGINT,
		fileset_megs BIGINT,
		total_netpaths BIGINT,
		server_name VARCHAR(32),
		tape_drive_name VARCHAR(32),
		status BIGINT,
		properties BIGINT,
		physical_fsno BIGINT,
		v_koffset BIGINT,
		btotal BIGINT,
		tape_dataset BIGINT,
		backup_no BIGINT,
		job_no BIGINT,
		data_format CHAR(5),
		is_multiplexed BOOLEAN,
		is_encrypted BOOLEAN,
		is_compressed BOOLEAN,
		no_label BOOLEAN,
		scsi_blksize INTEGER,
		os INTEGER,
		ssn_blocksize INTEGER,
		mux_fileset_no BIGINT,
		import_no BIGINT,
		guid BIGINT,
		device_properties BIGINT);
END IF;

IF NOT table_exists('bp', 'backup_snapshots') THEN
	CREATE TABLE backup_snapshots (
		backup_no BIGINT PRIMARY KEY
			REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		snapshot_time INTEGER NOT NULL);
END IF;

IF NOT table_exists('bp', 'sql_backups') THEN
	CREATE TABLE sql_backups (
		sql_ref BIGINT PRIMARY KEY,
		node_no INTEGER,
		database_name VARCHAR(128),
		server_instance_name VARCHAR(32),
		group_file_order BIGINT,
		file_order_no BIGINT,
		sql_type INTEGER);
END IF;

IF NOT table_exists('bp', 'backup_msg') THEN
	CREATE TABLE backup_msg (
		backup_no BIGINT PRIMARY KEY,
		messages VARCHAR,
		comment VARCHAR(480));
END IF;

IF NOT table_exists('bp', 'operation_names') THEN
  -- names of operations that support progress tracking
  --
  -- Operations for which support exists to track progress are
  --   herein identified by name.
  -- Many instances of each operation may be simultaneously in progress.
  CREATE TABLE operation_names (
    name_id SERIAL PRIMARY KEY NOT NULL UNIQUE, -- faster in join than CHAR()
    name_value VARCHAR(32) NOT NULL UNIQUE);    -- feature identifier
    -- additional columns (e.g. peculiar table names)
    --   may be added to simplify coordinating C/PHP code.
	-- vaulting
  --   operation_progress.current_size - cursor position in input
  --   operation_progress.final_size   - length of input
  INSERT INTO operation_names(name_value) VALUES ('delta');
  INSERT INTO operation_names(name_value) VALUES ('signature');
  INSERT INTO operation_names(name_value) VALUES ('patch');
  INSERT INTO operation_names(name_value) VALUES ('copy');
  INSERT INTO operation_names(name_value) VALUES ('migrate');
    -- As features become newly trackable, insert new choices above
    --   and alter TABLE operation_progress if necessary.
END IF;

IF NOT table_exists('bp', 'operation_progress') THEN
  -- last status of an operation instance in progress
  --
  -- Bibliography:
  --   Backup Professional Library (bpl.so) ABP Guide Version V
  --     Operation APIs
  --       bp_get_operation_list()
  --
  -- Note:
  --   Column snapshot_time corresponds to API index current_time
  --     but was renamed to avoid namespace conflict with
  --     builtin postgres function current_time
  --
  -- Writer (run-time)
  -- To start an operation whose progress may be tracked:
  --   INSERT an SQL record with new operation string and initial data.
  -- During operation, UPDATE (explicit columns)
  --   Current_size and final_size are instantaneous values. 
  -- Upon completion, DELETE record.
  --
  -- Reader (run-time)
  -- To determine average throughput and/or estimate completion time:
  --   poll instantaneous values, keep history in memory, and smooth.
  -- Completion status must be determined by alternate means,
  --   as record is deleted upon completion.
  --
  -- To accomodate future long duration actions (e.g. database cleanup)
	--   (between releases)
	--     IF NOT column_exists('bp', 'operation_progress', 'new_field') THEN
  --       ALTER TABLE operation_progress ADD COLUMN new_field ...
	--     END IF;
  --   (with new release)
  --      Add columns here and remove from update_bpdb.sql.
  CREATE TABLE operation_progress (
    progress_id SERIAL PRIMARY KEY NOT NULL UNIQUE, -- instance identifier
    name_id INTEGER REFERENCES operation_names(name_id) NOT NULL,
          -- operation name
    start_time    BIGINT NOT NULL, -- operation began seconds since epoch
    snapshot_time BIGINT,          -- last update seconds since epoch
    final_size    BIGINT,          -- 0, units in 100%,
                                   --    e.g. completion bytecount,
    current_size  BIGINT,          -- 0, units out of final_size,
    pid_to_kill   BIGINT DEFAULT 0,
    on_behalf_of  VARCHAR(256),    -- who is watching
    source        VARCHAR(256),    -- sender client/DPU/DPV/...
    destination   VARCHAR(256),	   -- receiver client/DPU/DPV/...
         -- backup related information
    backup_no     BIGINT,          -- nominally backups(backup_no)
    backup_type   VARCHAR(32));    -- nominally (mapped) backups(type)
END IF;

IF NOT table_exists('bp', 'operation_status') THEN
-- Unlike operation_progess, these records persist so that an operation that
-- should only occur once can leave evidence that is has been done.  For
-- example, a migration may need to take place on upgrade, or the first time
-- some subsystem runs after upgrade, but should never be performed again.
-- This table can be checked to see if the operation is pending, done, etc.
  CREATE TABLE operation_status (
    operation_id INTEGER PRIMARY KEY NOT NULL UNIQUE,
    subsystem    VARCHAR NOT NULL,
    description  VARCHAR NOT NULL,
    status       VARCHAR NOT NULL
  );
END IF;

		
IF NOT table_exists('bp', 'hash_values') THEN
  -- This hash_values table is used by file deduplication to map a
  -- hash value and its attributes to a single, more managable value --
  -- its hash_id.  The database is responsible for and performs the
  -- mapping for us.  If we could do this quicker outside of the database
  -- we should.
  CREATE TABLE hash_values (
    hash_id		BIGSERIAL PRIMARY KEY,
    hash_type		VARCHAR(8) NOT NULL,
    hashed_length	BIGINT NOT NULL,
    hash_value 		BYTEA,
    UNIQUE (hash_type, hashed_length, hash_value)
  );
END IF;
		
IF NOT table_exists('bp', 'backups_hash_values') THEN
  -- This backups_hash_values table associates a file hashed for the
  -- purposes of file deduplication in a backup with its hash value and
  -- hash attributes.
  CREATE TABLE backups_hash_values (
    backup_no		BIGINT REFERENCES bp.backups (backup_no)
				    ON DELETE CASCADE ON UPDATE CASCADE,
    hash_id		BIGINT REFERENCES bp.hash_values (hash_id)
				    ON DELETE RESTRICT ON UPDATE RESTRICT,
    file_offset		BIGINT NOT NULL,
    deduplicated	BIGINT DEFAULT NULL,
    hash_incomplete	BOOLEAN DEFAULT TRUE,
    UNIQUE (backup_no, hash_id, file_offset)
  );

  CREATE INDEX backups_hash_values_hash_id_index ON bp.backups_hash_values (hash_id);
END IF;
		
IF NOT table_exists('bp', 'dedup_stats') THEN
  -- This dedup_stats table records some global statistics about our
  -- deduplication operations.
  CREATE TABLE dedup_stats (
    operation		VARCHAR(8) NOT NULL,
    done_epoch		BIGINT NOT NULL,
    bytes_processed	BIGINT NOT NULL,
    elapsed_seconds	INTEGER NOT NULL
  );
END IF;

IF table_exists('bp', 'data_reduction_stats') THEN
  -- First version of the table is obsolete
  DROP TABLE bp.data_reduction_stats;
END IF;

IF NOT table_exists('bp', 'data_reduction_ratios') THEN
  -- This records the data reduction ratios for the system.
  CREATE TABLE data_reduction_ratios (
    sample_time  BIGINT PRIMARY KEY,
    dedup        DECIMAL NOT NULL,
    compr        DECIMAL NOT NULL,
    alldr        DECIMAL NOT NULL
  );
END IF;

		
IF NOT table_exists('bp', 'lcs') THEN
	CREATE TABLE lcs (
		lcs_no INTEGER PRIMARY KEY,
		is_online BOOLEAN,
		start_time BIGINT, 
		sched_name VARCHAR(80) UNIQUE NOT NULL,
		device_no INTEGER,
		status BIGINT,
		last_start_time BIGINT,
		last_end_time BIGINT,
		total_netpaths BIGINT,
		total_megs BIGINT,
		job_no BIGINT, 
		failure_report_is_online BOOLEAN,
		failure_report_notification_type INTEGER,
		backup_summary_is_online BOOLEAN,
		backup_summary_notification_type INTEGER,
		weekly_summary_is_online BOOLEAN,
		weekly_summary_notification_type INTEGER,
		cmd_before VARCHAR (80),
		cmd_after VARCHAR (80),
		server_name VARCHAR (32),
		last_message VARCHAR (256),
		working_element INTEGER,
		is_recurring BOOLEAN);
END IF;
		
IF NOT table_exists('bp', 'lcs_client') THEN
	CREATE TABLE lcs_client (
		lcs_no INTEGER,
		node_no INTEGER,
		type INTEGER,
		freq INTEGER,
		element_freq INTEGER,
		seq_no INTEGER,
		profile_path VARCHAR(256),
		is_online BOOLEAN,
		comment VARCHAR(80),
		status BIGINT,
		last_job BIGINT,
		last_job_status BIGINT,
		last_backup BIGINT,
		last_verify_job BIGINT,
		last_verify_status BIGINT,
		last_verify_backup BIGINT,
		last_message VARCHAR(256),
		retries_cnt INTEGER,
		last_queued_time BIGINT,
		use_juke BOOLEAN,
		verify_after INTEGER,
		priority INTEGER,
		sub_type INTEGER,
		PRIMARY KEY (lcs_no, node_no, type, freq, element_freq, seq_no));
END IF;
		
IF NOT table_exists('bp', 'lcs_hist') THEN
	CREATE TABLE lcs_hist (
		lcs_no INTEGER,
		start_time BIGINT,
		node_no INTEGER,
		job_no BIGINT,
		last_verify_job BIGINT,
		backup_no BIGINT,
		last_verify_backup BIGINT,
		status BIGINT,
		verify_after INTEGER,
		type INTEGER,
		comment VARCHAR(80),
		PRIMARY KEY (lcs_no, start_time, node_no, job_no));
END IF;

IF NOT table_exists('bp', 'library') THEN
	CREATE TABLE library (
		total_netpaths BIGINT,
		location VARCHAR(80),
		dispatcher INTEGER,
		library_no INTEGER PRIMARY KEY,
		dispatch_pid BIGINT,
		comment VARCHAR(80),
		unref_netpaths BIGINT,
		purge_status BIGINT,
		f_master_version BIGINT,
		lcs_client_version BIGINT);
END IF;
		
IF NOT table_exists('bp', 'lookups') THEN
	CREATE TABLE lookups (
		lookup_type BIGINT,
		lookup_no BIGINT,
		lookup_str VARCHAR(80),
		misc1 BIGINT,
		PRIMARY KEY (lookup_type, lookup_no));
END IF;
		
IF NOT table_exists('bp', 'pids') THEN
	CREATE TABLE pids (
		pid BIGINT PRIMARY KEY,
		filename VARCHAR(128),
		operator_name VARCHAR(100),
		comment VARCHAR(80),
		misc1 BIGINT,
		time_stamp BIGINT);
END IF;
-- end of core tables

-- begin alertman tables
IF NOT table_exists('bp', 'alert_source') THEN
	CREATE TABLE alert_source (
		source_id SERIAL PRIMARY KEY UNIQUE,
		source_name VARCHAR(35) NOT NULL);
END IF;
		
IF NOT table_exists('bp', 'alerts') THEN
	CREATE TABLE alerts (
		alert_id SERIAL PRIMARY KEY UNIQUE,
		alert_text TEXT,
		source_id INTEGER REFERENCES alert_source(source_id),
		target TEXT,
		severity INTEGER,
		created BIGINT,
		updated BIGINT,
		closed BOOLEAN DEFAULT FALSE);
END IF;
		
IF NOT table_exists('bp', 'temperature') THEN
	CREATE TABLE temperature (
		temp_id SERIAL PRIMARY KEY UNIQUE,
		time_stamp BIGINT,
		cpu1 DECIMAL,
		cpu2 DECIMAL,
		sys DECIMAL);
END IF;
		
IF NOT table_exists('bp', 'pci_table') THEN
	CREATE TABLE pci_table (
		bus_id VARCHAR(7) PRIMARY KEY UNIQUE,
		class_id INTEGER,
		vendor_id INTEGER,
		device_id INTEGER,
		device_name TEXT);
END IF;
		
IF NOT table_exists('bp', 'alert_runs') THEN
	CREATE TABLE alert_runs (
		run_id SERIAL PRIMARY KEY UNIQUE,
		started BIGINT,
		ended BIGINT);
END IF;
		
IF NOT table_exists('bp', 'inactive_clients') THEN
	CREATE TABLE inactive_clients (
		name TEXT PRIMARY KEY UNIQUE,
		time_stamp BIGINT);
END IF;
		
IF NOT table_exists('bp', 'disk_status') THEN
	CREATE TABLE disk_status (
		status_id INTEGER PRIMARY KEY UNIQUE,
		status TEXT);
END IF;
		
IF NOT table_exists('bp', 'unit_table') THEN
	CREATE TABLE unit_table (
		controller INTEGER,
		unit INTEGER,
		type TEXT,
		status_id INTEGER REFERENCES disk_status(status_id));
END IF;
		
IF NOT table_exists('bp', 'drive_table') THEN
	CREATE TABLE drive_table (
		controller INTEGER,
		unit INTEGER,
		port INTEGER,
		status_id INTEGER REFERENCES disk_status(status_id));
END IF;

IF NOT table_exists('bp', 'raid_units') THEN
	CREATE TABLE raid_units (
		controller VARCHAR(128) NOT NULL,
		unit VARCHAR(128) PRIMARY KEY UNIQUE,
		type VARCHAR(128) NOT NULL,
		uuid VARCHAR(128) NOT NULL,
		status INTEGER REFERENCES disk_status(status_id) NOT NULL
	);
END IF;

IF NOT table_exists('bp', 'raid_drives') THEN
	CREATE TABLE raid_drives (
		controller VARCHAR(128) NOT NULL,
		device VARCHAR(128) PRIMARY KEY UNIQUE,
		status INTEGER REFERENCES disk_status(status_id) NOT NULL,
		serial VARCHAR(128) NOT NULL,
		partition INTEGER NOT NULL DEFAULT 0
	);
END IF;
-- end of alertman tables

-- begin Notification tables
IF NOT table_exists('bp', 'notification_mgmt') THEN
	CREATE TABLE notification_mgmt(
		revision INTEGER PRIMARY KEY,
		upgrade_from INTEGER NOT NULL,
		status VARCHAR NOT NULL);
	INSERT INTO notification_mgmt(revision, upgrade_from, status)
		VALUES (0, 0, 'needs migration');
END IF;

IF NOT table_exists('bp', 'categories') THEN
	CREATE TABLE categories (
		category_id INTEGER PRIMARY KEY,
		category VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'notifications') THEN
	CREATE TABLE notifications(
		notification_id INTEGER PRIMARY KEY,
    	category_id INTEGER REFERENCES categories(category_id) NOT NULL,
    	short_description VARCHAR NOT NULL,
    	primary_text VARCHAR NOT NULL,
    	severity INTEGER NOT NULL,
    	state BOOLEAN NOT NULL,
    	action VARCHAR,
    	alert BOOLEAN NOT NULL,
    	trap BOOLEAN NOT NULL,
    	email BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'notify_alerts') THEN
	CREATE TABLE notify_alerts(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
		source_id INTEGER REFERENCES alert_source(source_id) NOT NULL);
END IF;

IF NOT table_exists('bp', 'trap_types') THEN
	CREATE TABLE trap_types(
		trap_type_id INTEGER PRIMARY KEY,
		trap_type VARCHAR NOT NULL);

END IF;

IF NOT table_exists('bp', 'notify_traps') THEN
	CREATE TABLE notify_traps(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
    	trap_type_id INTEGER REFERENCES trap_types(trap_type_id) NOT NULL,
		oid VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'notify_emails') THEN
	CREATE TABLE notify_emails(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
	--  The section/keyword to look up in master.ini for the recipient list
		to_section VARCHAR NOT NULL,
		to_keyword VARCHAR NOT NULL,
	--  from_name is just the from_name part of from_name@dpu.domain.com
		from_name VARCHAR NOT NULL,
		subject VARCHAR NOT NULL,
		body VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'audit_history') THEN
	CREATE TABLE audit_history (
	id SERIAL PRIMARY KEY,
	notification_id INTEGER NOT NULL,
	event_time TIMESTAMP DEFAULT now(),
	username VARCHAR NOT NULL,
	system VARCHAR NOT NULL,
	message VARCHAR NOT NULL);
END IF;
-- end of Notification tables

-- begin SDS stats tables
IF NOT table_exists('bp', 'sync_type') THEN
	CREATE TABLE sync_type (
		type_id SERIAL PRIMARY KEY NOT NULL UNIQUE,
		type_name VARCHAR(32) NOT NULL);
END IF;

IF NOT table_exists('bp', 'sds_stats') THEN
	CREATE TABLE sds_stats (
		client_name VARCHAR(32) NOT NULL,
		backup_no BIGINT NOT NULL,
		sync_type INTEGER REFERENCES sync_type(type_id) NOT NULL,
		presync_seconds INTEGER,
		sync_seconds INTEGER,
		postsync_seconds INTEGER,
		done_epoch BIGINT NOT NULL,
		bytes_on_dpu BIGINT,
		bytes_syncd BIGINT,
		bytes_on_dpv BIGINT,
		message text NOT NULL,
		stat_time TIMESTAMP DEFAULT now());
END IF;

-- Records in this table block securesync from starting a new operation for
-- the given client and syncInfo.syncType while a cleanup is occurring.
IF NOT table_exists('bp', 'securesync_guard') THEN
	CREATE TABLE securesync_guard (
		client_name VARCHAR NOT NULL,
		syncinfo_type VARCHAR NOT NULL,
		guard_time TIMESTAMP DEFAULT now(),
		PRIMARY KEY (client_name, syncinfo_type));
END IF;
-- end of SDS stats tables

-- begin security tables
IF NOT table_exists('bp', 'customers') THEN
	CREATE TABLE customers (
		customer_id SERIAL PRIMARY KEY,
		name VARCHAR(128) NOT NULL);
END IF;
	
IF NOT table_exists('bp', 'locations') THEN
	CREATE TABLE locations (
		location_id SERIAL PRIMARY KEY,
		customer_id INTEGER REFERENCES customers (customer_id) ON DELETE CASCADE,
		name VARCHAR(128) NOT NULL,
		latitude NUMERIC,
		longitude NUMERIC);
END IF;
	
IF NOT table_exists('bp', 'systems') THEN
	CREATE TABLE systems (
		system_id SERIAL PRIMARY KEY,
		location_id INTEGER REFERENCES locations (location_id) ON DELETE SET NULL DEFAULT NULL,
		name VARCHAR(128) NOT NULL,
		is_local_system BOOLEAN NOT NULL DEFAULT FALSE,
		host TEXT,
		version VARCHAR NOT NULL DEFAULT 'Unknown',
		role VARCHAR NOT NULL,
		is_active BOOLEAN NOT NULL DEFAULT TRUE,
		check_timestamp BIGINT NOT NULL DEFAULT 0,
	 	storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id),
		CONSTRAINT systems_unique_host UNIQUE (host, is_local_system));
END IF;
	
IF NOT table_exists('bp', 'users') THEN
	CREATE TABLE users (
		user_id SERIAL PRIMARY KEY,
		username VARCHAR(64) UNIQUE,
		password VARCHAR(64) NOT NULL);
END IF;
	
IF NOT table_exists('bp', 'priv_lookup') THEN
	CREATE TABLE priv_lookup (
		priv_level INTEGER PRIMARY KEY,
		priv_name VARCHAR(64) NOT NULL,
		priv_description TEXT);
END IF;
	
IF NOT table_exists('bp', 'superusers') THEN
	CREATE TABLE superusers (
		user_id INTEGER PRIMARY KEY REFERENCES users (user_id) ON DELETE CASCADE
	);
END IF;
	
IF NOT table_exists('bp', 'customer_priv') THEN
	CREATE TABLE customer_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		customer_id INTEGER NOT NULL REFERENCES customers (customer_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, customer_id));
END IF;
	
IF NOT table_exists('bp', 'location_priv') THEN
	CREATE TABLE location_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		location_id INTEGER NOT NULL REFERENCES locations (location_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, location_id));
END IF;
	
IF NOT table_exists('bp', 'system_priv') THEN
	CREATE TABLE system_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		system_id INTEGER NOT NULL REFERENCES systems (system_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, system_id));
END IF;
-- end of security tables

IF NOT table_exists('bp', 'async_query_job') THEN
	CREATE TABLE async_query_job (
		real_pid INTEGER,
		timestr VARCHAR,
		fork_pid INTEGER,
		status VARCHAR,
		temptable_name VARCHAR,
		PRIMARY KEY (real_pid, timestr));
	
	-- Create the trigger function for this table (this only happens on the first
	-- time this table is added to the schema).  
	-- NOTE: if this ever needs to be modified on an upgrade, put the new version
	-- in create_functions.sql.
	CREATE OR REPLACE FUNCTION bp.DropTmpfilesearchTbl() RETURNS TRIGGER AS
	$async_tbl_tag$
	DECLARE
		cmd VARCHAR;
		tbl VARCHAR;
	BEGIN 
		SELECT INTO tbl relname FROM pg_catalog.pg_class WHERE relname = OLD.temptable_name ;
		IF tbl IS NULL
		THEN
	--		RAISE NOTICE 'table not found: %', OLD.temptable_name;
			RETURN NULL;
		ELSE
	--		RAISE NOTICE 'dropping table: %', OLD.temptable_name ;
			cmd := 'DROP TABLE ' || OLD.temptable_name ;
			EXECUTE cmd;
			RETURN OLD;
		END IF;
	END
	$async_tbl_tag$
	LANGUAGE 'plpgsql' VOLATILE;

	-- Now define the trigger
	CREATE TRIGGER drop_tmp_tbl_trig AFTER DELETE ON async_query_job FOR EACH ROW
	EXECUTE PROCEDURE DropTmpfilesearchTbl();

END IF;

IF NOT table_exists('bp', 'nvp') THEN
	CREATE TABLE nvp (
		nvp_type VARCHAR NOT NULL,
		nvp_name VARCHAR NOT NULL,
		item_name VARCHAR NOT NULL,
		item_value VARCHAR,
		PRIMARY KEY (nvp_type, nvp_name, item_name));
END IF;

IF NOT table_exists('bp', 'node_volumes') THEN
	CREATE TABLE node_volumes (
		node_no INTEGER NOT NULL REFERENCES nodes (node_no) ON DELETE CASCADE,
		volume VARCHAR NOT NULL,
		CONSTRAINT node_volumes_unique_volume UNIQUE (node_no, volume));
END IF;

-- begin scheduling tables
IF NOT table_exists('bp', 'selection_lists') THEN
	CREATE TABLE selection_lists (
		selection_list_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		type VARCHAR NOT NULL,
		family VARCHAR NOT NULL,
		node_no INTEGER,
		read_only BOOLEAN NOT NULL,
		netmounts BOOLEAN NOT NULL,
		registry BOOLEAN NOT NULL,
		tempfiles BOOLEAN NOT NULL,
		readmounts BOOLEAN NOT NULL,
		allmounts BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'selection_list_filenames') THEN
	CREATE TABLE selection_list_filenames (
		selection_list_id INTEGER NOT NULL REFERENCES selection_lists (selection_list_id) ON DELETE CASCADE,
		filename VARCHAR NOT NULL);

		CREATE INDEX selection_list_filenames_selection_list_id ON bp.selection_list_filenames(selection_list_id);
END IF;

IF NOT table_exists('bp', 'option_lists') THEN
	CREATE TABLE option_lists (
		option_list_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		type VARCHAR NOT NULL,
		family VARCHAR NOT NULL,
		read_only BOOLEAN NOT NULL,
		device_name VARCHAR REFERENCES devices (dev_name) ON DELETE SET NULL ON UPDATE CASCADE,
		read_locking INTEGER NOT NULL,
		speed_option BOOLEAN NOT NULL,
		verify_level INTEGER NOT NULL,
		depth INTEGER NOT NULL,
		comment VARCHAR,
		raw_device VARCHAR,
		raw_device_command VARCHAR,
		before_command VARCHAR(79),
		after_command VARCHAR(79),
		make_catalog BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'strategies') THEN
	CREATE TABLE strategies (
		strategy_id SERIAL PRIMARY KEY,
		name VARCHAR UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		read_only BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'calendars') THEN
	CREATE TABLE calendars (
		calendar_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		family VARCHAR NOT NULL,
		read_only BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'events') THEN
  -- repeatable points in time
  --
	CREATE TABLE events (
		event_id SERIAL PRIMARY KEY,
		calendar_id INTEGER REFERENCES calendars (calendar_id) ON DELETE CASCADE,
		strategy_id INTEGER REFERENCES strategies (strategy_id) ON DELETE CASCADE,
		summary INTEGER,
		dtstart BIGINT,      -- start of freq unit earlier than first occurance
		freq INTEGER,        -- include/bp/bpstatus.h: Logical Client Set Backup
		interval INTEGER,    -- freq units (days, weeks, ...) between repetitions
		bymonthday INTEGER,
		description VARCHAR,
		CONSTRAINT event_has_parent CHECK (calendar_id IS NOT NULL OR strategy_id IS NOT NULL),
		CONSTRAINT event_has_one_parent CHECK (calendar_id IS NULL OR strategy_id IS NULL));
END IF;

IF NOT table_exists('bp', 'byday') THEN
	CREATE TABLE byday (
		event_id INTEGER NOT NULL REFERENCES events (event_id) ON DELETE CASCADE,
		day_no INTEGER NOT NULL,
		CONSTRAINT byday_unique_day_no UNIQUE (event_id, day_no));
END IF;

IF NOT table_exists('bp', 'byhour') THEN
	CREATE TABLE byhour (
		event_id INTEGER NOT NULL REFERENCES events (event_id) ON DELETE CASCADE,
		hour_no INTEGER NOT NULL,
		CONSTRAINT byhour_unique_hour_no UNIQUE (event_id, hour_no));
END IF;

IF NOT table_exists('bp', 'schedules') THEN
	CREATE TABLE schedules (
		schedule_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		calendar_id INTEGER REFERENCES calendars (calendar_id),
		run_only BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF NOT table_exists('bp', 'schedule_lcs') THEN
	CREATE TABLE schedule_lcs (
		schedule_id INTEGER NOT NULL REFERENCES schedules (schedule_id),
		lcs_no INTEGER NOT NULL REFERENCES lcs (lcs_no) DEFERRABLE INITIALLY DEFERRED);
END IF;

IF NOT table_exists('bp', 'lcs_client_assoc') THEN
	CREATE TABLE lcs_client_assoc (
		lcs_no INTEGER,
		node_no INTEGER,
		type INTEGER,
		option_list_id INTEGER REFERENCES option_lists (option_list_id),
		selection_incl_id INTEGER REFERENCES selection_lists (selection_list_id),
		selection_excl_id INTEGER REFERENCES selection_lists (selection_list_id),
		PRIMARY KEY (lcs_no, node_no, type));
END IF;
-- end scheduling tables

IF NOT table_exists('bp', 'reports') THEN
	CREATE TABLE reports (
		report_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		based_on VARCHAR,
		fields VARCHAR(65536));
END IF;

IF NOT table_exists('bp', 'bpcleanup_stats') THEN
	CREATE TABLE bpcleanup_stats (
		node_no INTEGER PRIMARY KEY REFERENCES nodes (node_no) ON DELETE CASCADE,
		last_clean_time BIGINT DEFAULT 0,
		last_elapsed_time BIGINT DEFAULT 0,
		current_elapsed_time BIGINT DEFAULT 0,
		next_dir_no BIGINT DEFAULT 0);
END IF;

IF NOT table_exists('bp', 'managers') THEN
	CREATE TABLE managers (
		ip VARCHAR PRIMARY KEY,
		hostname VARCHAR,
		version VARCHAR DEFAULT 'Unknown');
END IF;

IF NOT table_exists('bp', 'snmp_config') THEN
	CREATE TABLE snmp_config (
	id SERIAL PRIMARY KEY,
	destination VARCHAR UNIQUE NOT NULL,
	community VARCHAR DEFAULT 'public',
	enabled BOOLEAN);
END IF;

IF NOT table_exists('bp', 'snmp_history') THEN
	CREATE TABLE snmp_history (
	id SERIAL PRIMARY KEY,
	oid VARCHAR NOT NULL,
	time_sent TIMESTAMP DEFAULT now(),
	destination VARCHAR NOT NULL,
	community VARCHAR,
	traptype VARCHAR,
	object VARCHAR,
	description VARCHAR,
	severity INT,
	status INT);
END IF;

IF NOT table_exists('bp', 'selection_list_app_items') THEN
	CREATE TABLE selection_list_app_items (
		selection_list_id INTEGER NOT NULL REFERENCES selection_lists(selection_list_id) ON DELETE CASCADE,
		app_instance_id INTEGER NOT NULL REFERENCES application_instances(instance_id),
		UNIQUE (selection_list_id, app_instance_id));
END IF;

	-- Used in trigger on bp.application_schedules table
	CREATE OR REPLACE FUNCTION application_schedules_unique_add_new_instances()
		RETURNS TRIGGER AS $app_sched_tag$
	DECLARE
		already_exists BOOLEAN := FALSE;
		app_type varchar := FALSE;
		do_check BOOLEAN := FALSE;
	BEGIN
		-- Only run the query if a new row with add_new_instances is being
		-- added, or if the value is changing from false to true.
		IF TG_OP = 'INSERT' AND NEW.add_new_instances = TRUE THEN
			do_check := TRUE;
		ELSIF TG_OP = 'UPDATE' THEN
			-- Can't check OLD in the ELSE clause because it's not shortcut-evaluated
			IF OLD.add_new_instances = FALSE AND NEW.add_new_instances = TRUE THEN
				do_check := TRUE;
			END IF;
		END IF;
		-- Only one schedule per (node_no, app_id) can have add_new_instances = TRUE
		-- except for VMware schedules, in which any ESX server can be in only one
		-- such schedule
		IF do_check = TRUE THEN
			SELECT TRUE,al.type INTO already_exists,app_type
			  FROM bp.application_schedules
			  JOIN bp.application_lookup al USING (app_id)
			  WHERE add_new_instances = TRUE
			  AND node_no = NEW.node_no
			  AND app_id = NEW.app_id LIMIT 1;
			IF (already_exists = TRUE AND app_type != 'VMware') THEN
				RAISE EXCEPTION 'A schedule for node %, app % already has add_new_instances = TRUE', NEW.node_no, NEW.app_id;
				RETURN NULL;
			ELSIF (already_exists = TRUE AND app_type = 'VMware') THEN
				-- See if this ESX server is in any schedules with the policy
				SELECT TRUE INTO already_exists
				  FROM bp.vmware_schedules
				  WHERE application_schedule_id = NEW.application_schedule_id
				  AND esx_uuid IN
					 (SELECT esx_uuid
					  FROM bp.application_schedules
					  JOIN bp.vmware_schedules USING (application_schedule_id)
					  WHERE add_new_instances = TRUE)
				  LIMIT 1;
				IF (already_exists = TRUE) THEN
					RAISE EXCEPTION 'A schedule for this ESX server already has add_new_instances = TRUE';
					RETURN NULL;
				END IF;
			END IF;
		END IF;
		RETURN NEW;
	END
	$app_sched_tag$ LANGUAGE 'plpgsql';

IF NOT table_exists('bp', 'application_schedules') THEN
	CREATE TABLE application_schedules (
		application_schedule_id INTEGER PRIMARY KEY DEFAULT nextval('schedules_schedule_id_seq'::regclass),
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		calendar_id INTEGER REFERENCES calendars (calendar_id),
		app_id INTEGER REFERENCES application_lookup (app_id),
		option_list_id INTEGER REFERENCES option_lists (option_list_id),
		node_no INTEGER NOT NULL,
		add_new_instances BOOLEAN NOT NULL DEFAULT FALSE,
		enabled BOOLEAN NOT NULL DEFAULT FALSE,
		email_report BOOLEAN NOT NULL DEFAULT FALSE,
		before_command VARCHAR(79),
		after_command VARCHAR(79));

	CREATE TRIGGER application_schedules_add_new_instances_constraint
		BEFORE INSERT OR UPDATE ON application_schedules
		FOR EACH ROW EXECUTE PROCEDURE application_schedules_unique_add_new_instances();
END IF;
UPDATE pg_trigger SET tgdeferrable = TRUE, tginitdeferred = TRUE
  WHERE tgname = 'application_schedules_add_new_instances_constraint';

IF NOT table_exists('bp', 'schedule_instances') THEN
	CREATE TABLE schedule_instances (
		application_schedule_id INTEGER REFERENCES application_schedules (application_schedule_id) ON DELETE CASCADE,
		instance_id INTEGER REFERENCES application_instances (instance_id));
END IF;

IF NOT table_exists('bp', 'schedule_history') THEN
	CREATE TABLE schedule_history (
		schedule_id INTEGER,
		instance_id INTEGER, --0 reserved for "no events to launch" status
		scheduled_start_time BIGINT,
		status_time BIGINT,
		type INTEGER,    --LCST_* backup type
		job_no INTEGER,
		backup_no INTEGER,
		queued INTEGER,
		status_string VARCHAR,
		PRIMARY KEY (schedule_id, instance_id, scheduled_start_time));
END IF;


IF NOT table_exists('bp', 'state') THEN
	-- Unique identifier of database state, used to guarantee
	-- that node_no and/or instance_id values match
	CREATE TABLE bp.state (
		install_id VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'dirs') THEN
	-- Part of the new "main" schema, here to avoid migration later
	CREATE TABLE bp.dirs (
		dir_no SERIAL PRIMARY KEY,
		dname VARCHAR UNIQUE NOT NULL);
END IF;

IF NOT table_exists('bp', 'backup_files') THEN
	CREATE TABLE bp.backup_files (
		backup_no INTEGER REFERENCES bp.backups(backup_no) ON DELETE CASCADE,
		end_backup_no INTEGER NOT NULL DEFAULT 0,
		dir_no INTEGER NOT NULL REFERENCES bp.dirs(dir_no),
		fname VARCHAR NOT NULL,
		type INT2 NOT NULL,
		size BIGINT NOT NULL,
		mod_date INTEGER NOT NULL,
		blk_offset BIGINT NOT NULL,
		blocks BIGINT NOT NULL,
		PRIMARY KEY (backup_no, blk_offset));
	-- This non-intuitive column order was the fastest indexing strategy
	-- for queries in 6.0 synthesis, QuickSeek, file browse, and updatedb.
	CREATE INDEX backup_files_backup_dir_idx ON bp.backup_files(dir_no, backup_no, end_backup_no);
END IF;


IF NOT table_exists('bp', 'archive_instances') THEN
	-- This table allows us to properly enforce integrity with NULL keyN values
	-- while providing a snapshot of the client at the time of backup
	CREATE TABLE bp.archive_instances (
		archive_instance_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		instance_id INTEGER UNIQUE
			REFERENCES bp.application_instances(instance_id),
		client_name VARCHAR NOT NULL,
		app_id INTEGER NOT NULL,
		key1 VARCHAR,
		key2 VARCHAR,
		key3 VARCHAR,
		key4 VARCHAR,
		client_os INTEGER NOT NULL,
		UNIQUE (instance_id, client_name, app_id, key1, key2, key3, key4));
END IF;

-- Used in trigger to disallow modification to referenced archive profiles
CREATE OR REPLACE FUNCTION bp.archive_profile_no_modification()
	RETURNS TRIGGER AS $app_sched_tag$
DECLARE
	referenced BOOLEAN := FALSE;
BEGIN
	IF TG_OP = 'INSERT' THEN
		SELECT INTO referenced TRUE
		  FROM bp.archive_sets WHERE profile_id = NEW.profile_id LIMIT 1;
	ELSIF TG_OP = 'UPDATE' OR TG_OP = 'DELETE' THEN
		SELECT INTO referenced TRUE
		  FROM bp.archive_sets WHERE profile_id = OLD.profile_id LIMIT 1;
	END IF;
	IF (referenced = TRUE) THEN
		RAISE EXCEPTION 'Archive profile cannot be modified: it is referenced in one or more archive sets.';
		RETURN NULL;
	ELSE
		RETURN NEW;
	END IF;
END
$app_sched_tag$ LANGUAGE 'plpgsql';

IF NOT table_exists('bp', 'archive_profiles') THEN
	CREATE TABLE bp.archive_profiles (
		profile_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		description VARCHAR(128) NOT NULL,
		target VARCHAR NOT NULL,
		range_end INTEGER NOT NULL,
		range_size INTEGER NOT NULL,
		install_id VARCHAR NOT NULL,
		revision INTEGER NOT NULL,
		append BOOLEAN NOT NULL DEFAULT FALSE,
		compress BOOLEAN NOT NULL DEFAULT FALSE,
		encrypt BOOLEAN NOT NULL DEFAULT FALSE,
		email_report BOOLEAN NOT NULL DEFAULT FALSE,
		retention_days INTEGER NOT NULL DEFAULT 0);

		CREATE TRIGGER archive_profiles_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profiles
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_backtypes') THEN
	CREATE TABLE bp.archive_profile_backtypes (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		backtype INTEGER NOT NULL,
		PRIMARY KEY (profile_id, backtype));

		CREATE TRIGGER archive_profile_backtypes_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_backtypes
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_clients') THEN
	CREATE TABLE bp.archive_profile_clients (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		node_no INTEGER NOT NULL
			REFERENCES bp.nodes(node_no) ON DELETE RESTRICT,
		PRIMARY KEY (profile_id, node_no));

		CREATE TRIGGER archive_profile_clients_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_clients
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_instances') THEN
	CREATE TABLE bp.archive_profile_instances (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		archive_instance_id INTEGER NOT NULL
			REFERENCES bp.archive_instances(archive_instance_id) ON DELETE RESTRICT,
		PRIMARY KEY (profile_id, archive_instance_id));

		CREATE TRIGGER archive_profile_instances_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_instances
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_localdirs') THEN
	CREATE TABLE bp.archive_profile_localdirs (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		localdir VARCHAR NOT NULL,
		PRIMARY KEY (profile_id, localdir));

		CREATE TRIGGER archive_profile_localdirs_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_localdirs
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_schedule_profiles') THEN
	CREATE TABLE bp.archive_schedule_profiles (
		application_schedule_id INTEGER UNIQUE NOT NULL
			REFERENCES bp.application_schedules(application_schedule_id)
			ON DELETE CASCADE,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT);
END IF;

IF NOT table_exists('bp', 'archive_job_profiles') THEN
	CREATE TABLE bp.archive_job_profiles (
		job_no INTEGER PRIMARY KEY
			REFERENCES bp.jobs(job_no) ON DELETE CASCADE,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT);
END IF;

IF NOT table_exists('bp', 'archive_sets') THEN
	CREATE TABLE bp.archive_sets (
		archive_set_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT,
		media_label VARCHAR NOT NULL,
		originating_asset VARCHAR NOT NULL,
		creation_timestamp INTEGER NOT NULL,
		elapsed_secs INTEGER NOT NULL,
		archive_set_path VARCHAR NOT NULL,
		state_success BOOLEAN NOT NULL,
		status VARCHAR NOT NULL,
		pid INTEGER NOT NULL);
END IF;

IF NOT table_exists('bp', 'archive_disks') THEN
	CREATE TABLE bp.archive_disks (
		serial_no VARCHAR NOT NULL,
		archive_set_id INTEGER NOT NULL
			REFERENCES bp.archive_sets(archive_set_id) ON DELETE CASCADE,
		slot INTEGER NOT NULL,
		PRIMARY KEY (serial_no, archive_set_id),
		UNIQUE (archive_set_id, slot));
END IF;

IF NOT table_exists('bp', 'archives') THEN
	CREATE TABLE bp.archives (
		archive_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		archive_set_id INTEGER NOT NULL
			REFERENCES bp.archive_sets(archive_set_id) ON DELETE RESTRICT,
		orig_description VARCHAR,
		orig_backup_no INTEGER,
		dep_backup_no INTEGER,
		archive_instance_id INTEGER
			REFERENCES bp.archive_instances(archive_instance_id)
			ON DELETE RESTRICT,
		-- Properties of the backup
		backtype INTEGER NOT NULL,
		orig_time INTEGER NOT NULL,
		orig_mib_size INTEGER NOT NULL,
		file_mib_size INTEGER NOT NULL,
		file_count INTEGER NOT NULL DEFAULT 0,
		compressed BOOLEAN NOT NULL,
		encrypted BOOLEAN NOT NULL,
		-- Properties of the archive
		elapsed_secs INTEGER NOT NULL,
		archive_path VARCHAR,
		archive_success BOOLEAN NOT NULL,
		error_string VARCHAR,
		UNIQUE (archive_set_id, orig_description, orig_backup_no),
		CONSTRAINT archive_has_source
			CHECK (orig_description IS NOT NULL OR orig_backup_no IS NOT NULL));

	CREATE OR REPLACE FUNCTION bp.archives_instance_in_archive_set()
		RETURNS TRIGGER AS $app_sched_tag$
	DECLARE
		exists BOOLEAN := FALSE;
	BEGIN
		IF (NEW.archive_instance_id IS NULL AND NEW.orig_backup_no IS NULL) THEN
			RETURN NEW;
		END IF;

		-- Check instance assignments
		SELECT INTO exists TRUE
		  FROM bp.archive_profile_instances api
		  JOIN bp.archive_sets USING (profile_id)
		  WHERE archive_sets.archive_set_id = NEW.archive_set_id
		    AND api.archive_instance_id = NEW.archive_instance_id
		  LIMIT 1;

		IF (exists = TRUE) THEN
			RETURN NEW;
		END IF;

		-- Check client assignments
		SELECT INTO exists TRUE
		  FROM bp.archive_profile_clients
		  JOIN bp.archive_sets USING (profile_id)
		  JOIN bp.nodes USING (node_no)
		  JOIN bp.archive_instances ai ON (nodes.node_name = ai.client_name)
		  WHERE ai.instance_id IS NOT NULL
		    AND archive_sets.archive_set_id = NEW.archive_set_id
		    AND ai.archive_instance_id = NEW.archive_instance_id
		  LIMIT 1;

		IF (exists = TRUE) THEN
			RETURN NEW;
		ELSE
			RAISE EXCEPTION 'Archive instance % must be in archive set %',
				NEW.archive_instance_id, NEW.archive_set_id;
			RETURN NULL;
		END IF;
	END
	$app_sched_tag$ LANGUAGE 'plpgsql';

	CREATE TRIGGER archives_instance_in_archive_set_constraint_constraint
		BEFORE INSERT OR UPDATE ON bp.archives
		FOR EACH ROW EXECUTE PROCEDURE bp.archives_instance_in_archive_set();
END IF;

IF NOT table_exists('bp', 'archive_files') THEN
	CREATE TABLE bp.archive_files (
		archive_id INTEGER NOT NULL
			REFERENCES bp.archives(archive_id) ON DELETE CASCADE,
		dir_no INTEGER NOT NULL REFERENCES bp.dirs(dir_no),
		fname VARCHAR NOT NULL,
		type INT2 NOT NULL,
		size BIGINT NOT NULL,
		mod_date INTEGER NOT NULL,
		blocks BIGINT NOT NULL,
		blk_offset BIGINT NOT NULL,
		PRIMARY KEY (archive_id, blk_offset));
	CREATE INDEX archive_files_archive_dir_idx
			ON bp.archive_files(archive_id, dir_no);
END IF;

IF NOT table_exists('bp', 'archive_restore_specs') THEN
	CREATE TABLE bp.archive_restore_specs (
		job_no INTEGER PRIMARY KEY REFERENCES bp.jobs(job_no) ON DELETE CASCADE,
		archive_id INTEGER NOT NULL
			REFERENCES bp.archives(archive_id) ON DELETE CASCADE,
		device_no INTEGER NOT NULL
			REFERENCES bp.devices(device_no) ON DELETE CASCADE,
		node_no INTEGER REFERENCES bp.nodes(node_no) ON DELETE CASCADE,
		instance_id INTEGER
			REFERENCES bp.application_instances(instance_id) ON DELETE CASCADE,
		dr_mode BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF NOT table_exists('bp', 'archive_restore_files') THEN
	CREATE TABLE bp.archive_restore_files (
		job_no INTEGER NOT NULL
			REFERENCES bp.archive_restore_specs(job_no) ON DELETE CASCADE,
		pathname VARCHAR NOT NULL,
		is_inclusion BOOLEAN NOT NULL,
		PRIMARY KEY (job_no, pathname));
END IF;


IF NOT table_exists('bp', 'vmware_esx_servers') THEN
	CREATE TABLE bp.vmware_esx_servers (
		esx_uuid VARCHAR NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		version VARCHAR NOT NULL,
		PRIMARY KEY(esx_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_vcenter_servers') THEN
	CREATE TABLE bp.vmware_vcenter_servers (
		vcenter_uuid VARCHAR NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		PRIMARY KEY (vcenter_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_vcenter_assoc') THEN
	CREATE TABLE bp.vmware_vcenter_assoc (
		esx_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_esx_servers ON DELETE CASCADE,
		vcenter_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_vcenter_servers ON DELETE CASCADE,
		PRIMARY KEY (esx_uuid, vcenter_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_connections') THEN
	CREATE TABLE bp.vmware_connections (
		connection_id SERIAL NOT NULL,
		esx_uuid VARCHAR
			REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		vcenter_uuid VARCHAR
			REFERENCES bp.vmware_vcenter_servers (vcenter_uuid) ON DELETE CASCADE,
		login VARCHAR NOT NULL,
		password VARCHAR NOT NULL,
		address VARCHAR NOT NULL,
		PRIMARY KEY(connection_id));
END IF;

IF NOT table_exists('bp', 'vmware_snapshots') THEN
	CREATE TABLE bp.vmware_snapshots (
		backup_no BIGINT NOT NULL
			REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		change_id VARCHAR NOT NULL,
		PRIMARY KEY (backup_no));
END IF;

IF NOT table_exists('bp', 'vmware_vms') THEN
	CREATE TABLE bp.vmware_vms (
		vm_uuid VARCHAR NOT NULL,
		esx_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		connection_id INTEGER NOT NULL
			REFERENCES bp.vmware_connections (connection_id) ON DELETE RESTRICT,
		instance_id INTEGER
			REFERENCES bp.application_instances (instance_id) NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		change_tracking_enabled BOOLEAN NOT NULL,
		virtual_hw_version VARCHAR NOT NULL,
		PRIMARY KEY (vm_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_schedules') THEN
	CREATE TABLE bp.vmware_schedules (
		application_schedule_id INTEGER NOT NULL
			REFERENCES bp.application_schedules (application_schedule_id) ON DELETE CASCADE,
		-- For now leave off the constraint.  In 5.1, esx_uuid can refer to
		-- an ESX or vCenter server.  This table is going away in 6.0.
		esx_uuid VARCHAR NOT NULL,
		PRIMARY KEY (application_schedule_id));

	-- Used to enforce constraint on bp.application_schedules.add_new_instances
	CREATE OR REPLACE FUNCTION vmware_schedules_unique_add_new_instances()
		RETURNS TRIGGER AS $app_sched_tag$
	DECLARE
		already_exists BOOLEAN := FALSE;
	BEGIN
		-- See if this ESX server is in any schedules with the policy
		SELECT TRUE INTO already_exists
		  FROM bp.vmware_schedules
		  WHERE application_schedule_id = NEW.application_schedule_id
		  AND esx_uuid IN
			 (SELECT esx_uuid
			  FROM bp.application_schedules
			  JOIN bp.vmware_schedules USING (application_schedule_id)
			  WHERE add_new_instances = TRUE)
		  LIMIT 1;
		IF (already_exists = TRUE) THEN
			RAISE EXCEPTION 'A schedule for this ESX server already has add_new_instances = TRUE';
			RETURN NULL;
		END IF;
		RETURN NEW;
	END
	$app_sched_tag$ LANGUAGE 'plpgsql';

	CREATE TRIGGER vmware_schedules_add_new_instances_constraint
		BEFORE INSERT OR UPDATE ON vmware_schedules
		FOR EACH ROW EXECUTE PROCEDURE vmware_schedules_unique_add_new_instances();
	UPDATE pg_trigger SET tgdeferrable = TRUE, tginitdeferred = TRUE
	  WHERE tgname = 'vmware_schedules_add_new_instances_constraint';

	-- Populate the table upon creation, modifying schedules for compliance:
	-- each schedule can contain VMs for only a single ESX server

	-- Apply the ESX UUID of the lowest-numbered VM app_instance to each schedule
	ALTER TABLE vmware_schedules DISABLE TRIGGER ALL;
	INSERT INTO vmware_schedules (application_schedule_id, esx_uuid)
		SELECT application_schedule_id, esx_uuid
		FROM (SELECT application_schedule_id, MIN(instance_id) AS instance_id
		      FROM application_schedules
		      JOIN schedule_instances USING (application_schedule_id)
		      GROUP BY application_schedule_id) AS schedinst
		JOIN vmware_vms USING (instance_id);
	ALTER TABLE vmware_schedules ENABLE TRIGGER ALL;

	-- Remove all VMs not assigned to the schedules
	DELETE FROM schedule_instances
		WHERE (application_schedule_id, instance_id)
		IN (SELECT application_schedule_id, instance_id
			FROM application_schedules
			JOIN schedule_instances USING (application_schedule_id)
			JOIN vmware_vms USING (instance_id)
			JOIN vmware_schedules USING (application_schedule_id)
			WHERE vmware_vms.esx_uuid != vmware_schedules.esx_uuid);

	-- Fix up policy flag so that it is set in only one schedule per ESX server
	ALTER TABLE application_schedules DISABLE TRIGGER ALL;
	UPDATE application_schedules
		SET add_new_instances = FALSE
		WHERE add_new_instances = TRUE
		AND application_schedule_id IN
		 (SELECT application_schedule_id
		  FROM application_schedules
		  JOIN vmware_schedules USING (application_schedule_id))
		AND application_schedule_id NOT IN
		 (SELECT MIN(application_schedule_id)
		  FROM application_schedules
		  JOIN vmware_schedules USING (application_schedule_id)
		  GROUP BY esx_uuid);
	ALTER TABLE application_schedules ENABLE TRIGGER ALL;
END IF;



IF NOT sequence_exists('bp', 'restore_group_seq') THEN
	CREATE SEQUENCE bp.restore_group_seq;
END IF;


-- The snmp_temp table was an interim table in 4.1.1 for
-- implementing traps that are not in the alerts table,
-- and is no longer needed since these traps are now
-- represented in the notification framework.
IF table_exists('bp', 'snmp_temp') THEN
	DROP TABLE snmp_temp; 
END IF;

RETURN res;
END;
$BODY$


LANGUAGE 'plpgsql' VOLATILE;

SELECT create_bp();
DROP FUNCTION create_bp();

SET search_path TO statussync,public;

CREATE OR REPLACE FUNCTION create_statussync()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
BEGIN
	res:=0;

-- begin statussync tables
IF NOT table_exists('statussync', 'sync_metadata') THEN
	CREATE TABLE sync_metadata (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		last_sync_remotetime BIGINT,
		last_sync_localtime BIGINT,
		PRIMARY KEY (system_id));
END IF;
	
IF NOT table_exists('statussync', 'sync_alerts') THEN
	CREATE TABLE sync_alerts (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		alert_id INTEGER,
		source_id INTEGER,
		source_name VARCHAR,
		severity INTEGER,
		alert_text TEXT,
		target TEXT,
		created BIGINT,
		updated BIGINT,
		closed BOOLEAN,
		PRIMARY KEY (system_id, alert_id));
END IF;
	
IF NOT table_exists('statussync', 'sync_nodes') THEN
	CREATE TABLE sync_nodes (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		node_no INTEGER,
		node_name VARCHAR,
		type INTEGER,
		os INTEGER,
		status INTEGER,
		priority INTEGER,
		properties INTEGER,
		PRIMARY KEY (system_id, node_no));
END IF;
	
IF NOT table_exists('statussync', 'sync_backups') THEN
	CREATE TABLE sync_backups (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		backup_no INTEGER,
		node_no INTEGER,
		start_time BIGINT,
		elapsed_time BIGINT,
		job_no INTEGER,
		total_netpaths INTEGER,
		total_megs INTEGER,
		status INTEGER,
		exit_code INTEGER,
		type INTEGER,
		properties INTEGER,
		PRIMARY KEY (system_id, backup_no),
		FOREIGN KEY (system_id, node_no) REFERENCES sync_nodes (system_id, node_no) ON DELETE CASCADE);
END IF;
	
IF NOT table_exists('statussync', 'sync_sds') THEN
	CREATE TABLE sync_sds (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		backup_no BIGINT NOT NULL,
		sync_type INTEGER NOT NULL,
		presync_seconds INTEGER,
		sync_seconds INTEGER,
		postsync_seconds INTEGER,
		done_epoch BIGINT NOT NULL,
		bytes_syncd BIGINT,
		message TEXT NOT NULL);
END IF;

IF NOT table_exists('statussync', 'sync_sql_backups') THEN
	CREATE TABLE sync_sql_backups (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		sql_ref BIGINT,
		node_no INTEGER,
		database_name VARCHAR,
		server_instance_name VARCHAR,
		group_file_order BIGINT,
		file_order_no BIGINT,
		sql_type INTEGER,
		PRIMARY KEY (system_id, sql_ref));
END IF;
-- end of statussync tables

RETURN res;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT create_statussync();
DROP FUNCTION create_statussync();

DROP FUNCTION public.table_exists(sch_name VARCHAR,tbl_name VARCHAR);
DROP FUNCTION public.sequence_exists(sch_name VARCHAR, seq_name VARCHAR);
