-- Define any functions here.
-- Use "CREATE or REPLACE FUNCTION" since this script will run on both
-- a fresh install or upgrade.  If a function needs to be removed or if
-- it changes signatures, call drop_function_if_exists() to remove the
-- old function before creating a new one.

SET search_path to bp;


---------------------------------------------------------------------------------------
-- Returns TRUE if all functions specified were deleted successfully
CREATE OR REPLACE FUNCTION conditionally_drop_function(sch_name VARCHAR, func_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$
	DECLARE fn_row RECORD;
	DECLARE p_string VARCHAR := '';
	DECLARE type_array INT8[];
	DECLARE type_rec RECORD;
	DECLARE i INTEGER;
	DECLARE type_oid INT8;

BEGIN
	-- Select all functions with this name and retrieve the argument types
	FOR fn_row IN
		SELECT CAST(proargtypes AS INT8[]) FROM pg_catalog.pg_proc p, pg_catalog.pg_namespace n
		WHERE p.pronamespace = n.oid
			AND lower(n.nspname) = lower(sch_name)
			AND lower(p.proname) = lower(func_name)
	LOOP
		-- Build the query string for this function by iterating over parameter type array
		-- and concatenating the typenames into the string
		type_array := fn_row.proargtypes;
		FOR i IN COALESCE(array_lower(type_array,1),0) .. COALESCE(array_upper(type_array,1),-1)
		LOOP
			type_oid := type_array[i];
			SELECT INTO type_rec typname FROM pg_catalog.pg_type WHERE oid = type_oid;
			IF (i > 0) THEN p_string := p_string || ','; END IF;
			p_string := p_string || type_rec.typname;
		END LOOP;

		-- Execute the query string
		p_string := 'DROP FUNCTION ' || sch_name || '.' || func_name || '(' || p_string || ')';
		EXECUTE p_string;
		-- RAISE NOTICE 'Executed: %', p_string;
		-- RAISE NOTICE 'DROP FUNCTION';
	END LOOP;

	RETURN TRUE;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;


---------------------------------------------------------------------------------------
-- Drop removed functions that were in previous product versions
SELECT conditionally_drop_function('bp', 'MarkNetpathsForPurge');
SELECT conditionally_drop_function('bp', 'PurgeUnreferencedNetpaths');


---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION ApproxCount(SchemaName VARCHAR, TableName VARCHAR)
RETURNS BIGINT AS
$BODY$

DECLARE
	approx_count BIGINT := 0;

BEGIN

-- This function returns the approximate count of rows in a table
SELECT INTO approx_count reltuples::numeric
	FROM pg_catalog.pg_class, pg_catalog.pg_namespace
	WHERE pg_class.relnamespace = pg_namespace.oid
	  AND pg_namespace.nspname = SchemaName
	  AND pg_class.relname = TableName;

RETURN approx_count;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
SELECT conditionally_drop_function('bp', 'ReallyPurgeFileset'); -- return value changed
CREATE OR REPLACE FUNCTION ReallyPurgeFileset(fs_no BIGINT, NetpathsPurge BOOLEAN,
	OUT unref_details INTEGER, OUT unref_netpaths INTEGER)
RETURNS SETOF RECORD
AS
$BODY$
-- Deletes the fileset and detail records for the given fileset_no.
-- If NetpathsPurge is true, deletes the netpaths records associated with the fileset.
-- Returns count of details purged and count of netpaths purged

BEGIN
-- RAISE NOTICE 'ReallyPurgeFileset: fileset_no %, NetpathsPurge is %', fs_no, NetpathsPurge;

IF (NetpathsPurge)
THEN
	-- Delete all netpaths referenced by detail entries in this fileset but not
	-- referenced by any detail entries not in this fileset
	DELETE FROM bp.netpaths
		USING bp.detail AS src
		WHERE src.node_no = netpaths.node_no
		AND src.dir_no = netpaths.dir_no
		AND src.file_no = netpaths.file_no
		AND src.lib_inst_no = netpaths.lib_inst_no
		AND src.fileset_no = fs_no
		AND (SELECT node_no FROM bp.netpaths
		     WHERE src.node_no = netpaths.node_no
		       AND src.dir_no = netpaths.dir_no
		       AND src.file_no = netpaths.file_no
		       AND src.lib_inst_no = netpaths.lib_inst_no
		       AND src.fileset_no != netpaths.fileset_no) IS NULL;
	GET DIAGNOSTICS unref_netpaths = ROW_COUNT;
ELSE
	unref_netpaths := 0;
END IF;

DELETE FROM bp.detail WHERE fileset_no = fs_no;
GET DIAGNOSTICS unref_details = ROW_COUNT;
DELETE FROM bp.filesets WHERE fileset_no = fs_no;

-- RAISE NOTICE 'Netpaths processed for fileset_no %: Total=%, Deleted=%, Unreferenced=%',
--			fs_no, total_count, delete_count, unref_count;

RETURN NEXT;
RETURN;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION PurgeEmptyBackups(PurgeDays INTEGER, NetpathsPurge BOOLEAN)
RETURNS INTEGER AS
$BODY$

DECLARE
	window BIGINT;
	b bp.backups%ROWTYPE;
	f RECORD;
	COMMENT_REC CONSTANT INTEGER := 65525; 
	CNTL_REC CONSTANT INTEGER := 65527; 
	total_unref INTEGER := 0;
	bkup_count INTEGER := 0;
	P_SQL CONSTANT INTEGER := 128;
	fs_fn_result RECORD;

BEGIN

SELECT INTO window EXTRACT(EPOCH FROM now())::BIGINT;
window := window - (PurgeDays * 3600 * 24);
-- RAISE NOTICE 'window is epoch % and timestamp %', window, to_timestamp(window);

-- Bug 8695: do not delete records where P_HASHED is still set
FOR b IN SELECT * FROM bp.backups WHERE total_netpaths = 0
			AND type IN (1,2,4,8,1024,4096)
			AND ((properties & 512) = 0)
			AND start_time < window
LOOP
	bkup_count := bkup_count + 1;

	-- Remove unreferenced netpaths entries if NetpathsPurge is true
	IF (NetpathsPurge)
	THEN
		FOR f IN SELECT fileset_no FROM bp.filesets WHERE backup_no = b.backup_no
		LOOP
			SELECT INTO fs_fn_result * FROM bp.ReallyPurgeFileset(f.fileset_no, NetpathsPurge);	
			total_unref := total_unref + fs_fn_result.unref_netpaths;
		END LOOP; 
	END IF;

	-- If it's a sql backup, delete the corresponding records in sql_backups
	IF ((b.properties & P_SQL) = P_SQL)
	THEN
		DELETE FROM bp.sql_backups WHERE bp.sql_backups.sql_ref = b.backup_no;
	END IF;
END LOOP;

IF(bkup_count > 0)
THEN
-- Bug 8695: do not delete records where P_HASHED is still set
	DELETE FROM bp.backup_msg WHERE backup_no IN
		(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
			AND type IN (1,2,4,8,1024,4096)
			AND ((properties & 512) = 0)
			AND start_time < window);

	-- The verify records
	DELETE FROM bp.backup_msg WHERE backup_no IN
		(SELECT backup_no FROM bp.backups where data_bkno IN
			(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
				AND type IN (1,2,4,8,1024,4096)
				AND ((properties & 512) = 0)
				AND start_time < window));

	DELETE FROM bp.lcs_hist WHERE backup_no IN
		(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
			AND type IN (1,2,4,8,1024,4096)
			AND ((properties & 512) = 0)
			AND start_time < window)
		AND node_no != COMMENT_REC AND node_no != CNTL_REC;

	-- The verify records
	DELETE FROM bp.lcs_hist WHERE backup_no IN
		(SELECT backup_no FROM bp.backups where data_bkno IN
			(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
				AND type IN (1,2,4,8,1024,4096)
				AND ((properties & 512) = 0)
				AND start_time < window))
		AND node_no != COMMENT_REC AND node_no != CNTL_REC;

	-- The verify records
	DELETE FROM bp.backups WHERE backup_no IN 
		(SELECT backup_no FROM bp.backups where data_bkno IN
			(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
				AND type IN (1,2,4,8,1024,4096)
				AND ((properties & 512) = 0)
				AND start_time < window));

	DELETE FROM bp.backups WHERE backup_no IN 
		(SELECT backup_no FROM bp.backups WHERE total_netpaths = 0
			AND type IN (1,2,4,8,1024,4096)
			AND ((properties & 512) = 0)
			AND start_time < window);
END IF;

RETURN bkup_count;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION ReallyPurgeBackups(NetpathsPurge BOOLEAN)
RETURNS INTEGER AS
$BODY$
-- This function deletes that backups that have been marked as logically deleted,
-- along with their associated filesets, backup_msg, lcs_hist and sql_backups
-- records.

DECLARE
	bkup RECORD;
	f RECORD;
	COMMENT_REC CONSTANT INTEGER := 65525; 
	CNTL_REC CONSTANT INTEGER := 65527; 
	total_unref INTEGER := 0;
	bkup_count INTEGER := 0;
	P_SQL CONSTANT INTEGER := 128;
	fs_fn_result RECORD;

BEGIN
-- RAISE NOTICE 'ReallyPurgeBackups: NetpathsPurge is %', NetpathsPurge;

-- Bug 8695: do not delete records where P_HASHED is still set
FOR bkup IN SELECT backup_no, type, properties FROM bp.backups WHERE tape_no = 4294967295
				AND ((properties & 512) = 0)
LOOP
	bkup_count := bkup_count + 1;

	IF (bkup.type != 128 AND bkup.type != 512)
	-- it is not a verify
	THEN
		-- Remove the fileset itself
		FOR f IN SELECT fileset_no FROM bp.filesets WHERE backup_no = bkup.backup_no
		LOOP
			SELECT INTO fs_fn_result * FROM bp.ReallyPurgeFileset(f.fileset_no, NetpathsPurge);	
			total_unref := total_unref + fs_fn_result.unref_netpaths;
		END LOOP; 

		-- If it's a sql backup, delete the corresponding records in sql_backups
		IF ((bkup.properties & P_SQL) = P_SQL)
		THEN
			DELETE FROM bp.sql_backups WHERE bp.sql_backups.sql_ref = bkup.backup_no;
		END IF;
	END IF;
END LOOP;

IF (bkup_count > 0)
THEN
-- Bug 8695: do not delete records where P_HASHED is still set
	DELETE FROM bp.backup_msg WHERE backup_no IN
		(SELECT backup_no FROM bp.backups WHERE tape_no = 4294967295 AND ((properties & 512) = 0));

	DELETE FROM bp.lcs_hist WHERE backup_no IN
		(SELECT backup_no FROM bp.backups WHERE tape_no = 4294967295 AND ((properties & 512) = 0))
		AND node_no != COMMENT_REC AND node_no != CNTL_REC;

	DELETE FROM bp.backups WHERE backup_no IN 
		(SELECT backup_no FROM bp.backups WHERE tape_no = 4294967295 AND ((properties & 512) = 0));
END IF;

-- RAISE NOTICE 'Purged % backups, total unreferenced netpaths = %', bkup_count, total_unref;

RETURN bkup_count;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
SELECT conditionally_drop_function('bp', 'UnreferencedFilesets'); -- return value changed
CREATE OR REPLACE FUNCTION UnreferencedFilesets(DoPurge BOOLEAN, NetpathsPurge BOOLEAN,
	OUT unref_filesets INTEGER, OUT unref_details INTEGER, OUT unref_netpaths INTEGER)
RETURNS SETOF RECORD
AS
$BODY$
-- This function will either report the number of unreferenced filesets (DoPurge=false), or
-- actually remove them (DoPurge=true).  Note that NetpathsPurge is a setting in master.ini that
-- indicates whether or not the associated netpaths record should be deleted(T) or updated(F)
-- if the removal of this fileset and its corresponding detail records have rendered some
-- netpaths records as unreferenced by any fileset.

-- Returns one record with 3 columns: (unref_filesets, unref_details, unref_netpaths)

DECLARE
	fs RECORD;
	fs_fn_result RECORD;

BEGIN
-- RAISE NOTICE 'UnreferencedFilesets: NetpathsPurge is %, DoPurge is %', NetpathsPurge, DoPurge;
unref_filesets := 0;
unref_details := 0;
unref_netpaths := 0;

FOR fs IN SELECT fileset_no FROM bp.filesets
			WHERE backup_no NOT IN (SELECT backup_no FROM bp.backups)
LOOP
	unref_filesets := unref_filesets + 1;
	IF (DoPurge) 
	THEN
		SELECT INTO fs_fn_result * FROM bp.ReallyPurgeFileset(fs.fileset_no, NetpathsPurge);	
		unref_details := unref_details + fs_fn_result.unref_details;
		unref_netpaths := unref_netpaths + fs_fn_result.unref_netpaths;
	END IF;
END LOOP;

-- IF (DoPurge)
-- THEN
--	RAISE NOTICE 'Purged % unreferenced filesets, resulting in % unreferenced netpaths ',
--			unref_filesets, unref_netpaths; 
-- ELSE
--	RAISE NOTICE 'Unreferenced filesets count: %', unref_filesets;
-- END IF;

RETURN NEXT;
RETURN;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION UnreferencedDetails(DoPurge BOOLEAN)
RETURNS BIGINT AS
$BODY$
-- This function will either report the number of unreferenced details (DoPurge=false), or
-- actually remove them (DoPurge=true).
DECLARE
	unref_details BIGINT := 0;

BEGIN
-- RAISE NOTICE 'UnreferencedDetails: DoPurge is %', DoPurge;

IF(DoPurge)
THEN
	DELETE FROM bp.detail WHERE fileset_no NOT IN (SELECT fileset_no FROM bp.filesets);
	GET DIAGNOSTICS unref_details = ROW_COUNT;
ELSE
	SELECT INTO unref_details count(*) FROM bp.detail WHERE fileset_no NOT IN
			(SELECT fileset_no FROM bp.filesets);
END IF;

-- RAISE NOTICE 'Unreferenced detail count: %', unref_details;
RETURN unref_details;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION SetLibraryPurgeStatus(status INTEGER, cmt_str VARCHAR)
RETURNS INTEGER AS
$BODY$

BEGIN
	UPDATE bp.library SET purge_status = status, comment = cmt_str
	WHERE library_no = 1;

RETURN status;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION PurgeUnreferencedNodeNetpaths(NodeNo INTEGER,
	StartDirNo INTEGER, LastDirNo INTEGER)
RETURNS BIGINT AS
$BODY$

DECLARE
	begin_num_netpaths BIGINT := 0;
	netpaths_purged BIGINT := 0;
	netpaths_remaining BIGINT;
	msg VARCHAR;
	dir_no_cond VARCHAR;
	query VARCHAR;

BEGIN

-- create string representing dir_no conditional in query
dir_no_cond := 'dir_no >= ' || StartDirNo;
IF (LastDirNo > 0)
THEN
	dir_no_cond := dir_no_cond || ' AND dir_no < ' || LastDirNo;
END IF;

LOCK TABLE bp.netpaths, bp.detail;

SELECT INTO begin_num_netpaths bp.ApproxCount('bp', 'netpaths');

query := 'DELETE FROM bp.netpaths'
	|| ' WHERE node_no = ' || NodeNo
	|| ' AND ' || dir_no_cond
	||   ' AND (node_no, dir_no, file_no, lib_inst_no)'
	||   ' NOT IN (SELECT node_no, dir_no, file_no, lib_inst_no'
	||           ' FROM bp.detail WHERE node_no = ' || NodeNo
	||                            ' AND ' || dir_no_cond || ')';
EXECUTE query;

GET DIAGNOSTICS netpaths_purged = ROW_COUNT;

netpaths_remaining := begin_num_netpaths - netpaths_purged;
UPDATE bp.library SET purge_status = 0, total_netpaths = netpaths_remaining,
	unref_netpaths = 0, comment = '' WHERE library_no = 1;

-- RAISE NOTICE 'Unreferenced netpaths count: %', netpaths_purged;
RETURN netpaths_purged;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION UnreferencedNetpaths(DoPurge BOOLEAN)
RETURNS BIGINT AS
$BODY$
-- This function will either report the number of unreferenced netpaths (DoPurge=false), or
-- actually remove them (DoPurge=true).
DECLARE
	unref_netpaths BIGINT := 0;

BEGIN
-- RAISE NOTICE 'UnreferencedNetpaths: DoPurge is %', DoPurge;

IF(DoPurge)
THEN
	DELETE FROM bp.netpaths WHERE node_no NOT IN (SELECT node_no FROM bp.nodes);
	GET DIAGNOSTICS unref_netpaths = ROW_COUNT;
ELSE
	SELECT INTO unref_netpaths count(*) FROM bp.netpaths WHERE node_no NOT IN
			(SELECT node_no FROM bp.nodes);
END IF;

-- RAISE NOTICE 'Unreferenced netpaths count: %', unref_netpaths;
RETURN unref_netpaths;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION PurgeDirnames()
RETURNS INTEGER AS
$BODY$

DECLARE
	d_count BIGINT := 0;
	ISLNK CONSTANT INTEGER := 49;
	SYMLNK CONSTANT INTEGER := 50;

BEGIN
	DELETE FROM bp.d_master WHERE dir_no NOT IN (SELECT DISTINCT dir_no FROM bp.netpaths
		UNION SELECT DISTINCT start_dir_no FROM bp.backups
		UNION SELECT DISTINCT link_to_dir_no FROM bp.netpaths
		        WHERE file_type = ISLNK OR file_type = SYMLNK);
	GET DIAGNOSTICS d_count = ROW_COUNT;

RETURN d_count;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION PurgeFilenames()
RETURNS INTEGER AS
$BODY$

DECLARE
	f_count BIGINT := 0;
	ISLNK CONSTANT INTEGER := 49;
	SYMLNK CONSTANT INTEGER := 50;

BEGIN
	DELETE FROM bp.f_master WHERE file_no NOT IN (SELECT DISTINCT file_no FROM bp.netpaths
		UNION SELECT DISTINCT link_to_file_no FROM bp.netpaths
		        WHERE file_type = ISLNK OR file_type = SYMLNK);
	GET DIAGNOSTICS f_count = ROW_COUNT;

RETURN f_count;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;



---------------------------------------------------------------------------------------
SELECT conditionally_drop_function('bp', 'ImportTapeCreateReport');
CREATE OR REPLACE FUNCTION ImportTapeCreateReport()
RETURNS VARCHAR AS
$BODY$

DECLARE
	node_row RECORD;
	fs_row RECORD;
	rpt_row RECORD;
	fs_query TEXT;
	node_number INTEGER;
	nbackups INTEGER := 0; 
	msg VARCHAR;
	str VARCHAR;
	backup_details VARCHAR[];
	field_lengths INTEGER[];

BEGIN

field_lengths[0] = length('tape');
field_lengths[1] = length('client');
field_lengths[2] = length('backup');

-- Process each node separately for performance
FOR node_row IN SELECT node_no FROM bp.nodes ORDER BY node_no
LOOP
	node_number := node_row.node_no;
	-- Obtain all filesets with missing netpaths
	fs_query := 'SELECT DISTINCT fileset_no FROM bp.detail'
	          || ' WHERE fileset_no IN (SELECT fileset_no FROM bp.filesets)'
	          ||   ' AND node_no = ' || node_number
	          ||   ' AND (dir_no, file_no, lib_inst_no)'
	          ||       ' NOT IN (SELECT dir_no, file_no, lib_inst_no'
	          ||               ' FROM bp.netpaths WHERE node_no = ' || node_number || ')';
	FOR fs_row IN EXECUTE fs_query
	LOOP
		-- Obtain tape_no, node_name, backup_no for all filesets
		SELECT INTO rpt_row backups.tape_no, node_name, backup_no
			FROM bp.backups
				JOIN bp.nodes USING (node_no)
				JOIN bp.filesets USING (backup_no)
			WHERE bp.nodes.node_no = node_number
			  AND bp.filesets.fileset_no = fs_row.fileset_no;

		-- Store details (in groups of 3) and max field widths for later reporting
		backup_details[(3*nbackups)+0] := rpt_row.tape_no;
		backup_details[(3*nbackups)+1] := rpt_row.node_name;
		backup_details[(3*nbackups)+2] := rpt_row.backup_no;
		field_lengths[0] := int4larger(field_lengths[0], length(rpt_row.tape_no));
		field_lengths[1] := int4larger(field_lengths[1], length(rpt_row.node_name));
		field_lengths[2] := int4larger(field_lengths[2], length(rpt_row.backup_no));
		nbackups := nbackups + 1;
	END LOOP;
END LOOP;

IF (nbackups <= 0)
THEN
	msg := '\nNo backups need to be imported.\n';
ELSE
	IF (nbackups = 1) THEN
		str := 'backup needs';
	ELSE
		str := 'backups need';
	END IF;
	msg := '\nThe following ' || nbackups || ' ' || str || ' to be imported because some'
	    || '\ndatabase information necessary to restore the data is incomplete.'
	    || '\nPlease refer to the section "Importing Backups" in the "Backups"'
	    || '\nchapter of the DPU Manual for instructions on how to import a backup.\n\n';

	-- Add formatted query output to msg string
	msg := msg || ' ' || rpad('tape', field_lengths[0])
	         || ' | ' || rpad('client', field_lengths[1])
	         || ' | ' || 'backup' || '\n'
	         || repeat('-', field_lengths[0]+1) || '+++'
	         || repeat('-', field_lengths[1])   || '+++'
	         || repeat('-', field_lengths[2]+1);

	FOR i IN COALESCE(array_lower(backup_details,1),0) .. (COALESCE(array_upper(backup_details,1),-1) / 3)
	LOOP
		-- RAISE NOTICE '% | % | %', backup_details[(3*i)+0], backup_details[(3*i)+1], backup_details[(3*i)+2];
		msg := msg || '\n ' || rpad(backup_details[(3*i)+0], field_lengths[0])
				   || ' | ' || rpad(backup_details[(3*i)+1], field_lengths[1])
				   || ' | ' || rpad(backup_details[(3*i)+2], field_lengths[2]);
	END LOOP;
END IF;
-- RAISE NOTICE '%', msg;

RETURN msg;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;


---------------------------------------------------------------------------------------

DROP FUNCTION conditionally_drop_function(varchar,varchar);

