SET search_path TO public;

-- support routine which validates if table exist or not
CREATE OR REPLACE FUNCTION is_table_exists(pschemaname "varchar",ptablename "varchar")
RETURNS boolean AS
$BODY$BEGIN	
-- table names are case insensitive, but where varchar compare function is
 return ((SELECT COUNT(*) FROM pg_catalog.pg_tables WHERE lower(schemaname)=lower(pschemaname) AND lower(tablename)=lower(ptablename))>0);
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

-- support routine which validates if table exist or not
CREATE OR REPLACE FUNCTION is_seq_exists(pschemaname "varchar",pseqname "varchar")
RETURNS boolean AS
$BODY$BEGIN	
-- table names are case insensitive, but where varchar compare function is
 return ((SELECT COUNT(*) FROM pg_catalog.pg_statio_user_sequences WHERE lower(schemaname)=lower(pschemaname) AND lower(relname)=lower(pseqname))>0);
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


-------------------------------------------------------------------------------------------------------
-- "world" installation

-- this routine creates bp,bpexch schemas and roles, also creates dbmirror tables in public
-- should be called before other tasks
CREATE OR REPLACE FUNCTION create_world()
RETURNS int4 AS
$BODY$DECLARE
 res int4;
BEGIN	
	res:=0;
-- Create schemas if missing
	IF (SELECT COUNT(*) FROM pg_catalog.pg_namespace WHERE nspname='bp')=0 THEN 
		CREATE SCHEMA bp;
	END IF; 

	IF (SELECT COUNT(*) FROM pg_catalog.pg_namespace WHERE nspname='bpexch')=0 THEN 
		CREATE SCHEMA bpexch;
	END IF; 

-- Create Roles 	
	IF (SELECT COUNT(*) FROM pg_catalog.pg_user WHERE usename='wguest')=0 THEN
		CREATE ROLE wguest LOGIN PASSWORD 'wguest' NOSUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE;
		GRANT USAGE ON SCHEMA bp TO wguest;
	END IF; 

	IF (SELECT COUNT(*) FROM pg_catalog.pg_group WHERE groname='bpexch')=0 THEN
		CREATE ROLE bpexch NOSUPERUSER NOINHERIT CREATEDB NOCREATEROLE;
		-- Dont really need this since this permissions are set on role level
		GRANT USAGE ON SCHEMA bp TO GROUP bpexch;
		GRANT USAGE ON SCHEMA bpexch TO GROUP bpexch;
	END IF;

-- DBMirror setup ( will go to bp schema after 2.1.5)
-- CREATE OR REPLACE FUNCTION "recordchange" () RETURNS trigger
--	AS '$libdir/pending', 'recordchange'	
-- LANGUAGE C;

-- Create public tables 
-- after 2.1.5 tables will be created inside each workspace schema with createdbmirrortables sp
IF NOT is_table_exists('public','dbmirror_MirrorHost') THEN
	CREATE TABLE dbmirror_MirrorHost (
	    MirrorHostId serial PRIMARY KEY,
	    SlaveName varchar NOT NULL
	);
	INSERT INTO dbmirror_MirrorHost(SlaveName) VALUES ('vault');
END IF;

IF NOT is_table_exists('public','dbmirror_Pending') THEN 
	CREATE TABLE dbmirror_Pending (
	    SeqId serial PRIMARY KEY,
	    TableName name NOT NULL,
	    Op character,
	    XID integer NOT NULL
	);		
	CREATE INDEX dbmirror_Pending_XID_Index ON dbmirror_Pending (XID);
END IF;

IF NOT is_table_exists('public','dbmirror_PendingData') THEN
	CREATE TABLE public.dbmirror_PendingData (
	    SeqId integer NOT NULL,
	    IsKey boolean NOT NULL,
	    Data varchar,
	    PRIMARY KEY (SeqId, IsKey) ,
	    FOREIGN KEY (SeqId) REFERENCES dbmirror_Pending (SeqId) ON UPDATE CASCADE  ON DELETE CASCADE
	);
END IF;

IF NOT is_table_exists('public','dbmirror_MirroredTransaction') THEN 
	CREATE TABLE dbmirror_MirroredTransaction (
	    XID integer NOT NULL,
	    LastSeqId integer NOT NULL,
	    MirrorHostId integer NOT NULL,
	    PRIMARY KEY (XID, MirrorHostId),
	    FOREIGN KEY (MirrorHostId) REFERENCES dbmirror_MirrorHost (MirrorHostId) ON UPDATE CASCADE ON DELETE CASCADE,
	    FOREIGN KEY (LastSeqId) REFERENCES dbmirror_Pending (SeqId)  ON UPDATE CASCADE ON DELETE CASCADE
	);
END IF;

GRANT ALL PRIVILEGES ON dbmirror_MirroredTransaction TO public;
GRANT ALL PRIVILEGES ON dbmirror_PendingData TO public;
GRANT ALL PRIVILEGES ON dbmirror_Pending TO public;
GRANT ALL PRIVILEGES ON dbmirror_MirrorHost TO public; 
GRANT ALL PRIVILEGES ON dbmirror_mirrorhost_mirrorhostid_seq TO public;
GRANT ALL PRIVILEGES ON dbmirror_pending_seqid_seq TO public;
	
RETURN res;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

-- create bp_world
SELECT create_world();
-- cleanup
DROP FUNCTION create_world();
--===============================================================================================--


----------------------------------------------------------------------------------------------------
-- bp schema installation
SET search_path TO bp,public;

-- this routine creates bp tables and init it
-- should be called before other tasks
CREATE OR REPLACE FUNCTION build_bp()
RETURNS int4 AS
$BODY$DECLARE
 res int4;
BEGIN	
	res:=0;

-- Create tables
IF NOT is_table_exists('bp','WorkSpaceTypeRef') THEN
	create table WorkSpaceTypeRef ( 
		workspacetypeid INTEGER not null,
		description VARCHAR(80) null, constraint WorkSpaceTypeRef_PK primary key (workspacetypeid) );
	GRANT SELECT ON TABLE workspacetyperef TO public;
	insert into WorkSpaceTypeRef (workspacetypeid,description) values(1,'Exchange workspace');
END IF;

IF NOT is_table_exists('bp','Workspace') THEN
	create table Workspace ( 
		workspacename VARCHAR(80) not null,
		dpuid VARCHAR(20) not null,
		workspacetypeid INTEGER not null,
		description VARCHAR(128) null,
		creationdate TIMESTAMP null,
		transferenabled BOOL null,
		lastreceivedmaster TIMESTAMP null,
		lastreceiveddiff TIMESTAMP null,
		isvaulted BOOL not null DEFAULT false, constraint Workspace_PK primary key (workspacename, workspacetypeid, dpuid) );
	GRANT SELECT ON TABLE workspace TO public;
	-- Add foreign key constraints to table Workspace.
	alter table Workspace
		add constraint WorkSpaceTypeRef_Workspace_FK1 foreign key (
			workspacetypeid)
		 references WorkSpaceTypeRef (
			workspacetypeid);
END IF;

IF NOT is_table_exists('bp','EventType') THEN
	create table EventType ( 
		EventTypeId INTEGER not null,
		Description VARCHAR(100) null, constraint EventType_PK primary key (EventTypeId) );
	GRANT SELECT ON TABLE EventType TO public;
END IF;

IF NOT is_table_exists('bp','EventLevel') THEN
	create table EventLevel ( 
		EventLevelId INTEGER not null,
		Description VARCHAR(16) null, constraint EventLevel_PK primary key (EventLevelId) );  
	GRANT SELECT ON TABLE EventLevel TO public;
	INSERT INTO eventlevel(eventlevelid, description) VALUES(0,'FATAL');
	INSERT INTO eventlevel(eventlevelid, description) VALUES(1,'ERROR');
	INSERT INTO eventlevel(eventlevelid, description) VALUES(2,'WARNING');
	INSERT INTO eventlevel(eventlevelid, description) VALUES(3,'INFO');
	INSERT INTO eventlevel(eventlevelid, description) VALUES(4,'DEBUG');
END IF;


IF NOT is_table_exists('bp','EventGroup') THEN
	create table EventGroup ( 
		EventGroupId INTEGER not null,
		Description VARCHAR(100) null,
		CreationDate TIMESTAMP null,
		ParentEventGroupId INTEGER null, constraint EventGroup_PK primary key (EventGroupId) );
END IF;

IF NOT is_table_exists('bp','EventLog') THEN
	create table EventLog ( 
		EventId INTEGER not null,
		EventLevel INTEGER null,
		EventTypeId INTEGER null,
		EventDate TIMESTAMP null,
		Message VARCHAR(1024) null,
		Source VARCHAR(64) null,
		Host VARCHAR(64) null,
		EventGroupId INTEGER null, constraint EventLog_PK primary key (EventId) );	
	-- Add the remaining keys, constraints and indexes for the table EventLog.
	create index EventLog_EventGroupId_Idx on EventLog (
		EventGroupId );
	create index EventLog_EventDate_Idx on EventLog (
		EventDate );
END IF;

IF NOT is_table_exists('bp','config') THEN
	CREATE TABLE config (
	  key varchar(64) NOT NULL PRIMARY KEY,
	  value varchar(64)
	);	
	INSERT INTO CONFIG VALUES ('vault_transfer_enabled','yes');
	INSERT INTO CONFIG VALUES ('version','2.1.5');
	INSERT INTO CONFIG VALUES ('build','1');
END IF;

IF NOT is_seq_exists('bp','eventlog_id_seq') THEN
	-- Sequence: eventlog_id_seq
	-- DROP SEQUENCE eventlog_id_seq;
	CREATE SEQUENCE eventlog_id_seq
	  INCREMENT 1
	  MINVALUE 1
	  MAXVALUE 9223372036854775807
	  START 1
	  CACHE 1;
END IF;

IF NOT is_seq_exists('bp','eventgroup_id_seq') THEN 
	-- Sequence: eventgroup_id_seq
	-- DROP SEQUENCE eventgroup_id_seq;
	CREATE SEQUENCE eventgroup_id_seq
	  INCREMENT 1
	  MINVALUE 1
	  MAXVALUE 9223372036854775807
	  START 1
	  CACHE 1;
	-- This is the end of the Microsoft Visual Studio generated SQL DDL script.
END IF;

RETURN res;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

SELECT build_bp();
DROP FUNCTION build_bp();

-- creates record in event_type table if such id does not exist, overwise update description
CREATE OR REPLACE FUNCTION create_event_type(peventtypeid int4, pdescription "varchar")
RETURNS int4 AS
$BODY$DECLARE
 res int4;
BEGIN	
	IF (SELECT COUNT(*) FROM eventtype WHERE eventtypeid=peventtypeid) = 0 THEN 
		INSERT INTO eventtype(eventtypeid, description) VALUES(peventtypeid,pdescription);
	ELSE 
		UPDATE eventtype SET description=pdescription WHERE eventtypeid=peventtypeid;
	END IF;
	res:=0;
	RETURN res;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

SELECT create_event_type(1001,'There are no items to backup with current server / workspace');
SELECT create_event_type(1002,'Backup Operation was cancelled by the user');
SELECT create_event_type(1003,'Backup Operation was successful');
SELECT create_event_type(1004,'Backup warning');
SELECT create_event_type(1005,'Operation started');
SELECT create_event_type(1006,'Operation failed');
SELECT create_event_type(1007,'Mailbox checkpoint out of sync');
SELECT create_event_type(1008,'Public folder checkpoint out of sync');
SELECT create_event_type(1009,'Public folder checkpoint holder was changed');
SELECT create_event_type(1010,'Disaster recovery started');
SELECT create_event_type(1011,'Disaster recovery failed');
SELECT create_event_type(1012,'Disaster recovery successful');
SELECT create_event_type(1013,'Disaster recovery warning');
SELECT create_event_type(2000, 'Processing was successful');
SELECT create_event_type(2001, 'Transfer to the vault is currently disabled');
SELECT create_event_type(2002, 'Error connecting to database');
SELECT create_event_type(2003, 'Could not open output file for writing');
SELECT create_event_type(2004, 'Error executing a query');

DROP FUNCTION create_event_type(peventtypeid int4, pdescription "varchar");

-- function which creates dbmirror tables in CURRENT schema so search path should be set properly
CREATE OR REPLACE FUNCTION bp.createdbmirrortables()
  RETURNS int4 AS
$BODY$DECLARE
-- oldschemaname varchar;	
BEGIN

CREATE TABLE dbmirror_MirrorHost (
    MirrorHostId serial PRIMARY KEY,
    SlaveName varchar NOT NULL
);

CREATE TABLE dbmirror_Pending (
    SeqId serial PRIMARY KEY,
    TableName name NOT NULL,
    Op character,
    XID integer NOT NULL
);

CREATE INDEX dbmirror_Pending_XID_Index ON dbmirror_Pending (XID);

CREATE TABLE dbmirror_PendingData (
    SeqId integer NOT NULL,
    IsKey boolean NOT NULL,
    Data varchar,
    PRIMARY KEY (SeqId, IsKey) ,
    FOREIGN KEY (SeqId) REFERENCES dbmirror_Pending (SeqId) ON UPDATE CASCADE  ON DELETE CASCADE
);

CREATE TABLE dbmirror_MirroredTransaction (
    XID integer NOT NULL,
    LastSeqId integer NOT NULL,
    MirrorHostId integer NOT NULL,
    PRIMARY KEY (XID, MirrorHostId),
    FOREIGN KEY (MirrorHostId) REFERENCES dbmirror_MirrorHost (MirrorHostId) ON UPDATE CASCADE ON DELETE CASCADE,
    FOREIGN KEY (LastSeqId) REFERENCES dbmirror_Pending (SeqId)  ON UPDATE CASCADE ON DELETE CASCADE
);


create table dbmirror_HostStatus ( 
	MirrorHostId integer not null,
	IsMasterDone boolean NOT NULL DEFAULT FALSE,
	MasterDate TIMESTAMP null, constraint dbmirror_HostStatus_PK primary key (MirrorHostId),
	FOREIGN KEY (MirrorHostId) REFERENCES dbmirror_MirrorHost (MirrorHostId) ON UPDATE CASCADE ON DELETE CASCADE
 );  

 RETURN 0;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION bp.createeventgroup(pglobal "bool",pdescripttion "varchar")
RETURNS int4 AS
$BODY$DECLARE
 evgrid int4;
BEGIN	
        IF pglobal Then  
		evgrid:=nextval('bp.eventgroup_id_seq');
		INSERT INTO bp.eventgroup(eventgroupid,description,creationdate) 
			VALUES(evgrid,pdescripttion,now());
	ELSE 
		evgrid:=nextval('eventgroup_id_seq');
		INSERT INTO eventgroup(eventgroupid,description,creationdate) 
			VALUES(evgrid,pdescripttion,now());		
	END IF;
	RETURN evgrid;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION bp.addeventlog(pglobal "bool",plevel "int4", pevtype "int4", psource "varchar", pmsg "varchar", phost "varchar", pEventGroupId int4)
RETURNS int4 AS
$BODY$BEGIN
       IF pglobal Then  
		INSERT INTO bp.EventLog(eventid,EventLevel,EventTypeId,Source,Message,eventdate,Host,EventGroupId ) 
			VALUES(nextval('bp.eventlog_id_seq'),plevel,pevtype,substring(psource,1,64),
			substring(pmsg,1,1023),now(),substring(phost,1,64),pEventGroupId);
	ELSE 
		INSERT INTO EventLog(eventid,EventLevel,EventTypeId,Source,Message,eventdate,Host,EventGroupId ) 
			VALUES(nextval('eventlog_id_seq'),plevel,pevtype,substring(psource,1,64),
			substring(pmsg,1,1023),now(),substring(phost,1,64),pEventGroupId);
	END IF;
	RETURN 0;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

-- DROP FUNCTION bp.convert_to_boolean(pparam "varchar");
CREATE OR REPLACE FUNCTION bp.convert_to_boolean(pparam "varchar")
  RETURNS bool AS
$BODY$BEGIN 
RETURN upper(pparam) IN ('YES', 'Y','T', 'TRUE', '1');
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

--===============================================================================================--


----------------------------------------------------------------------------------------------------
-- bpexch schema installation
SET search_path TO bpexch,public;

-- this routine creates bp tables and init it
-- should be called before other tasks
CREATE OR REPLACE FUNCTION build_bpexch()
RETURNS int4 AS
$BODY$DECLARE
 res int4;
BEGIN	
	res:=0;

-- Create tables
IF NOT is_table_exists('bpexch','VaultedTable') THEN
	-- contains all tables need to be locked or dbmirror triggers set 
	CREATE TABLE VaultedTable(OrderNo int4,TblName varchar(100) not null primary key); 
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(1,'EventGroup');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(2,'VaultedSettings');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(3,'ExclusionList');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(4,'EventLog');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(5,'FolderTree');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(6,'Folder');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(7,'FolderMap');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(8,'FolderVersion');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(9,'Item');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(10,'ItemMap');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(11,'ItemVersion');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(12,'Content');
	INSERT INTO VaultedTable(OrderNo,TblName) VALUES(13,'DataBank');
END IF;

RETURN res;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

SELECT build_bpexch();
DROP FUNCTION build_bpexch();

CREATE OR REPLACE FUNCTION bpexch.createowner(prole "varchar", ppassword "varchar")
  RETURNS int4 AS
$BODY$BEGIN
  EXECUTE 'CREATE ROLE ' || quote_ident(prole) || ' LOGIN PASSWORD ' || quote_literal(ppassword) || ' 
    NOINHERIT
     VALID UNTIL ''infinity'' IN ROLE bpexch';
  EXECUTE 'GRANT USAGE ON SCHEMA bpexch TO ' || quote_ident(prole);
  EXECUTE 'GRANT USAGE ON SCHEMA bp TO ' || quote_ident(prole); 
  RETURN 0;
END $BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION bpexch.getworkspaceschema(pworkspacename "varchar", pdpuid "varchar")
  RETURNS "varchar" AS
$BODY$BEGIN
 RETURN pworkspacename || '_' || pdpuid || '_1';
END $BODY$
  LANGUAGE 'plpgsql' VOLATILE;
-- Let this function be called by bpexch usage roles (all exchange apps)
GRANT EXECUTE ON FUNCTION bpexch.getworkspaceschema("varchar", "varchar") TO public;



-- Function: bpexch.initworkspace(pworkspaceschema "varchar", pdomain "varchar", pnetpath "varchar", psambalogin "varchar", psambapassword "varchar")
-- DROP FUNCTION bpexch.initworkspace(pworkspaceschema "varchar", pdomain "varchar", pnetpath "varchar", psambalogin "varchar", psambapassword "varchar");
CREATE OR REPLACE FUNCTION bpexch.initworkspace(pdomain "varchar", pnetpath "varchar",pserverpath "varchar",psambalogin "varchar", psambapassword "varchar")
  RETURNS int4 AS
$BODY$DECLARE
 i int4;
BEGIN
	-- RAISE NOTICE 'bpexch.initworkspace: %','SET search_path TO ' || quote_literal(pworkspaceschema);
	--EXECUTE 'SET search_path TO ' || quote_ident(pworkspaceschema);   
	DELETE FROM VaultedSettings; 
	DELETE FROM NotVaultedSettings;
	INSERT INTO VaultedSettings(VaultedKey,VaultedValue) VALUES ('Version','1.0.1');
	INSERT INTO VaultedSettings(VaultedKey,VaultedValue) VALUES ('DomainName',pdomain);
	INSERT INTO NotVaultedSettings(NotVaultedKey,NotVaultedValue) VALUES ('SharePath',pnetpath);
	INSERT INTO NotVaultedSettings(NotVaultedKey,NotVaultedValue) VALUES ('ServerPath',pserverpath);
	INSERT INTO NotVaultedSettings(NotVaultedKey,NotVaultedValue) VALUES ('NDaysKeepBeforePurge','100');
	INSERT INTO NotVaultedSettings(NotVaultedKey,NotVaultedValue) VALUES ('SambaLogin', psambalogin);
	INSERT INTO NotVaultedSettings(NotVaultedKey,NotVaultedValue) VALUES ('SambaPassword', psambapassword);        
	RETURN 0;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

-- Function: bpexch.createexchworkspace(pworkspacename "varchar", pdpuid "varchar", pdescription "varchar", prole "varchar")
-- DROP FUNCTION bpexch.createexchworkspace(pworkspacename "varchar", pdpuid "varchar", pdescription "varchar", prole "varchar");
-- create exchange workspace, dbmirror tables inside it. 
-- ! Warning ! sets search_path to new schema
CREATE OR REPLACE FUNCTION bpexch.createworkspace(pworkspacename "varchar", pdpuid "varchar", pdescription "varchar", prole "varchar")
  RETURNS "varchar" AS
$BODY$DECLARE
 i int4;
 schemaname varchar;	
BEGIN
 -- Check if workspace is not exist
 SELECT COUNT(*) INTO i FROM bp.workspace 
   WHERE workspacename=pworkspacename AND dpuid=pdpuid AND workspacetypeid=1;
 IF i>0 Then
   RAISE NOTICE 'Schema for %', pworkspacename || ':' || pdpuid || ' already exists in bp.workspace table';
   RETURN NULL	;
 END IF; 

 schemaname := bpexch.getworkspaceschema(lower(pworkspacename), lower(pdpuid));

 INSERT INTO bp.workspace(workspacename,dpuid,workspacetypeid,description,creationdate,isvaulted) 
	VALUES (lower(pworkspacename),lower(pdpuid),1,pdescription,now(),false);

--CREATE SCHEMA schemaname AUTHORIZATION prole;
RAISE NOTICE 'EXECUTE %','CREATE SCHEMA ' || quote_ident(schemaname) || ' AUTHORIZATION ' || quote_ident(prole);
 EXECUTE 'CREATE SCHEMA ' || lower(quote_ident(schemaname)) || ' AUTHORIZATION ' || quote_ident(prole);

 EXECUTE 'SET ROLE ' || quote_ident(prole);

 EXECUTE 'SET search_path TO ' || lower(quote_ident(schemaname));

create table EventGroup ( 
	EventGroupId INTEGER not null,
	Description VARCHAR(100) null,
	CreationDate TIMESTAMP null,
	ParentEventGroupId INTEGER null, constraint EventGroup_PK primary key (EventGroupId) );


create table VaultedSettings ( 
	VaultedKey VARCHAR(64) not null,
	VaultedValue VARCHAR(100) null, constraint VaultedSettings_PK primary key (VaultedKey) );

create table ExclusionList ( 
	FolderTreeId VARCHAR(50) not null,
	ExclusionPath VARCHAR(255) not null, constraint ExclusionList_PK primary key (FolderTreeId, ExclusionPath) );


create table EventLog ( 
	EventId INTEGER not null,
	EventLevel INTEGER null,
	EventTypeId INTEGER null,
	EventDate TIMESTAMP null,
	Message VARCHAR(1024) null,
	Source VARCHAR(64) null,
	Host VARCHAR(64) null,
	EventGroupId INTEGER null, constraint EventLog_PK primary key (EventId) );

create table FolderMap ( 
	RealFolderId VARCHAR(110) not null,
	FolderId INTEGER,
	FolderCheckPoint VARCHAR(255) null,
	Status INTEGER null,
	LockTill TIMESTAMP null,
	RealFolderTreeId VARCHAR(40) null, constraint FolderMap_PK primary key (RealFolderId) );

 
create table Item ( 
	Itemid int8 not null,
	LastVersion INTEGER null,
	Status INTEGER null,
	FolderId INTEGER null,
	CreationDate TIMESTAMP null,
	DeletionDate TIMESTAMP null,
	FolderTreeId INTEGER null, constraint Item_PK primary key (Itemid) );


create table Folder ( 
	FolderId INTEGER not null,
	LastVersion INTEGER null,
	HasSub BOOL null,
	ItemCount INTEGER null,
	ItemSize int8 null,
	Status INTEGER null,
	FolderType INTEGER null,
	CreationDate TIMESTAMP null,
	DeletionDate TIMESTAMP null,
	FolderTreeId INTEGER null, constraint Folder_PK primary key (FolderId) );


create table ItemMap ( 
	RealItemId VARCHAR(160) not null,
	ItemId int8 null,
	RealModifTime TIMESTAMP null,
	Status INTEGER null,
	LockTill TIMESTAMP null,
	RealFolderId VARCHAR(110) null,
	RealFolderTreeId VARCHAR(40) null, constraint ItemMap_PK primary key (RealItemId) );


create table ItemVersion ( 
	ItemId int8 not null,
	ItemVersion INTEGER not null,
	ModifTime TIMESTAMP null,
	ContentId int8 null, constraint Messages_PK primary key (ItemId, ItemVersion) );


create table FolderVersion ( 
	FolderId INTEGER not null,
	FolderVersion INTEGER not null,
	ParentId INTEGER null,
	Name VARCHAR(100) null,
	FullPath VARCHAR(255) null,
	ModifTime TIMESTAMP null, constraint Folders_PK primary key (FolderId, FolderVersion) );


create table Content ( 
	ContentId int8 not null,
	GUIDField VARCHAR(128) null,
	CheckSum VARCHAR(64) null,
	DataBankId INTEGER null,
	BinaryOffset INTEGER null,
	SizeOnDisk INTEGER null,
	ContentSize INTEGER null,
	ItemType INTEGER null,
	FromField VARCHAR(100) null,
	Subject VARCHAR(100) null,
	DateField TIMESTAMP null,
	HasAttachments BOOL null,
	ToField VARCHAR(100) null, constraint Content_PK primary key (ContentId) );


create table FolderTree ( 
	FolderTreeId INTEGER not null,
	RealFolderTreeId VARCHAR(40) not null,
	DistinguishedName VARCHAR(512) null,
	DisplayName VARCHAR(128) null,
	DBName VARCHAR(128) null,
	InformationStore VARCHAR(100) null,
	StorageGroup VARCHAR(100) null,
	Server VARCHAR(100) null,
	SID VARCHAR(64) null,
	Status INTEGER null,
	CreationDate TIMESTAMP null,
	DeletionDate TIMESTAMP null,
	TreeType INTEGER null,
	TreeCheckPoint VARCHAR(255) null,
	MailNickname VARCHAR(128) null, constraint Mailbox_PK primary key (FolderTreeId) );


create table NotVaultedSettings ( 
	NotVaultedKey VARCHAR(64) not null,
	NotVaultedValue VARCHAR(100) null, constraint NotVaultedSettings_PK primary key (NotVaultedKey) );


create table DataBank ( 
	DataBankId INTEGER not null,
	Path VARCHAR(255) null,
	Status INTEGER null,
	DeviceId INTEGER null,
	FromDate DATE null,
	ToDate DATE null,
	LastInsertDate TIMESTAMP null,
	ByteSize int8 null, constraint DataBank_PK primary key (DataBankId) );

-- Add the remaining keys, constraints and indexes for the table EventLog.
create index EventLog_EventDate_Idx on EventLog (
	EventDate ); 


create index EventLog_EventGroupId_Idx on EventLog (
	EventGroupId );  


-- Add the remaining keys, constraints and indexes for the table FolderMap.
create index FolderMap_FolderId_Idx on FolderMap (
	FolderId );  


create index FolderMap_RealFolderTreeId_Idx on FolderMap (
	RealFolderTreeId );  


-- Add the remaining keys, constraints and indexes for the table Item.
create index Item_FolderId_Idx on Item (
	FolderId );  


create index Item_FolderTreeId_Idx on Item (
	FolderTreeId );  


-- Add the remaining keys, constraints and indexes for the table Folder.
create index Folder_FolderTreeId_Idx on Folder (
	FolderTreeId );  


-- Add the remaining keys, constraints and indexes for the table ItemMap.
create index ItemMap_ItemId_Idx on ItemMap (
	ItemId ); 


create index ItemMap_RealFolderId_Idx on ItemMap (
	RealFolderId ); 


create index ItemMap_RealFolderTreeId_Idx on ItemMap (
	RealFolderTreeId );  


-- Add the remaining keys, constraints and indexes for the table ItemVersion.
create index ItemVersion_ContentId_Idx on ItemVersion (
	ContentId );  


-- Add the remaining keys, constraints and indexes for the table FolderVersion.
create index FolderVersion_ParentId on FolderVersion (
	ParentId );  


-- Add the remaining keys, constraints and indexes for the table Content.
create index Content_GUID_CheckSum_Idx on Content (
	GUIDField ,
	CheckSum ); 


create index Content_DataBankId_Idx on Content (
	DataBankId );  


create index Content_DateField_Idx on Content (
	DateField );  


create index Content_ItemType_Idx on Content (
	ItemType );  


-- Add the remaining keys, constraints and indexes for the table FolderTree.
create unique index FolderTree_RealFolderTreeID_idx on FolderTree (
	RealFolderTreeId);  


-- Add the remaining keys, constraints and indexes for the table DataBank.
create index DataBank_FromDate_Idx on DataBank (
	FromDate );  


-- Add foreign key constraints to table FolderMap.
alter table FolderMap
	add constraint Folder_FolderMap_FK1 foreign key (
		FolderId)
	 references Folder (
		FolderId)  
	on update cascade on delete cascade;

alter table FolderMap
	add constraint FolderTree_FolderMap_FK1 foreign key (
		RealFolderTreeId)
	 references FolderTree (
		RealFolderTreeId)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table Item.
alter table Item
	add constraint Folder_Item_FK1 foreign key (
		FolderId)
	 references Folder (
		FolderId)  
	on update cascade on delete cascade;

alter table Item
	add constraint FolderTree_Item_FK1 foreign key (
		FolderTreeId)
	 references FolderTree (
		FolderTreeId)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table Folder.
alter table Folder
	add constraint FolderTree_Folder_FK1 foreign key (
		FolderTreeId)
	 references FolderTree (
		FolderTreeId)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table ItemMap.
alter table ItemMap
	add constraint Item_ItemMap_FK1 foreign key (
		ItemId)
	 references Item (
		Itemid)  
	on update cascade on delete cascade;

alter table ItemMap
	add constraint FolderMap_ItemMap_FK1 foreign key (
		RealFolderId)
	 references FolderMap (
		RealFolderId)  
	on update cascade on delete cascade;

alter table ItemMap
	add constraint FolderTree_ItemMap_FK1 foreign key (
		RealFolderTreeId)
	 references FolderTree (
		RealFolderTreeId)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table ItemVersion.
alter table ItemVersion
	add constraint Item_ItemVersion_FK1 foreign key (
		ItemId)
	 references Item (
		Itemid)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table FolderVersion.
alter table FolderVersion
	add constraint Folder_FolderVersion_FK1 foreign key (
		FolderId)
	 references Folder (
		FolderId)  
	on update cascade on delete cascade;

-- Add foreign key constraints to table Content.
alter table Content
	add constraint DataBank_Content_FK1 foreign key (
		DataBankId)
	 references DataBank (
		DataBankId)
	on update cascade on delete cascade;
  
-- Create new views.
-- Create view SettingsView.
create view SettingsView ( SettingsKey, SettingsValue) as 
	select VaultedKey, VaultedValue FROM VaultedSettings
	UNION 
	select NotVaultedKey,NotVaultedValue from NotVaultedSettings;

-- Sequence: item_id_seq
-- DROP SEQUENCE item_id_seq;

CREATE SEQUENCE item_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: folder_id_seq
-- DROP SEQUENCE folder_id_seq;

CREATE SEQUENCE folder_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: folder_tree_id_seq
-- DROP SEQUENCE folder_tree_id_seq;

CREATE SEQUENCE folder_tree_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: databank_id_seq
-- DROP SEQUENCE databank_id_seq;

CREATE SEQUENCE databank_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: content_id_seq
-- DROP SEQUENCE content_id_seq;
CREATE SEQUENCE content_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: eventlog_id_seq
-- DROP SEQUENCE eventlog_id_seq;
CREATE SEQUENCE eventlog_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- Sequence: eventgroup_id_seq
-- DROP SEQUENCE eventgroup_id_seq;
CREATE SEQUENCE eventgroup_id_seq
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;

-- create dbmirror tables
-- PERFORM bp.createdbmirrortables();
RESET ROLE;

 RETURN schemaname;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


-- Setup dbmirror triggers for all vaulted tables
-- TODO: make it generic and move to bp
CREATE OR REPLACE FUNCTION bpexch.setdbmirrortriggers(turnon "bool")
  RETURNS int4 AS
$BODY$DECLARE 
  ref refcursor;
  tbln varchar;
  ssql varchar;
BEGIN
  OPEN ref FOR SELECT TblName FROM bpexch.VaultedTable ORDER BY OrderNo ASC;
  LOOP
    FETCH ref INTO tbln;
    IF NOT FOUND THEN
     EXIT;
    END IF;
    IF turnon THEN
	ssql:= 'CREATE TRIGGER ' || tbln || '_dbmirror_trig' || 
	       ' AFTER INSERT OR DELETE OR UPDATE ON ' || tbln ||
	       ' FOR EACH ROW EXECUTE PROCEDURE public.recordchange()';
    ELSE
	--DELETE FROM dbmirror_pending;
	ssql:='DROP TRIGGER ' || tbln || '_dbmirror_trig ON ' || tbln;
    END IF; 
    EXECUTE ssql;
  END LOOP;
  CLOSE ref;
  RETURN 0;
END $BODY$
  LANGUAGE 'plpgsql' VOLATILE;

-- Setup dbmirror triggers for all vaulted tables
-- TODO: make it generic and move to bp
CREATE OR REPLACE FUNCTION bpexch.lockvaultedtables()
  RETURNS int4 AS
$BODY$DECLARE 
  ref refcursor;
  tbln varchar;
  ssql varchar;
BEGIN
  -- Do it in proper order (OrderNo) otherwise deadlock possible
  OPEN ref FOR SELECT TblName FROM bpexch.VaultedTable ORDER BY OrderNo ASC;
  LOOP
    FETCH ref INTO tbln;
    IF NOT FOUND THEN
     EXIT;
    END IF;
    ssql:='LOCK TABLE ' || tbln || ' IN SHARE MODE';
    EXECUTE ssql;
  END LOOP;
  CLOSE ref;
  RETURN 0;
END $BODY$
  LANGUAGE 'plpgsql' VOLATILE;

--
CREATE OR REPLACE FUNCTION bpexch.getdatabank(pdate "date",pcontentsize "int4")
  RETURNS int4 AS
-- $1 - date for which we are seeking databank
$BODY$DECLARE 
 i int4;
BEGIN
  SELECT databankid INTO i FROM databank WHERE $1 >= fromdate AND $1 < todate AND status=0 AND bytesize + pcontentsize < 2000000000 FOR UPDATE;
  IF i IS NULL Then
	i:=nextval('databank_id_seq');
	INSERT INTO databank(databankid,fromdate,todate,path,status,deviceid,bytesize) 
		VALUES(i,date_trunc('month',$1),date_trunc('month',$1+interval '1 month'),'',0,0,0);
  END IF; 
 RETURN i;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

--
CREATE OR REPLACE FUNCTION bpexch.createcontent(pguidfield "varchar", pchecksum "varchar",pdatabankid int4,pbinaryoffset int4,
	psizeondisk int4,pcontentsize int4,pitemtype int4,pfromfield "varchar",psubject "varchar",pdatefield timestamp,
	phasattachment bool, ptofield "varchar")
  RETURNS int4 AS
-- $1 - GUIDField
-- $2 - Checksum
-- $3 - type
-- $4 - timestamp
-- $5 - databankid
$BODY$DECLARE 
 i int4;
BEGIN
   i:=nextval('content_id_seq');
   INSERT INTO content(contentid,guidfield,checksum,databankid,binaryoffset,sizeondisk,contentsize,itemtype,fromfield,
      subject,datefield,hasattachments,tofield)
     VALUES(i,substring(pguidfield,1,128), pchecksum,pdatabankid,pbinaryoffset,
	psizeondisk,pcontentsize,pitemtype,substring(pfromfield,1,100),substring(psubject,1,100),pdatefield, 
	phasattachment, substring(ptofield,1,100));
 RETURN i;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

--
CREATE OR REPLACE FUNCTION bpexch.createfoldertree(prealfoldertreeid "varchar", pdistinguishedname "varchar",pdisplayname "varchar",
	pDBName "varchar",pinformationstore "varchar",pStorageGroup "varchar",pServer "varchar",pStoreID "varchar",pStatus int4,
	pTreeType int4,pTreeCheckPoint "varchar",pmailnickname "varchar")
  RETURNS int4 AS
$BODY$DECLARE 
 i int4;
BEGIN
   i:=nextval('folder_tree_id_seq');
   INSERT INTO FolderTree(FolderTreeId,RealFolderTreeId,DistinguishedName,DisplayName,
		DBName,InformationStore,StorageGroup,Server,SID,Status,
		CreationDate,TreeType,TreeCheckPoint,mailnickname) 
	VALUES (i,prealfoldertreeid,pdistinguishedname,pdisplayname,
		pDBName,pinformationstore,pStorageGroup ,pServer,pStoreID,pStatus,
		now(),pTreeType,pTreeCheckPoint,pmailnickname);
   RETURN i;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
---

CREATE OR REPLACE FUNCTION bpexch.createfolder(pLastVersion int4,pHasSub boolean,pItemCount int4,pItemSize int8,
		pStatus int4,pFolderType int4,prealfoldertreeid "varchar")
  RETURNS int4 AS
$BODY$DECLARE 
 i int4;
 pftreeid int4;
BEGIN
   i:=nextval('folder_id_seq');
   SELECT foldertreeid INTO pftreeid FROM foldertree WHERE RealFolderTreeId=prealfoldertreeid;
   INSERT INTO Folder(folderid,LastVersion,HasSub,ItemCount,ItemSize,Status,FolderType,CreationDate,FolderTreeId) 
	VALUES(i,pLastVersion,pHasSub,pItemCount,pItemSize,pStatus,pFolderType,now(),pftreeid);
   RETURN i;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
---


CREATE OR REPLACE FUNCTION bpexch.createitem(pLastVersion int4,pStatus int4,pFolderId int4,prealfoldertreeid "varchar")
  RETURNS int8 AS
$BODY$DECLARE 
 i int8;
 pftreeid int4;
BEGIN
   i:=nextval('item_id_seq');
   SELECT foldertreeid INTO pftreeid FROM foldertree WHERE RealFolderTreeId=prealfoldertreeid;
   INSERT INTO Item(ItemId,LastVersion,Status,FolderId,CreationDate,FolderTreeId)
      VALUES(i,pLastVersion,pStatus,pFolderId,now(),pftreeid);
   RETURN i;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
---

CREATE OR REPLACE FUNCTION bpexch.updatefoldermapping(pRealFolderId "varchar",pFolderId int4,pStatus int4,pRealFolderTreeId "varchar")
  RETURNS int4 AS
$BODY$DECLARE 
   pexistingrealfolderid "varchar";
BEGIN	
   SELECT realfolderid INTO pexistingrealfolderid FROM foldermap WHERE FolderId=pFolderId;
   IF pexistingrealfolderid IS NULL Then
	INSERT INTO FolderMap(RealFolderId,FolderId,Status,LockTill,RealFolderTreeId) 
		VALUES(pRealFolderId,pFolderId,pStatus,now(),pRealFolderTreeId);
   ELSE 
	UPDATE FolderMap SET realfolderid=pRealFolderId 
		WHERE FolderId=pFolderId;
   END IF; 
   RETURN 0;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
---

CREATE OR REPLACE FUNCTION bpexch.updateitemmapping(prealitemid "varchar",pitemid int8,prealmodiftime timestamp,
							pstatus int4,prealfolderid "varchar",prealfoldertreeid "varchar")
  RETURNS int4 AS
$BODY$DECLARE 
-- i int8;
  pexistingrealitemid "varchar";
BEGIN
   SELECT realitemid INTO pexistingrealitemid FROM ItemMap Where itemid=pItemId;
   IF pexistingrealitemid IS NULL Then
	INSERT INTO ItemMap(realitemid,itemid,realmodiftime,status,locktill,realfolderid,realfoldertreeid) 
            VALUES(prealitemid, pitemid, prealmodiftime, pstatus, now(), prealfolderid, prealfoldertreeid );
   ELSE 
	UPDATE ItemMap SET realitemid=prealitemid, realmodiftime=prealmodiftime 
		WHERE ItemId=pitemid;
   END IF; 	

   RETURN 0;
END$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

--===============================================================================================--

-- cleanup
DROP FUNCTION public.is_table_exists(pschemaname "varchar", ptablename "varchar");
DROP FUNCTION public.is_seq_exists(pschemaname "varchar", pseqname "varchar");
