-- Add statements/functions for updating the database.
-- This script is primarily used for doing upgrades to the schema
-- in future releases.  It will run on fresh install after the
-- base schema scripts have run.
SET search_path TO public;

--
-- Create table modification functions
--

-- Returns TRUE if the specified index exists
CREATE OR REPLACE FUNCTION index_exists(sch_name VARCHAR,idx_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
-- index names are case insensitive, but VARCHAR compare function is
    RETURN ((
	SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n
    WHERE c.relkind = 'i'
    	AND c.relnamespace = n.oid
		AND lower(n.nspname) = lower(sch_name)
		AND lower(c.relname) = lower(idx_name)
	) > 0);

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Returns TRUE if the specified table column exists
CREATE OR REPLACE FUNCTION column_exists(sch_name VARCHAR,tbl_name VARCHAR,col_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN

	RETURN((
	SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n, pg_catalog.pg_attribute a
	WHERE a.attnum > 0
		AND NOT a.attisdropped
		AND a.attrelid = c.oid
		AND c.relnamespace = n.oid
		AND pg_catalog.pg_table_is_visible(c.oid)
		AND lower(n.nspname) = lower(sch_name)
		AND lower(c.relname) = lower(tbl_name)
		AND lower(a.attname) = lower(col_name)
	) > 0);

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Returns TRUE if the specified view exists
CREATE OR REPLACE FUNCTION view_exists(sch_name VARCHAR,view_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
    RETURN ((
	SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n
    WHERE c.relkind = 'v'
    	AND c.relnamespace = n.oid
		AND lower(n.nspname) = lower(sch_name)
		AND lower(c.relname) = lower(view_name)
	) > 0);
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Renames conflicts for a given table and column name.  Returns the given
-- requested_name so that the function can be used inside INSERT/UPDATE
-- statements.  If something had to be renamed and canonical_obj_name is
-- supplied, an alert specifying the change is raised.
CREATE OR REPLACE FUNCTION clear_name(sch_name VARCHAR, tbl_name VARCHAR, col_name VARCHAR, requested_name VARCHAR, canonical_obj_name VARCHAR)
RETURNS VARCHAR AS
$BODY$
	DECLARE i INTEGER := 1;
	DECLARE qprefix VARCHAR;
	DECLARE qsuffix VARCHAR;
	DECLARE qstr VARCHAR;
	DECLARE name_exists BOOLEAN;
	DECLARE new_name VARCHAR;
	DECLARE alert_str VARCHAR;
	DECLARE first_char VARCHAR;
	DECLARE len INTEGER;
BEGIN
	-- Store query prefix and suffix, the parts before/after the requested name
	qprefix := 'SELECT COUNT(*) > 0 FROM ' || quote_ident(sch_name) || '.' || quote_ident(tbl_name)
	        || ' WHERE ' || quote_ident(col_name) || ' = ';
	qsuffix := ' LIMIT 1';
	qstr := qprefix || quote_literal(requested_name) || qsuffix;

	-- If the name exists, loop through numbers until we have an unused name; else exit
	EXECUTE qstr INTO name_exists;
	IF (name_exists = true) THEN
		LOOP
			-- Set new name: "requested_name-customer" or "requested_name-customer2" are ideal, but we
			-- can go with "requested_name-2" or some other shorthand if we're tight on space
			len := char_length(requested_name);
			new_name := requested_name || '-customer';
			IF (i = 1) THEN
				IF (len >= 31) THEN -- 1 or less chars available, take 31 chars of requested_name and append "2"
					i := 2;
					new_name := substring(requested_name from 1 for 31) || '2';
				ELSE -- 2 or more chars available, take requested_name and append as much of "-customer" as will fit
					new_name := requested_name || substring('-customer' from 1 for (32 - len));
				END IF;
			ELSE
				len := len + char_length(i);
				IF (len >= 32) THEN -- the name and number fill up the field, store as much of name as possible and append i
					new_name := substring(requested_name from 1 for (32 - char_length(i))) || i;
				ELSE -- there is extra room besides name and number, so store name, as much of "-customer" as we can, and i
					new_name := requested_name || substring('-customer' from 1 for (32 - len)) || i;
				END IF;
			END IF;
			qstr := qprefix || quote_literal(new_name) || qsuffix;

			-- Does the name exist?  If not, rename the value
			EXECUTE qstr INTO name_exists;
			IF (name_exists = false) THEN
				qstr := 'UPDATE ' || quote_ident(sch_name) || '.' || quote_ident(tbl_name)
				     || ' SET ' || quote_ident(col_name) || ' = ' || quote_literal(new_name)
				     || ' WHERE ' || quote_ident(col_name) || ' = ' || quote_literal(requested_name);
				EXECUTE qstr;

				-- Raise an alert about renaming this value
				IF (canonical_obj_name != '') THEN
					first_char := substring(canonical_obj_name from 1 for 1);
					alert_str := 'Update: A';
					IF (first_char = 'a' OR first_char = 'e' OR first_char = 'i' OR first_char = 'o' OR first_char = 'u') THEN
						alert_str := alert_str || 'n';
					END IF;
					alert_str := alert_str || ' ' || canonical_obj_name || ' with name "' || requested_name
					          || '" was found and renamed to "' || new_name || '".';
					INSERT INTO bp.alerts(alert_text, source_id, target, severity, created, updated, closed)
					 VALUES (alert_str, (SELECT source_id FROM bp.alert_source WHERE source_name = 'Process'), 'update', 3,
					 (SELECT FLOOR(EXTRACT(EPOCH FROM NOW()))), (SELECT FLOOR(EXTRACT(EPOCH FROM NOW()))), true);
				END IF;

				EXIT; -- Exit loop
			END IF;

			i := i + 1;
		END LOOP;
	END IF;
	RETURN requested_name;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Function to conditionally add new rows to lookup tables if the row doesn't exist already
CREATE OR REPLACE FUNCTION bp.ifn1(
	p_schtbl VARCHAR,
	p_idcol VARCHAR,
	p_idval VARCHAR)
RETURNS VARCHAR AS
$BODY$
	DECLARE qstr VARCHAR;
BEGIN
	qstr := 'INSERT INTO ' || p_schtbl || '(' || p_idcol || ')'
	     || ' SELECT ' || quote_literal(p_idval) 
	     || ' WHERE (SELECT COUNT(*) FROM ' || p_schtbl
	            || ' WHERE ' || p_idcol || ' = ' || quote_literal(p_idval) || ') = 0';
	EXECUTE qstr;
	RETURN p_schtbl||'(\''||p_idval||'\')';
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Function to conditionally add new rows to lookup tables if the row doesn't exist already
CREATE OR REPLACE FUNCTION bp.ifn2(
	p_schtbl VARCHAR,
	p_idcol VARCHAR,
	p_col2 VARCHAR,
	p_idval VARCHAR,
	p_val2 VARCHAR)
RETURNS VARCHAR AS
$BODY$
	DECLARE qstr VARCHAR;
BEGIN
	qstr := 'INSERT INTO ' || p_schtbl || '(' || p_idcol || ', ' || p_col2 || ')'
	     || ' SELECT ' || quote_literal(p_idval) || ', ' || quote_literal(p_val2)
	     || ' WHERE (SELECT COUNT(*) FROM ' || p_schtbl
	            || ' WHERE ' || p_idcol || ' = ' || quote_literal(p_idval) || ') = 0';
	EXECUTE qstr;
	RETURN p_schtbl||'(\''||p_idval||'\', \''||p_val2||'\')';
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;


SET search_path TO bp,public;


--
-- Modify tables
--
CREATE OR REPLACE FUNCTION update_bp()
RETURNS INT4 AS
$BODY$BEGIN

-- Drop views first so that table updates
-- are reflected when views are re-created
IF view_exists('bp', 'restorable_backups') THEN
	DROP VIEW restorable_backups;
END IF;
IF view_exists('bp', 'successful_backups') THEN
	DROP VIEW successful_backups;
END IF;

-- Add is_active and check_timestamp columns to the bp.systems table
IF NOT column_exists('bp', 'systems', 'is_active') THEN
	ALTER TABLE systems ADD COLUMN is_active BOOLEAN NOT NULL DEFAULT TRUE;
END IF;
IF NOT column_exists('bp', 'systems', 'check_timestamp') THEN
	ALTER TABLE systems ADD COLUMN check_timestamp BIGINT NOT NULL DEFAULT 0;
END IF;
IF NOT column_exists('bp', 'systems', 'version') THEN
	ALTER TABLE systems ADD COLUMN version VARCHAR NOT NULL DEFAULT 'Unknown';
END IF;
IF NOT column_exists('bp', 'systems', 'role') THEN
	ALTER TABLE systems ADD COLUMN role VARCHAR NOT NULL DEFAULT 'Unknown';
END IF;
IF NOT column_exists('bp', 'systems', 'storage_id') THEN
	ALTER TABLE systems ADD COLUMN storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id);
END IF;

IF NOT column_exists('bp', 'operation_progress', 'pid_to_kill') THEN
	ALTER TABLE operation_progress ADD COLUMN pid_to_kill BIGINT DEFAULT 0;
END IF;

IF NOT column_exists('bp','node_applications','visible') THEN
	ALTER TABLE node_applications ADD COLUMN visible BOOLEAN NOT NULL DEFAULT TRUE;
END IF;

	RETURN 0;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT update_bp();
DROP FUNCTION update_bp();




--
-- Add new values into lookup tables
--

-- Add new alert types
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '6', 'Disk');
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '7', 'CEP');
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '8', 'DPU');
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '9', 'RAID');
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '10', 'Backup');
SELECT bp.ifn2('alert_source', 'source_id', 'source_name', '11', 'Archiving');

-- Add new sync_type names
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '4', 'LocalDir');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '5', 'SQL');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '6', 'DPUstate');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '7', 'Exchange');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '8', 'MSExch Full');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '9', 'MSExch Incr');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '10', 'MSExch Diff');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '11', 'MSSQL Full');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '12', 'MSSQL Diff');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '13', 'MSSQL Trans');
UPDATE bp.sync_type SET type_name = 'Differential' WHERE type_id = 2
	AND type_name != 'Differential';
UPDATE bp.sync_type SET type_name = 'MSExch Diff' WHERE type_id = 10
	AND type_name != 'MSExch Diff';
UPDATE bp.sync_type SET type_name = 'MSSQL Diff' WHERE type_id = 12
	AND type_name != 'MSSQL Diff';
UPDATE bp.sync_type SET type_name = 'MSSQL Trans' WHERE type_id = 13
	AND type_name != 'MSSQL Trans';
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '14', 'ESX VM Full');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '15', 'ESX VM Diff');
SELECT bp.ifn2('sync_type', 'type_id', 'type_name', '16', 'ESX VM Incr');

-- Add new operation_names values for name_value column
SELECT bp.ifn1('operation_names', 'name_value', 'migrate');

-- Add new values for 3ware migration to disk_status table
SELECT bp.ifn2('disk_status', 'status_id', 'status', '10', 'MIGRATING');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '11', 'MIGRATE-PAUSED');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '12', 'INIT-PAUSED');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '13', 'RECOVERY');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '14', 'UNKNOWN');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '15', 'DCB-DATA-CHECK');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '16', 'DEGRADED-INIT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '17', 'DEGRADED-MGRT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '18', 'DEGRADED-RBLD');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '19', 'DEGRADED-VRFY');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '20', 'DEVICE-ERROR');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '21', 'DGRD-RBLD-INIT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '22', 'DGRD-RBLD-MGRT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '23', 'DGRD-RBLD-VRFY');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '24', 'ECC-ERROR');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '25', 'NOT-SUPPORTED');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '26', 'READ-FAILURE');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '27', 'READ-TIMEOUT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '28', 'REBUILD-INIT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '29', 'REBUILD-MGRT');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '30', 'REBUILD-VERIFY');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '31', 'OFFLINE-JBOD');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '32', 'ORPHAN');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '33', 'SMART-FAILURE');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '34', 'UNSUPP-DCB');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '35', 'UNCONV-DCB');
SELECT bp.ifn2('disk_status', 'status_id', 'status', '36', 'VERIFY-PAUSED');

-- Create pending migration operations
INSERT INTO bp.operation_status (operation_id, subsystem, description, status)
	SELECT 1, 'Securesync',
	    'Transition to using sync needed for BareMetals', 'pending' 
	WHERE NOT EXISTS (SELECT operation_id FROM bp.operation_status
			WHERE operation_id = 1);
INSERT INTO operation_status (operation_id, subsystem, description, status)
	SELECT 2, 'Installation',
		'Migrate file metadata for 5.0.2 schema', 'pending'
	WHERE NOT EXISTS (SELECT TRUE FROM operation_status WHERE operation_id = 2);
INSERT INTO operation_status (operation_id, subsystem, description, status)
	SELECT 3, 'Profiles',
		'Migrate templates and schedules for 5.1 unicode', 'pending'
	WHERE NOT EXISTS (SELECT TRUE FROM operation_status WHERE operation_id = 3);

--
-- Create view of schedules, lcs records, profiles, and option/selection lists
--
CREATE OR REPLACE VIEW schedule_profiles AS
  SELECT schedule_lcs.schedule_id, lcs_client_assoc.*, profiles.profile_path
  FROM schedule_lcs
  RIGHT JOIN lcs_client_assoc USING (lcs_no)
  JOIN (SELECT DISTINCT ON (lcs_no, node_no, type) lcs_no, node_no, type, profile_path
        FROM lcs_client ORDER BY lcs_no, node_no, type, profile_path) AS profiles
    USING (lcs_no, node_no, type);

--
-- Create view of application schedules, profiles, and backup options
--
CREATE OR REPLACE VIEW application_profiles AS
	SELECT DISTINCT application_schedule_id AS schedule_id, node_no,
		summary AS type, instance_id, verify_level, device_name,
		'schedules/' || node_name || '_' || instance_id || '_' ||
		CASE
			WHEN summary=5 THEN 'exchmast_'
			WHEN summary=6 THEN 'exchdiff_'
			WHEN summary=7 THEN 'exchincr_'
			WHEN summary=10 THEN 'mssqlfull_'
			WHEN summary=11 THEN 'mssqldiff_'
			WHEN summary=12 THEN 'mssqltrans_'
			WHEN summary=14 THEN 'vmesxfull_'
			WHEN summary=15 THEN 'vmesxdiff_'
			WHEN summary=16 THEN 'vmesxincr_'
		END || application_schedule_id || '.spr' AS profile_path
	FROM application_schedules JOIN events USING(calendar_id)
		JOIN option_lists USING(option_list_id)
		JOIN schedule_instances USING(application_schedule_id)
		JOIN nodes USING(node_no);


--
-- Populate needed entries in the lookups table
--
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 3, 'The next NODE_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 3) = 0;
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 4, 'The next FILESET_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 4) = 0;
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 5, 'The next TAPE_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 5) = 0;
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 7, 'The next JOB_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 7) = 0;
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 8, 'The next BACKUP_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 8) = 0;
INSERT INTO lookups (lookup_type, lookup_no, lookup_str, misc1)
	SELECT 4, 10, 'The next LCS_NO value to use', 1
	WHERE (SELECT COUNT(*) FROM bp.lookups WHERE lookup_type = 4 AND lookup_no = 10) = 0;


--
-- Add default entries for calendars and schedules.
-- If adding any events or calendars, be sure to adjust the sequence number.
--
-- PostgreSQL database dump created using pg_dump -a -D -t <tablename> >> dumpfile.sql
--

SET check_function_bodies = false;

CREATE OR REPLACE FUNCTION populate_cal_strat()
RETURNS INT4 AS
$BODY$
DECLARE
	Vstr VARCHAR;
BEGIN

IF (SELECT COUNT(*) FROM bp.calendars)=0
THEN

	-- SEQUENCES -- there's room for up to 50 calendars, 50 strategies, and 200 events.
	-- This is to avoid complicated sequence adjustments if any more calendars or
	-- strategies are added, especially if the user has saved any of their own.
	ALTER SEQUENCE bp.strategies_strategy_id_seq RESTART WITH 51;
	ALTER SEQUENCE bp.calendars_calendar_id_seq RESTART WITH 51;
	ALTER SEQUENCE bp.events_event_id_seq RESTART WITH 201;

	-- STRATEGIES
	INSERT INTO strategies (strategy_id, name, description, read_only)
	 VALUES (5, 'Quarterly Master, Weekly Diff',
	         'Master each quarter, Differentials once each week', true);
	INSERT INTO strategies (strategy_id, name, description, read_only)
	 VALUES (8, 'Beginning of Month Master',
	         'Master Backup the first day of each month', true);
	INSERT INTO strategies (strategy_id, name, description, read_only)
	 VALUES (12, 'Weekday Selectives',
	         'Selective backup each weekday', true);

	-- STRATEGY EVENTS
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                     dtstart, freq, "interval", bymonthday, description)
	 VALUES (9, NULL, 5, 1, 1199156400, 4, 3, -1, 'quarterly master');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (10, NULL, 5, 2, 1199156400, 3, NULL, NULL, 'weekly differential');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (13, NULL, 8, 1, 1199156400, 4, NULL, 1, 'monthly master');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (17, NULL, 12, 3, 1199156400, 3, NULL, NULL, 'weekday selectives');

	-- STRATEGY BYDAY
	INSERT INTO byday (event_id, day_no) VALUES (10, 6);
	INSERT INTO byday (event_id, day_no) VALUES (17, 1);
	INSERT INTO byday (event_id, day_no) VALUES (17, 2);
	INSERT INTO byday (event_id, day_no) VALUES (17, 3);
	INSERT INTO byday (event_id, day_no) VALUES (17, 4);
	INSERT INTO byday (event_id, day_no) VALUES (17, 5);

	-- CALENDARS
	INSERT INTO calendars (calendar_id, name, description, family, read_only)
	 VALUES (5, 'Quarterly Master, Weekly Diff',
	         'Master each quarter, Differentials once each week', 'Any', true);
	INSERT INTO calendars (calendar_id, name, description, family, read_only)
	 VALUES (8, 'Beginning of Month Master',
	         'Master Backup the first day of each month', 'Any', true);
	INSERT INTO calendars (calendar_id, name, description, family, read_only)
	 VALUES (12, 'Weekday Selectives',
	         'Selective backup each weekday', 'Any', true);

	-- CALENDAR EVENTS
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (32, 5, NULL, 1, 1199156400, 4, 3, -1, 'quarterly master');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (33, 5, NULL, 2, 1199156400, 3, NULL, NULL, 'weekly differential');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (36, 8, NULL, 1, 1199156400, 4, NULL, 1, 'monthly master');
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (40, 12, NULL, 3, 1199156400, 3, NULL, NULL, 'weekday selectives');

	-- CALENDAR BYDAY
	INSERT INTO byday (event_id, day_no) VALUES (33, 6);
	INSERT INTO byday (event_id, day_no) VALUES (40, 1);
	INSERT INTO byday (event_id, day_no) VALUES (40, 2);
	INSERT INTO byday (event_id, day_no) VALUES (40, 3);
	INSERT INTO byday (event_id, day_no) VALUES (40, 4);
	INSERT INTO byday (event_id, day_no) VALUES (40, 5);
ELSE 
	-- Update "Incr" to "Diff"
	UPDATE strategies SET name = 'Quarterly Master, Weekly Diff', 
	       description = 'Master each quarter, Differentials once each week'
	   WHERE strategy_id = 5 AND name != 'Quarterly Master, Weekly Diff';
	UPDATE events SET description = 'weekly differential'
	   WHERE event_id = 10 AND description != 'weekly differential';
	UPDATE calendars SET name = 'Quarterly Master, Weekly Diff', 
	       description = 'Master each quarter, Differentials once each week'
	   WHERE calendar_id = 5 AND name != 'Quarterly Master, Weekly Diff';
	UPDATE events SET description = 'weekly differential'
	   WHERE event_id = 33 AND description != 'weekly differential';
END IF;

-- Add the "Friday Master, Daily Diff" calendar and strategy
IF (SELECT COUNT(*) FROM bp.calendars WHERE calendar_id = 22) = 0
THEN
	-- Add strategy
	PERFORM clear_name('bp', 'strategies', 'name',
	                   'Friday Master, Daily Incr', 'strategy');
	INSERT INTO bp.strategies (strategy_id, name, description, read_only)
	 VALUES (22, 'Friday Master, Daily Diff',
	 'Master each Friday, Differentials daily during the week', true);
	INSERT INTO bp.events (event_id, calendar_id, strategy_id, summary,
	                       dtstart, freq, "interval", bymonthday, description)
	 VALUES (68, NULL, 22, 1, 1199127600, 3, NULL, NULL, 'Friday master');
	INSERT INTO bp.events (event_id, calendar_id, strategy_id, summary,
	                       dtstart, freq, "interval", bymonthday, description)
	 VALUES (69, NULL, 22, 2, 1199156400, 3, NULL, NULL, 'daily differential');
	INSERT INTO byday (event_id, day_no) VALUES (68, 5);
	INSERT INTO byday (event_id, day_no) VALUES (69, 0);
	INSERT INTO byday (event_id, day_no) VALUES (69, 1);
	INSERT INTO byday (event_id, day_no) VALUES (69, 2);
	INSERT INTO byday (event_id, day_no) VALUES (69, 3);
	INSERT INTO byday (event_id, day_no) VALUES (69, 4);

	-- Add calendar
	PERFORM clear_name('bp', 'calendars', 'name',
	                   'Friday Master, Daily Incr', 'calendar');
	INSERT INTO bp.calendars (calendar_id, name, description, family, read_only)
	 VALUES (22, 'Friday Master, Daily Diff',
	 'Master each Friday, Differentials daily during the week', 'Any', true);
	INSERT INTO bp.events (event_id, calendar_id, strategy_id, summary,
	                       dtstart, freq, "interval", bymonthday, description)
	 VALUES (70, 22, NULL, 1, 1199127600, 3, NULL, NULL, 'Friday master');
	INSERT INTO bp.events (event_id, calendar_id, strategy_id, summary,
	                       dtstart, freq, "interval", bymonthday, description)
	 VALUES (71, 22, NULL, 2, 1199156400, 3, NULL, NULL, 'daily differential');
	INSERT INTO byday (event_id, day_no) VALUES (70, 5);
	INSERT INTO byday (event_id, day_no) VALUES (71, 0);
	INSERT INTO byday (event_id, day_no) VALUES (71, 1);
	INSERT INTO byday (event_id, day_no) VALUES (71, 2);
	INSERT INTO byday (event_id, day_no) VALUES (71, 3);
	INSERT INTO byday (event_id, day_no) VALUES (71, 4);
ELSE 
	UPDATE bp.strategies SET name='Friday Master, Daily Diff',
         description = 'Master each Friday, Differentials daily during the week'
		 WHERE strategy_id = 22 AND description != 'Master each Friday, Differentials daily during the week';
	UPDATE bp.events SET description = 'daily differential' WHERE event_id = 69 AND description != 'daily differential';
	UPDATE bp.calendars SET name = 'Friday Master, Daily Diff',
         description = 'Master each Friday, Differentials daily during the week'
		 WHERE calendar_id = 22 AND description != 'Master each Friday, Differentials daily during the week';
	UPDATE bp.events SET description = 'daily differential' WHERE event_id = 71 AND description != 'daily differential';
	
END IF;

-- Add the "Friday BareMetal" calendar and strategy
IF (SELECT COUNT(*) FROM bp.calendars WHERE calendar_id = 23) = 0
THEN
	-- Add strategy
	PERFORM clear_name('bp', 'strategies', 'name',
	                   'Saturday BareMetal', 'strategy');
	INSERT INTO strategies (strategy_id, name, description, read_only)
	 VALUES (23, 'Saturday BareMetal',
	         'Hot BareMetal backup each week', true);
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (72, NULL, 23, 4, 1199127600, 3, NULL, NULL, 'weekly bare metal');
	INSERT INTO byday (event_id, day_no) VALUES (72, 6);

	-- Add calendar
	PERFORM clear_name('bp', 'calendars', 'name',
	                   'Saturday BareMetal', 'calendar');
	INSERT INTO calendars (calendar_id, name, description, family, read_only)
	 VALUES (23, 'Saturday BareMetal',
	         'Hot BareMetal backup each week', 'Windows', true);
	INSERT INTO events (event_id, calendar_id, strategy_id, summary,
	                    dtstart, freq, "interval", bymonthday, description)
	 VALUES (73, 23, NULL, 4, 1199127600, 3, NULL, NULL, 'weekly bare metal');
	INSERT INTO byday (event_id, day_no) VALUES (73, 6);
ELSE
	-- Strategy and calendar names before 5.0.2 had "Bare Metal"
	PERFORM clear_name('bp', 'strategies', 'name',
	                   'Saturday BareMetal', 'strategy');
	UPDATE bp.strategies SET name = 'Saturday BareMetal',
	 description = 'Hot BareMetal backup each week' WHERE strategy_id = 23;
	UPDATE bp.events SET description = 'weekly baremetal' WHERE event_id = 72;

	PERFORM clear_name('bp', 'calendars', 'name',
	                   'Saturday BareMetal', 'calendar');
	UPDATE bp.calendars SET name = 'Saturday BareMetal',
	 description = 'Hot BareMetal backup each week' WHERE calendar_id = 23;
	UPDATE bp.events SET description = 'weekly baremetal' WHERE event_id = 73;
END IF;


	CREATE OR REPLACE FUNCTION bp.update_cal_strat()
	RETURNS VARCHAR AS
	$UCS_BODY$
	DECLARE
		Vkeep_ids CONSTANT INT4[] := ARRAY[5,8,12,22,23];
	BEGIN

	-- If strategy id 1 exists and is read-only, then update the records
	IF (SELECT read_only = TRUE AS tf
		  FROM bp.strategies WHERE strategy_id = 1) = TRUE
	THEN
		-- Remove all strategies not in the Vkeep_ids array
		DELETE FROM bp.strategies
		 WHERE strategy_id < 50 AND NOT strategy_id = ANY (Vkeep_ids);

		-- Remove all calendars not in the Vkeep_ids array that aren't
		-- referenced.  If they are referenced, mark them as read/write.
		DELETE FROM bp.calendars
		 WHERE calendar_id < 50 AND (NOT calendar_id = ANY (Vkeep_ids))
		 AND calendar_id NOT IN
		  (SELECT calendar_id FROM bp.schedules WHERE calendar_id IS NOT NULL);
		UPDATE bp.calendars SET read_only = FALSE
		 WHERE calendar_id < 50 AND NOT calendar_id = ANY (Vkeep_ids);

		RETURN 'calendars and strategies updated successfully';
	ELSE
		RETURN 'calendars and strategies already updated';
	END IF;

	END;$UCS_BODY$
	LANGUAGE 'plpgsql' VOLATILE;

	SELECT INTO Vstr update_cal_strat();
	RAISE NOTICE '%', Vstr;
	DROP FUNCTION update_cal_strat();

	RETURN 0;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT populate_cal_strat();
DROP FUNCTION populate_cal_strat();
--
-- End of populating calendar and strategy default values
--



--
-- Populate default selection lists
--
CREATE OR REPLACE FUNCTION populate_selection_lists()
RETURNS INT4 AS
$BODY$BEGIN

IF (SELECT COUNT(*) FROM bp.selection_lists)=0
THEN

	-- There's room for up to 100 default selection lists. This is
	-- to avoid complicated sequence adjustments if any more lists 
	-- are added, especially if the user has saved any of their own.
	ALTER SEQUENCE bp.selection_lists_selection_list_id_seq RESTART WITH 101;

	PERFORM clear_name('bp', 'selection_lists', 'name', 'Include home directories (Unix)', 'selection list');
	PERFORM clear_name('bp', 'selection_lists', 'name', 'Include home directories (Linux)', 'selection list');
	PERFORM clear_name('bp', 'selection_lists', 'name', 'Include C: volume', 'selection list');
	PERFORM clear_name('bp', 'selection_lists', 'name', 'Select .pst files', 'selection list');

	INSERT INTO selection_lists (selection_list_id, name, description, type, family, node_no, read_only, netmounts, registry, tempfiles, readmounts, allmounts) VALUES (1, 'Include home directories (Unix)', 'Include home directories on Unix clients', 'inclusion', 'Unix', NULL, false, false, false, false, false, false);
	INSERT INTO selection_list_filenames (selection_list_id, filename) VALUES (1, '@@@:/home');
	INSERT INTO selection_lists (selection_list_id, name, description, type, family, node_no, read_only, netmounts, registry, tempfiles, readmounts, allmounts) VALUES (2, 'Include home directories (Linux)', 'Include home directories on Linux clients', 'inclusion', 'Linux', NULL, false, false, false, false, false, false);
	INSERT INTO selection_list_filenames (selection_list_id, filename) VALUES (2, '@@@:/home');
	INSERT INTO selection_lists (selection_list_id, name, description, type, family, node_no, read_only, netmounts, registry, tempfiles, readmounts, allmounts) VALUES (3, 'Include C: volume', 'Include all files on C: volume', 'inclusion', 'Windows', NULL, false, false, false, false, false, false);
	INSERT INTO selection_list_filenames (selection_list_id, filename) VALUES (3, 'C:/');
	INSERT INTO selection_lists (selection_list_id, name, description, type, family, node_no, read_only, netmounts, registry, tempfiles, readmounts, allmounts) VALUES (4, 'Select .pst files', 'Select all files with extension .pst for inclusion or exclusion', 'any', 'Windows', NULL, false, false, false, false, false, false);
	INSERT INTO selection_list_filenames (selection_list_id, filename) VALUES (4, '*.pst');

END IF;

RETURN 0;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT populate_selection_lists();
DROP FUNCTION populate_selection_lists();
--
-- End of populating default selection lists
--



--
-- Populate default option lists
--
CREATE OR REPLACE FUNCTION populate_option_lists()
RETURNS INT4 AS
$BODY$BEGIN

IF (SELECT COUNT(*) FROM bp.option_lists)=0
THEN

	-- There's room for up to 100 default option lists. This is
	-- to avoid complicated sequence adjustments if any more lists 
	-- are added, especially if the user has saved any of their own.
	ALTER SEQUENCE bp.option_lists_option_list_id_seq RESTART WITH 101;

	PERFORM clear_name('bp', 'option_lists', 'name', 'Default Any', 'option list');
	INSERT INTO option_lists (option_list_id, name, description, type, family, read_only, device_name, read_locking, speed_option, verify_level, depth, comment, raw_device, raw_device_command, before_command, after_command, make_catalog) VALUES (1, 'Default Any', 'Default backup options for all clients', 'backup', 'Any', true, NULL, 0, false, 1, 0, 'Default options', NULL, NULL, NULL, NULL, true);

END IF;

-- Exchange was added in a subsequent version, so use a different test
IF (SELECT TRUE FROM bp.option_lists WHERE option_list_id = 3) IS NULL
THEN

	PERFORM clear_name('bp', 'option_lists', 'name', 'Exchange Master', 'option list');
	PERFORM clear_name('bp', 'option_lists', 'name', 'Exchange Differential', 'option list');
	-- Add Exchange option lists
	INSERT INTO option_lists (option_list_id, name, description, type, family, read_only, device_name, read_locking, speed_option, verify_level, depth, comment, raw_device, raw_device_command, before_command, after_command, make_catalog) VALUES (2, 'Exchange Master', 'Exchange master backup-specific options', 'backup', 'Windows', true, NULL, 0, false, 0, 0, '', NULL, NULL, 'CNT:c:/pcbp/Exchange.dir/ExchMaster.bat', NULL, true);
	INSERT INTO option_lists (option_list_id, name, description, type, family, read_only, device_name, read_locking, speed_option, verify_level, depth, comment, raw_device, raw_device_command, before_command, after_command, make_catalog) VALUES (3, 'Exchange Differential', 'Exchange differential backup-specific options', 'backup', 'Windows', true, NULL, 0, false, 0, 0, '', NULL, NULL, 'CNT:c:/pcbp/Exchange.dir/ExchDiffer.bat', NULL, true);

END IF;

RETURN 0;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT populate_option_lists();
DROP FUNCTION populate_option_lists();


--
-- End of populating default option lists
--


-- Set statistics level of netpaths and detail higher
--ALTER TABLE bp.backup_files ALTER COLUMN node_no SET STATISTICS 100;
--ALTER TABLE bp.backup_files ALTER COLUMN dir_no SET STATISTICS 100;
--ALTER TABLE bp.detail ALTER COLUMN node_no SET STATISTICS 100;
--ALTER TABLE bp.detail ALTER COLUMN dir_no SET STATISTICS 100;


DROP FUNCTION clear_name(sch_name VARCHAR, tbl_name VARCHAR, col_name VARCHAR, requested_name VARCHAR, canonical_obj_name VARCHAR);


--
-- Changes required for 4.2 Application-specific agents
--
-- Function to add a application_lookup row, if the row doesn't exist already
CREATE OR REPLACE FUNCTION bp.add_application_lookup(p_app_id INTEGER, p_name VARCHAR, p_type VARCHAR)
RETURNS VARCHAR AS
$BODY$BEGIN

	INSERT INTO bp.application_lookup(app_id, name, type) SELECT p_app_id, p_name, p_type
	    WHERE (SELECT TRUE FROM bp.application_lookup WHERE app_id = p_app_id) IS NULL;
	RETURN p_app_id||' - '||p_name;

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Add application entries
-- See bpdb.sql for details
SELECT bp.add_application_lookup(1, 'file-level', 'file-level');
SELECT bp.add_application_lookup(2, 'Exchange 2003', 'Exchange');
SELECT bp.add_application_lookup(3, 'Exchange 2007', 'Exchange');
SELECT bp.add_application_lookup(4, 'Exchange 2010', 'Exchange');
SELECT bp.add_application_lookup(21, 'SQL Server 2005',    'SQL Server');
SELECT bp.add_application_lookup(22, 'SQL Server 2008',    'SQL Server');
SELECT bp.add_application_lookup(23, 'SQL Server 2008 R2', 'SQL Server');
SELECT bp.add_application_lookup(30, 'Archive', 'Archive');
SELECT bp.add_application_lookup(40, 'VMware', 'VMware');

DROP FUNCTION bp.add_application_lookup(INTEGER, VARCHAR, VARCHAR);

-- Add client application entries for file-level backups
INSERT INTO bp.node_applications (node_no, app_id, current)
   SELECT n.node_no, 1, TRUE
     FROM bp.nodes AS n LEFT JOIN bp.node_applications AS a ON (n.node_no = a.node_no)
     WHERE a.node_no IS NULL ORDER BY n.node_no;
INSERT INTO bp.application_instances (node_no, app_id)
   SELECT n.node_no, 1
     FROM bp.nodes AS n LEFT JOIN bp.application_instances AS a ON (n.node_no = a.node_no)
     WHERE a.node_no IS NULL ORDER BY n.node_no;


CREATE OR REPLACE FUNCTION update_bp()
RETURNS INT4 AS
$BODY$BEGIN

	-- Backups now refer to application instances
	IF NOT column_exists('bp', 'backups', 'instance_id') THEN
		ALTER TABLE bp.backups ADD COLUMN instance_id INTEGER
			 REFERENCES bp.application_instances (instance_id) ON DELETE SET NULL;
		UPDATE bp.backups SET instance_id = ai.instance_id
		  FROM bp.application_instances AS ai
		  WHERE bp.backups.node_no = ai.node_no AND ai.app_id = 1;
	END IF;

	-- Jobs now refer to application instances
	IF NOT column_exists('bp', 'jobs', 'instance_id') THEN
		ALTER TABLE bp.jobs ADD COLUMN instance_id INTEGER
			 REFERENCES bp.application_instances (instance_id) ON DELETE SET NULL;
		UPDATE bp.jobs SET instance_id = ai.instance_id
		  FROM bp.application_instances AS ai
		  WHERE bp.jobs.node_no = ai.node_no AND ai.app_id = 1;
	END IF;

	-- Support jobs for application schedules
	IF NOT column_exists('bp', 'jobs', 'schedule_id') THEN
		ALTER TABLE bp.jobs ADD COLUMN schedule_id INTEGER;
	END IF;

	-- Client version is stored in the nodes table
	IF NOT column_exists('bp', 'nodes', 'version') THEN
		ALTER TABLE bp.nodes ADD COLUMN version VARCHAR DEFAULT 'Unknown';
	END IF;

	-- Update raid_drives tables
	IF NOT column_exists('bp', 'raid_drives', 'partition') THEN
		ALTER TABLE bp.raid_drives ADD COLUMN partition INTEGER NOT NULL DEFAULT 0;
	END IF;

	-- Update devices table 
	IF NOT column_exists('bp', 'devices', 'is_deleted') THEN
		ALTER TABLE bp.devices ADD COLUMN is_deleted BOOLEAN NOT NULL DEFAULT FALSE;
	END IF;
	IF NOT column_exists('bp', 'devices', 'dedup') THEN
		ALTER TABLE bp.devices ADD COLUMN dedup BOOLEAN NOT NULL DEFAULT TRUE;
	END IF;
	IF NOT column_exists('bp', 'devices', 'storage_id') THEN
    		ALTER TABLE bp.devices ADD COLUMN storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id);
	END IF;

	-- Add default SNMP manager, notifications.unitrends.com
	INSERT INTO bp.snmp_config (destination, enabled)
		SELECT 'notifications.unitrends.com', TRUE
		WHERE NOT EXISTS (SELECT * FROM bp.snmp_config
			WHERE destination = 'notifications.unitrends.com');

	IF index_exists('bp', 'schedule_instances_instance_id_key') THEN
		-- Instances can be present multiple times in some application schedules
		ALTER TABLE bp.schedule_instances DROP CONSTRAINT schedule_instances_instance_id_key;
	END IF;

	IF column_exists('bp', 'archive_profiles', 'is_legacy') THEN
		ALTER TABLE bp.archive_profiles ALTER COLUMN is_legacy DROP DEFAULT;
		ALTER TABLE bp.archive_profiles ALTER COLUMN is_legacy TYPE INTEGER
			USING CASE WHEN is_legacy=FALSE THEN 1 ELSE 0 END;
		ALTER TABLE bp.archive_profiles RENAME COLUMN is_legacy TO revision;
	END IF;

	-- Update bp.backup_files tables from 5.0.2 beta builds
	-- TODO: This can be removed for 5.1
	IF NOT column_exists('bp', 'backup_files', 'end_backup_no') THEN
		DROP INDEX backup_files_backup_dir_idx;
		ALTER TABLE bp.backup_files DISABLE TRIGGER ALL;
		ALTER TABLE bp.backup_files ADD COLUMN end_backup_no INTEGER NOT NULL DEFAULT 0;
		CREATE INDEX backup_files_backup_dir_idx ON bp.backup_files(dir_no, backup_no, end_backup_no);
		ALTER TABLE bp.backup_files ENABLE TRIGGER ALL;
	END IF;

	IF EXISTS(SELECT TRUE FROM pg_catalog.pg_constraint WHERE conname = 'vmware_schedules_esx_uuid_fkey') THEN
		-- Allow esx_uuid to refer to vCenter servers
		ALTER TABLE bp.vmware_schedules DROP CONSTRAINT vmware_schedules_esx_uuid_fkey;
--		UPDATE bp.vmware_schedules SET esx_uuid =
--		 (SELECT COALESCE(vcenter_uuid, esx_uuid)
--		 FROM vmware_esx_servers LEFT JOIN vmware_vcenter_assoc USING (esx_uuid)
--		 WHERE esx_uuid = vmware_schedules.esx_uuid);

		ALTER TABLE application_schedules DISABLE TRIGGER ALL;
		ALTER TABLE vmware_schedules DISABLE TRIGGER ALL;
		-- Make schedules reference vCenter servers
		UPDATE bp.vmware_schedules
			SET esx_uuid = COALESCE(vva.vcenter_uuid, vmware_schedules.esx_uuid)
			FROM vmware_vcenter_assoc vva
			WHERE vva.esx_uuid = vmware_schedules.esx_uuid;
		-- Make application_instances.key1 be the vCenter name
		UPDATE bp.application_instances SET key1 = vcenter.name
			FROM bp.vmware_vms vm, bp.vmware_vcenter_assoc assoc,
			  bp.vmware_vcenter_servers vcenter
			WHERE application_instances.instance_id = vm.instance_id
			  AND vm.esx_uuid = assoc.esx_uuid
			  AND assoc.vcenter_uuid = vcenter.vcenter_uuid;
		-- Turn off policy flag for all but one schedule per vCenter server
		UPDATE bp.application_schedules
			SET add_new_instances = FALSE
			WHERE add_new_instances = TRUE
			AND application_schedule_id IN
			 (SELECT application_schedule_id
			  FROM bp.application_schedules
			  JOIN bp.vmware_schedules USING (application_schedule_id))
			AND application_schedule_id NOT IN
			 (SELECT MIN(application_schedule_id)
			  FROM bp.application_schedules
			  JOIN bp.vmware_schedules USING (application_schedule_id)
			  GROUP BY esx_uuid);
		ALTER TABLE application_schedules ENABLE TRIGGER ALL;
		ALTER TABLE vmware_schedules ENABLE TRIGGER ALL;
	END IF;

	RETURN 0;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT update_bp();
DROP FUNCTION update_bp();


--
-- Create view of restorable backups
--
CREATE OR REPLACE VIEW bp.restorable_backups AS
	SELECT * FROM backups
	  WHERE (properties & 1) = 0  -- not synthetic
	  AND tape_no != 4294967295   -- not logically deleted
	  AND (status & 64) > 0;      -- complete

--
-- Create view of successful backups (used for purging, etc)
--
CREATE OR REPLACE VIEW bp.successful_backups AS
	SELECT * FROM backups
	  WHERE (properties & 1) = 0  -- not synthetic
	  AND tape_no != 4294967295   -- not logically deleted
	  AND (status & 64) > 0       -- complete
	  AND (status & 36888) = 0;   -- not failed/cancelled/proc_died


--
-- Drop functions used in table modification
--
DROP FUNCTION public.index_exists(sch_name VARCHAR,idx_name VARCHAR);
DROP FUNCTION public.column_exists(sch_name VARCHAR,tbl_name VARCHAR,col_name VARCHAR);


